# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AssetPerformanceReportColumn(str, Enum):
    TIMEPERIOD = 'TimePeriod'

    ACCOUNTID = 'AccountId'

    ACCOUNTNAME = 'AccountName'

    CAMPAIGNID = 'CampaignId'

    CAMPAIGNNAME = 'CampaignName'

    ADGROUPID = 'AdGroupId'

    ADGROUPNAME = 'AdGroupName'

    ASSETID = 'AssetId'

    ASSETCONTENT = 'AssetContent'

    ASSETTYPE = 'AssetType'

    ASSETSOURCE = 'AssetSource'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    SPEND = 'Spend'

    VIDEOVIEWS = 'VideoViews'

    VIDEOVIEWSAT25PERCENT = 'VideoViewsAt25Percent'

    VIDEOVIEWSAT50PERCENT = 'VideoViewsAt50Percent'

    VIDEOVIEWSAT75PERCENT = 'VideoViewsAt75Percent'

    COMPLETEDVIDEOVIEWS = 'CompletedVideoViews'

    VIDEOCOMPLETIONRATE = 'VideoCompletionRate'

    CONVERSIONS = 'Conversions'

    REVENUE = 'Revenue'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AssetPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))


