# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class Industry(str, Enum):
    NA = 'NA'

    AGENCYSALESHOUSE = 'AgencySalesHouse'

    AUTOMOTIVE = 'Automotive'

    CONSUMERPACKAGEDGOODS = 'ConsumerPackagedGoods'

    EDUCATION = 'Education'

    ENTERTAINMENT = 'Entertainment'

    FINANCIALSERVICES = 'FinancialServices'

    FOODSERVICES = 'FoodServices'

    GAMING = 'Gaming'

    GOVERNMENTNONPROFITPOLITICAL = 'GovernmentNonprofitPolitical'

    HEALTHCARE = 'Healthcare'

    INTERNAL = 'Internal'

    PUBLISHINGANDWEBMEDIA = 'PublishingAndWebMedia'

    REALESTATE = 'RealEstate'

    RETAIL = 'Retail'

    SERVICES = 'Services'

    TECHNOLOGY = 'Technology'

    TELECOMMUNICATIONS = 'Telecommunications'

    TRAVELHOSPITALITY = 'TravelHospitality'

    OTHER = 'Other'

    PHARMACEUTICALS = 'Pharmaceuticals'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Industry from a JSON string"""
        return cls(json.loads(json_str))


