# coding: utf-8

# flake8: noqa
"""
    BulkService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.bulk.ad_api_error import AdApiError
from openapi_client.models.bulk.ad_api_fault_detail import AdApiFaultDetail
from openapi_client.models.bulk.ad_center_authentication_fault import AdCenterAuthenticationFault
from openapi_client.models.bulk.ad_center_authorization_fault import AdCenterAuthorizationFault
from openapi_client.models.bulk.ad_center_fault import AdCenterFault
from openapi_client.models.bulk.ad_center_internal_error_fault import AdCenterInternalErrorFault
from openapi_client.models.bulk.ad_center_system_fault import AdCenterSystemFault
from openapi_client.models.bulk.api_fault_detail import ApiFaultDetail
from openapi_client.models.bulk.application_fault import ApplicationFault
from openapi_client.models.bulk.authentication_fault_code import AuthenticationFaultCode
from openapi_client.models.bulk.batch_error import BatchError
from openapi_client.models.bulk.batch_error_base import BatchErrorBase
from openapi_client.models.bulk.campaign_scope import CampaignScope
from openapi_client.models.bulk.compression_type import CompressionType
from openapi_client.models.bulk.data_scope import DataScope
from openapi_client.models.bulk.download_campaigns_by_account_ids_request import DownloadCampaignsByAccountIdsRequest
from openapi_client.models.bulk.download_campaigns_by_account_ids_response import DownloadCampaignsByAccountIdsResponse
from openapi_client.models.bulk.download_campaigns_by_campaign_ids_request import DownloadCampaignsByCampaignIdsRequest
from openapi_client.models.bulk.download_campaigns_by_campaign_ids_response import DownloadCampaignsByCampaignIdsResponse
from openapi_client.models.bulk.download_entity import DownloadEntity
from openapi_client.models.bulk.download_file_type import DownloadFileType
from openapi_client.models.bulk.editorial_api_fault_detail import EditorialApiFaultDetail
from openapi_client.models.bulk.editorial_error import EditorialError
from openapi_client.models.bulk.fault_source import FaultSource
from openapi_client.models.bulk.get_bulk_download_status_request import GetBulkDownloadStatusRequest
from openapi_client.models.bulk.get_bulk_download_status_response import GetBulkDownloadStatusResponse
from openapi_client.models.bulk.get_bulk_upload_status_request import GetBulkUploadStatusRequest
from openapi_client.models.bulk.get_bulk_upload_status_response import GetBulkUploadStatusResponse
from openapi_client.models.bulk.get_bulk_upload_url_request import GetBulkUploadUrlRequest
from openapi_client.models.bulk.get_bulk_upload_url_response import GetBulkUploadUrlResponse
from openapi_client.models.bulk.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.bulk.operation_error import OperationError
from openapi_client.models.bulk.response_mode import ResponseMode
from openapi_client.models.bulk.upload_entity_records_request import UploadEntityRecordsRequest
from openapi_client.models.bulk.upload_entity_records_response import UploadEntityRecordsResponse
