class _StringTable:
    # CSV Header Strings
    ClientId = "Client Id"
    Type = "Type"
    Status = "Status"
    Campaign = "Campaign"
    Id = "Id"
    BusinessId = "Business Id"
    ParentId = "Parent Id"
    CampaignId = "Campaign Id"
    TimeZone = "Time Zone"
    Budget = "Budget"
    BudgetType = "Budget Type"
    BudgetName = "Budget Name"
    Experiment = "Experiment"
    TrafficSplitPercent = "Traffic Split Percent"
    BaseCampaignId = "Base Campaign Id"
    ExperimentCampaignId = "Experiment Campaign Id"
    ExperimentId = "Experiment Id"
    FeedId = "Feed Id"
    ExperimentType = "Experiment Type"
    BudgetId = "Budget Id"
    AdGroup = "Ad Group"
    AdGroupType = "Ad Group Type"
    HotelAdGroupType = "Hotel Ad Group Type"
    CommissionRate = "Commission Rate"
    PercentCpcBid = "Percent Cpc Bid"
    FinalUrlExpansionOptOut = "Url Expansion Opt Out"
    HotelListingGroupType = "Ad Group Hotel Listing Group"
    HotelAttribute = "Hotel Attribute"
    HotelAttributeValue = "Hotel Attribute Value"
    Keyword = "Keyword"
    TextAd = "Text Ad"
    ProductAd = "Product Ad"
    AppInstallAd = "App Install Ad"
    DynamicSearchAd = "Dynamic Search Ad"
    ExpandedTextAd = "Expanded Text Ad"
    Title = "Title"
    EditorialStatus = "Editorial Status"
    EditorialAppealStatus = "Editorial Appeal Status"
    Error = "Error"
    ErrorNumber = "Error Number"
    EditorialLocation = "Editorial Location"
    EditorialTerm = "Editorial Term"
    EditorialReasonCode = "Editorial Reason Code"
    MigrationErrors = "Migration Errors"
    DisplayUrl = "Display Url"
    DestinationUrl = "Destination Url"
    BusinessName = "Business Name"
    PhoneNumber = "Phone Number"
    PromotionalText = "Promotion"
    MatchType = "Match Type"
    Param1 = "Param1"
    Param2 = "Param2"
    Param3 = "Param3"
    DevicePreference = "Device Preference"
    Details = "Verified Tracking Setting"
    CampaignNegativeKeyword = "Campaign Negative Keyword"
    CampaignNegativeSite = "Campaign Negative Site"
    AdGroupNegativeKeyword = "Ad Group Negative Keyword"
    KeywordNegativeKeyword = "Keyword Negative Keyword"
    AdGroupNegativeSite = "Ad Group Negative Site"
    Text = "Text"
    TextPart2="Text Part 2"
    Website = "Website"
    Target = "Target"
    Feed = "Feed"
    FeedItem = "Feed Item"
    FeedName = "Feed Name"
    TargetAdGroupId = 'Target Ad Group Id'
    TargetCampaignId = 'Target Campaign Id'
    CustomAttributes = "Custom Attributes"
    PageFeedIds = "Page Feed Ids"
    PhysicalIntent = "Physical Intent"
    Bid = "Bid"
    Profile = "Profile"
    ProfileId = "Profile Id"
    BidAdjustment = "Bid Adjustment"
    CashbackAdjustment = "Cashback Percent"
    SubType = "Sub Type"
    OsNames = "OS Names"
    StartDate = "Start Date"
    EndDate = "End Date"
    NetworkDistribution = "Network Distribution"
    Language = "Language"
    CpcBid = "Cpc Bid"
    CpvBid = "Cpv Bid"
    CpmBid = "Cpm Bid"
    AdRotation = "Ad Rotation"
    PrivacyStatus = "Privacy Status"
    Account = "Account"
    SyncTime = "Sync Time"
    Name = "Name"
    MSCLKIDAutoTaggingEnabled = "MSCLKID Auto Tagging Enabled"
    LastModifiedTime = "Modified Time"
    AdFormatPreference = "Ad Format Preference"
    IncludeViewThroughConversions = "Include View Through Conversions"
    ProfileExpansionEnabled = "Profile Expansion Enabled"
    AdClickParallelTracking = "Ad Click Parallel Tracking"
    AutoApplyRecommendations = "Auto Apply Recommendations"
    AllowImageAutoRetrieve = "Allow Image Auto Retrieve"
    MultiMediaAdBidAdjustment = "Multi Media Ad Bid Adjustment"
    UseOptimizedTargeting = "Use Optimized Targeting"
    BusinessAttributes = "Business Attributes"
    Schedule = "Schedule"

    # Entity Types
    SemanticVersion = "Format Version"
    LocationTarget = "Location Target"
    RadiusTarget = "Radius Target"
    BusinessTarget = "Business Location Target"
    RadiusTargetId = "Radius Target Id"
    NegativeLocationTarget = "Negative Location Target"
    AgeTarget = "Age Target"
    GenderTarget = "Gender Target"
    DeviceOsTarget = "DeviceOS Target"
    Radius = "Radius"
    Unit = "Unit"
    TargetAll = "Target All"
    LocationAdExtension = "Location Ad Extension"
    CallAdExtension = "Call Ad Extension"
    ImageAdExtension = "Image Ad Extension"
    SocialNetworkAdExtension = "Social Network Ad Extension"
    Version = "Version"

    # Disclaimer Ads
    DisclaimerAdsEnabled = "Disclaimer Ads Enabled"
    DisclaimerName = "Disclaimer Name"
    DisclaimerTitle = "Disclaimer Title"
    DisclaimerLayout = "Disclaimer Layout"
    DisclaimerPopupText = "Disclaimer Popup Text"
    DisclaimerLineText = "Disclaimer Line Text"

    DisclaimerAdExtension = "Disclaimer Ad Extension"
    CampaignDisclaimerAdExtension = "Campaign Disclaimer Ad Extension"

    # Image Ad Extension
    AltText = "Alternative Text"
    MediaIds = "Media Ids"
    AccountImageAdExtension = "Account Image Ad Extension"
    CampaignImageAdExtension = "Campaign Image Ad Extension"
    AdGroupImageAdExtension = "Ad Group Image Ad Extension"
    Layouts = "Layouts"
    DisplayText = "Display Text"

    # Filter Link ad extension
    FilterLinkAdExtension = "Filter Link Ad Extension"
    AccountFilterLinkAdExtension = "Account Filter Link Ad Extension"
    CampaignFilterLinkAdExtension = "Campaign Filter Link Ad Extension"
    AdGroupFilterLinkAdExtension = "Ad Group Filter Link Ad Extension"
    AdExtensionHeaderType = "AdExtension Header Type"
    Texts = "Texts"

    SiteLinkExtensionOrder = "Sitelink Extension Order"
    SiteLinkDisplayText = "Sitelink Extension Link Text"
    SiteLinkDestinationUrl = "Sitelink Extension Destination Url"
    SiteLinkDescription1 = "Sitelink Extension Description1"
    SiteLinkDescription2 = "Sitelink Extension Description2"

    # Location Ad Extensions
    AddressLine1 = "Address Line 1"
    AddressLine2 = "Address Line 2"
    PostalCode = "Postal Code"
    City = "City"
    StateOrProvince = "State Or Province Code"
    ProvinceName = "Province Name"
    GeoCodeStatus = "Geo Code Status"
    ImageMediaId = "Business Icon"
    AccountLocationAdExtension = "Account Location Ad Extension"
    CampaignLocationAdExtension = "Campaign Location Ad Extension"

    # Call Ad Extensions
    CountryCode = "Country Code"
    IsCallOnly = "Call Only"
    CampaignCallAdExtension = "Campaign Call Ad Extension"
    IsCallTrackingEnabled = "Call Tracking Enabled"
    RequireTollFreeTrackingNumber = "Toll Free"

    #Action Ad Extension
    ActionAdExtension = "Action Ad Extension"
    AccountActionAdExtension = "Account Action Ad Extension"
    CampaignActionAdExtension = "Campaign Action Ad Extension"
    AdGroupActionAdExtension = "Ad Group Action Ad Extension"
    ActionType = "Action Type"
    ActionText = "Action Text"

    #Video Ad Extension
    VideoAdExtension = "Video Ad Extension"
    ThumbnailUrl = "Thumbnail Url"
    ThumbnailId = "Thumbnail Id"
    VideoId = "Video Id"
    AccountVideoAdExtension = "Account Video Ad Extension"
    CampaignVideoAdExtension = "Campaign Video Ad Extension"
    AdGroupVideoAdExtension = "Ad Group Video Ad Extension"

    #Promotion AdExtension
    PromotionAdExtension = "Promotion Ad Extension"
    AccountPromotionAdExtension = "Account Promotion Ad Extension"
    CampaignPromotionAdExtension = "Campaign Promotion Ad Extension"
    AdGroupPromotionAdExtension = "Ad Group Promotion Ad Extension"
    PromotionTarget = "Promotion Target"
    DiscountModifier = "Discount Modifier"
    PercentOff = "Percent Off"
    MoneyAmountOff = "Money Amount Off"
    PromotionCode = "Promotion Code"
    OrdersOverAmount = "Orders Over Amount"
    Occasion = "Occasion"
    PromotionStart = "Promotion Start"
    PromotionEnd = "Promotion End"
    CurrencyCode = "Currency Code"

    # Structured Snippet Ad Extension
    StructuredSnippetAdExtension = "Structured Snippet Ad Extension"
    AccountStructuredSnippetAdExtension = "Account Structured Snippet Ad Extension"
    CampaignStructuredSnippetAdExtension = "Campaign Structured Snippet Ad Extension"
    AdGroupStructuredSnippetAdExtension = "Ad Group Structured Snippet Ad Extension"
    StructuredSnippetHeader = "Structured Snippet Header"
    StructuredSnippetValues = "Structured Snippet Values"

    # Sitelink Ad Extension
    SitelinkAdExtension = "Sitelink Ad Extension"
    AdGroupSitelinkAdExtension = "Ad Group Sitelink Ad Extension"
    CampaignSitelinkAdExtension = "Campaign Sitelink Ad Extension"
    AccountSitelinkAdExtension = "Account Sitelink Ad Extension"


    # Flyer Ad Extension
    FlyerAdExtension = "Flyer Ad Extension"
    AccountFlyerAdExtension = "Account Flyer Ad Extension"
    CampaignFlyerAdExtension = "Campaign Flyer Ad Extension"
    AdGroupFlyerAdExtension = "Ad Group Flyer Ad Extension"
    FlyerName = "Flyer Name"
    MediaUrls = "Media Urls"


    # Editorial Rejection Reasons
    PublisherCountries = "Publisher Countries"

    # BTE Types
    KeywordFirstPageBidType = "Keyword First Page Bid"
    KeywordMainLineBidType = "Keyword Main Line Bid"
    KeywordBestPositionBidType = "Keyword Best Position Bid"

    ProductCondition1 = "Product Condition 1"
    ProductCondition2 = "Product Condition 2"
    ProductCondition3 = "Product Condition 3"
    ProductCondition4 = "Product Condition 4"
    ProductCondition5 = "Product Condition 5"
    ProductCondition6 = "Product Condition 6"
    ProductCondition7 = "Product Condition 7"
    ProductCondition8 = "Product Condition 8"
    ProductValue1 = "Product Value 1"
    ProductValue2 = "Product Value 2"
    ProductValue3 = "Product Value 3"
    ProductValue4 = "Product Value 4"
    ProductValue5 = "Product Value 5"
    ProductValue6 = "Product Value 6"
    ProductValue7 = "Product Value 7"
    ProductValue8 = "Product Value 8"

    MerchantCenterId = "Store Id"
    MerchantCenterName = "Store Name"

    # App Ad Extension
    AppAdExtension = "App Ad Extension"
    AppPlatform = "App Platform"
    AppStoreId = "App Id"
    AccountAppAdExtension = "Account App Ad Extension"
    CampaignAppAdExtension = "Campaign App Ad Extension"
    AdGroupAppAdExtension = "Ad Group App Ad Extension"
    IsTrackingEnabled = "Tracking Enabled"

    # Misc Stuff
    EntityActiveStatus = "Active"
    EntityDeletedStatus = "Deleted"
    IntraFieldSeparator = ';'
    Active = "Active"
    Latitude = "Latitude"
    Longitude = "Longitude"

    # BI
    Spend = "Spend"
    Impressions = "Impressions"
    Clicks = "Clicks"
    CTR = "CTR"
    AvgCPC = "Avg CPC"
    AvgCPM = "Avg CPM"
    AvgPosition = "Avg position"
    Conversions = "Conversions"
    CPA = "CPA"
    QualityScore = "Quality Score"
    KeywordRelevance = "Keyword Relevance"
    LandingPageRelevance = "Landing Page Relevance"
    LandingPageUserExperience = "Landing Page User Experience"

    # DayTime Criterion
    FromHour = "From Hour"
    ToHour = "To Hour"
    FromMinute = "From Minute"
    ToMinute = "To Minute"

    # Shared Entities
    NegativeKeywordList = "Negative Keyword List"
    CampaignNegativeKeywordList = "Campaign Negative Keyword List Association"
    ListNegativeKeyword = "Shared Negative Keyword"

    # Subtypes
    MetroAreaSubType = "Metro Area"
    CountrySubType = "Country"
    StateSubType = "State"
    CitySubType = "City"
    PostalCodeSubType = "Postal Code"

    # Bing Shopping Campaigns
    IsExcluded = "Is Excluded"
    ParentAdGroupCriterionId = "Parent Criterion Id"
    CampaignType = "Campaign Type"
    CampaignPriority = "Priority"
    LocalInventoryAdsEnabled = "LocalInventoryAdsEnabled"
    DynamicDescriptionEnabled = "Dynamic Description Enabled"

    CampaignNegativeStoreCriterion = "Campaign Negative Store Criterion"


    # CoOp
    BidOption = "Bid Option"
    BidBoostValue = "Bid Boost Value"
    MaximumBid = "Maximum Bid"

    # V10 added
    FieldPath = "Field Path"

    # Upgrade URL
    FinalUrl = "Final Url"
    FinalMobileUrl = "Mobile Final Url"
    TrackingTemplate = "Tracking Template"
    CustomParameter = "Custom Parameter"

    # Review Ad Extension
    ReviewAdExtension = "Review Ad Extension"
    AccountReviewAdExtension = "Account Review Ad Extension"
    CampaignReviewAdExtension = "Campaign Review Ad Extension"
    AdGroupReviewAdExtension = "Ad Group Review Ad Extension"
    IsExact = "Is Exact"
    Source = "Source"
    Url = "Url"

    # Price Ad Extension
    PriceAdExtension = "Price Ad Extension"
    AccountPriceAdExtension = "Account Price Ad Extension"
    CampaignPriceAdExtension = "Campaign Price Ad Extension"
    AdGroupPriceAdExtension = "Ad Group Price Ad Extension"
    PriceExtensionType = "Price Extension Type"
    Header1 = "Header 1"
    Header2 = "Header 2"
    Header3 = "Header 3"
    Header4 = "Header 4"
    Header5 = "Header 5"
    Header6 = "Header 6"
    Header7 = "Header 7"
    Header8 = "Header 8"
    PriceDescription1 = "Price Description 1"
    PriceDescription2 = "Price Description 2"
    PriceDescription3 = "Price Description 3"
    PriceDescription4 = "Price Description 4"
    PriceDescription5 = "Price Description 5"
    PriceDescription6 = "Price Description 6"
    PriceDescription7 = "Price Description 7"
    PriceDescription8 = "Price Description 8"
    FinalUrl1 = "Final Url 1"
    FinalUrl2 = "Final Url 2"
    FinalUrl3 = "Final Url 3"
    FinalUrl4 = "Final Url 4"
    FinalUrl5 = "Final Url 5"
    FinalUrl6 = "Final Url 6"
    FinalUrl7 = "Final Url 7"
    FinalUrl8 = "Final Url 8"
    FinalMobileUrl1 = "Final Mobile Url 1"
    FinalMobileUrl2 = "Final Mobile Url 2"
    FinalMobileUrl3 = "Final Mobile Url 3"
    FinalMobileUrl4 = "Final Mobile Url 4"
    FinalMobileUrl5 = "Final Mobile Url 5"
    FinalMobileUrl6 = "Final Mobile Url 6"
    FinalMobileUrl7 = "Final Mobile Url 7"
    FinalMobileUrl8 = "Final Mobile Url 8"
    Price1 = "Price 1"
    Price2 = "Price 2"
    Price3 = "Price 3"
    Price4 = "Price 4"
    Price5 = "Price 5"
    Price6 = "Price 6"
    Price7 = "Price 7"
    Price8 = "Price 8"
    CurrencyCode1 = "Currency Code 1"
    CurrencyCode2 = "Currency Code 2"
    CurrencyCode3 = "Currency Code 3"
    CurrencyCode4 = "Currency Code 4"
    CurrencyCode5 = "Currency Code 5"
    CurrencyCode6 = "Currency Code 6"
    CurrencyCode7 = "Currency Code 7"
    CurrencyCode8 = "Currency Code 8"
    PriceUnit1 = "Price Unit 1"
    PriceUnit2 = "Price Unit 2"
    PriceUnit3 = "Price Unit 3"
    PriceUnit4 = "Price Unit 4"
    PriceUnit5 = "Price Unit 5"
    PriceUnit6 = "Price Unit 6"
    PriceUnit7 = "Price Unit 7"
    PriceUnit8 = "Price Unit 8"
    PriceQualifier1 = "Price Qualifier 1"
    PriceQualifier2 = "Price Qualifier 2"
    PriceQualifier3 = "Price Qualifier 3"
    PriceQualifier4 = "Price Qualifier 4"
    PriceQualifier5 = "Price Qualifier 5"
    PriceQualifier6 = "Price Qualifier 6"
    PriceQualifier7 = "Price Qualifier 7"
    PriceQualifier8 = "Price Qualifier 8"

    # Callout Ad Extension
    CalloutAdExtension = "Callout Ad Extension"
    AccountCalloutAdExtension = "Account Callout Ad Extension"
    CampaignCalloutAdExtension = "Campaign Callout Ad Extension"
    AdGroupCalloutAdExtension = "Ad Group Callout Ad Extension"
    CalloutText = "Callout Text"

    # Bid Strategy
    BidStrategy = "Bid Strategy"
    BidStrategyId = "Bid Strategy Id"
    BidStrategyName = "Bid Strategy Name"
    BidStrategyType = "Bid Strategy Type"
    BidStrategyMaxCpc = "Bid Strategy MaxCpc"
    BidStrategyTargetCpa = "Bid Strategy TargetCpa"
    BidStrategyTargetRoas = "Bid Strategy TargetRoas"
    InheritedBidStrategyType = "Inherited Bid Strategy Type"
    BidStrategyTargetAdPosition = "Bid Strategy TargetAdPosition"
    BidStrategyTargetImpressionShare = "Bid Strategy TargetImpressionShare"
    BidStrategyPercentMaxCpc = "Bid Strategy PercentMaxCpc"
    BidStrategyCommissionRate = "Bid Strategy CommissionRate"
    BidStrategyTargetCostPerSale = "Bid Strategy TargetCostPerSale"

    # Remarketing
    Audience = "Audience"
    Description = "Description"
    MembershipDuration = "Membership Duration"
    Scope = "Scope"
    TagId = "UET Tag Id"
    AudienceId = "Audience Id"
    TargetSetting = "Target Setting"
    RemarketingRule = "Remarketing Rule"
    AudienceSearchSize = "Audience Search Size"
    ProductAudience = "Product Audience"
    AudienceNetworkSize = "Audience Network Size"
    SupportedCampaignTypes = "Supported Campaign Types"
    ProductAudienceType = "Product Audience Type"
    CombinationRule = "Combination Rule"
    SourceId = "Source Id"
    AdGroupProductAudienceAssociation = "Ad Group Product Audience Association"
    AdGroupNegativeProductAudienceAssociation = "Ad Group Negative Product Audience Association"
    CampaignProductAudienceAssociation = "Campaign Product Audience Association"
    CampaignNegativeProductAudienceAssociation = "Campaign Negative Product Audience Association"
    SimilarRemarketingList = "Similar Remarketing List"
    AdGroupSimilarRemarketingListAssociation = "Ad Group Similar Remarketing List Association"
    AdGroupNegativeSimilarRemarketingListAssociation = "Ad Group Negative Similar Remarketing List Association"
    CampaignSimilarRemarketingListAssociation = "Campaign Similar Remarketing List Association"
    CampaignNegativeSimilarRemarketingListAssociation = "Campaign Negative Similar Remarketing List Association"
    RemarketingList = "Remarketing List"
    AdGroupRemarketingListAssociation = 'Ad Group Remarketing List Association'
    AdGroupNegativeRemarketingListAssociation = 'Ad Group Negative Remarketing List Association'
    CampaignRemarketingListAssociation = 'Campaign Remarketing List Association'
    CampaignNegativeRemarketingListAssociation = 'Campaign Negative Remarketing List Association'
    CustomAudience = "Custom Audience"
    AdGroupCustomAudienceAssociation = "Ad Group Custom Audience Association"
    AdGroupNegativeCustomAudienceAssociation = "Ad Group Negative Custom Audience Association"
    CampaignCustomAudienceAssociation = "Campaign Custom Audience Association"
    CampaignNegativeCustomAudienceAssociation = "Campaign Negative Custom Audience Association"
    InMarketAudience = "In Market Audience"
    AdGroupInMarketAudienceAssociation = "Ad Group In Market Audience Association"
    AdGroupNegativeInMarketAudienceAssociation = "Ad Group Negative In Market Audience Association"
    CampaignInMarketAudienceAssociation = "Campaign In Market Audience Association"
    CampaignNegativeInMarketAudienceAssociation = "Campaign Negative In Market Audience Association"
    CombinedList = "Combined List"
    AdGroupCombinedListAssociation = "Ad Group Combined List Association"
    AdGroupNegativeCombinedListAssociation = "Ad Group Negative Combined List Association"
    CampaignCombinedListAssociation = "Campaign Combined List Association"
    CampaignNegativeCombinedListAssociation = "Campaign Negative Combined List Association"
    CustomerList = "Customer List"
    CustomerListItem = "Customer List Item"
    AdGroupCustomerListAssociation = "Ad Group Customer List Association"
    AdGroupNegativeCustomerListAssociation = "Ad Group Negative Customer List Association"
    CampaignCustomerListAssociation = "Campaign Customer List Association"
    CampaignNegativeCustomerListAssociation = "Campaign Negative Customer List Association"
    ImpressionBasedRemarketingList = "Impression Based Remarketing List"
    AdGroupImpressionBasedRemarketingListAssociation = "Ad Group Impression Based Remarketing List Association"
    AdGroupNegativeImpressionBasedRemarketingListAssociation = "Ad Group Negative Impression Based Remarketing List Association"
    CampaignImpressionBasedRemarketingListAssociation = "Campaign Impression Based Remarketing List Association"
    CampaignNegativeImpressionBasedRemarketingListAssociation = "Campaign Negative Impression Based Remarketing List Association"
    EntityType = "Entity Type"
    ImpressionCampaignId = "Impression Campaign Id"
    ImpressionAdGroupId = "Impression Ad Group Id"

    # Expanded Text Ad
    TitlePart1 = "Title Part 1"
    TitlePart2 = "Title Part 2"
    TitlePart3 = "Title Part 3"
    Path1 = "Path 1"
    Path2 = "Path 2"
    Domain = "Domain"

    # Ad Extension Scheduling
    AdSchedule = "Ad Schedule"

    #UseSearcherTimeZone
    UseSearcherTimeZone = "Use Searcher Time Zone"
    AdScheduleUseSearcherTimeZone = "Ad Schedule Use Searcher Time Zone"

    # Dynamic Search Ads
    DomainLanguage = "Domain Language"
    DynamicAdTargetCondition1 = "Dynamic Ad Target Condition 1"
    DynamicAdTargetValue1 = "Dynamic Ad Target Value 1"
    DynamicAdTargetCondition2 = "Dynamic Ad Target Condition 2"
    DynamicAdTargetValue2 = "Dynamic Ad Target Value 2"
    DynamicAdTargetCondition3 = "Dynamic Ad Target Condition 3"
    DynamicAdTargetValue3 = "Dynamic Ad Target Value 3"
    DynamicAdTargetConditionOperator1 = "Dynamic Ad Target Condition Operator 1"
    DynamicAdTargetConditionOperator2 = "Dynamic Ad Target Condition Operator 2"
    DynamicAdTargetConditionOperator3 = "Dynamic Ad Target Condition Operator 3"

    AdGroupAgeCriterion = "Ad Group Age Criterion"
    AdGroupDayTimeCriterion = "Ad Group DayTime Criterion"
    AdGroupDeviceCriterion = "Ad Group DeviceOS Criterion"
    AdGroupGenderCriterion = "Ad Group Gender Criterion"
    AdGroupRadiusCriterion = "Ad Group Radius Criterion"
    AdGroupLocationCriterion = "Ad Group Location Criterion"
    AdGroupLocationIntentCriterion = "Ad Group Location Intent Criterion"
    AdGroupNegativeLocationCriterion = "Ad Group Negative Location Criterion"
    AdGroupCompanyNameCriterion = "Ad Group Company Name Criterion"
    AdGroupIndustryCriterion = "Ad Group Industry Criterion"
    AdGroupJobFunctionCriterion = "Ad Group Job Function Criterion"
    AdGroupNegativeAgeCriterion = "Ad Group Negative Age Criterion"
    AdGroupNegativeCompanyNameCriterion = "Ad Group Negative Company Name Criterion"
    AdGroupNegativeGenderCriterion = "Ad Group Negative Gender Criterion"
    AdGroupNegativeIndustryCriterion = "Ad Group Negative Industry Criterion"
    AdGroupNegativeJobFunctionCriterion = "Ad Group Negative Job Function Criterion"
    AdGroupGenreCriterion = "Ad Group Genre Criterion"

    # Responsive Ad
    ResponsiveAd = "Responsive Ad"
    CallToAction = "Call To Action"
    Headline = "Headline"
    Images = "Images"
    LandscapeImageMediaId = "Landscape Image Media Id"
    LandscapeLogoMediaId = "Landscape Logo Media Id"
    LongHeadline = "Long Headline"
    LongHeadlines = "Long Headlines"
    SquareImageMediaId = "Square Image Media Id"
    SquareLogoMediaId = "Square Logo Media Id"
    ImpressionTrackingUrls = "Impression Tracking Urls"
    CallToActionLanguage = "Call To Action Language"
    Descriptions = "Descriptions"
    Headlines = "Headlines"
    Videos = "Videos"

    #Image
    Image = "Image"
    Height = "Height"
    Width = "Width"

    # Video
    Video = "Video"
    SourceUrl = "Source Url"
    AspectRatio = "Aspect Ratio"
    DurationInMillionSeconds = "Duration In Milliseconds"

    # Responsive Search Ad
    ResponsiveSearchAd = "Responsive Search Ad"

    # Labels
    ColorCode = "Color"
    Label = "Label"
    CampaignLabel = "Campaign Label"
    AdGroupLabel = "Ad Group Label"
    KeywordLabel = "Keyword Label"
    AppInstallAdLabel = "App Install Ad Label"
    DynamicSearchAdLabel = "Dynamic Search Ad Label"
    ExpandedTextAdLabel = "Expanded Text Ad Label"
    ProductAdLabel = "Product Ad Label"
    TextAdLabel = "Text Ad Label"
    ResponsiveAdLabel = "Responsive Ad Label"
    ResponsiveSearchAdLabel = "Responsive Search Ad Label"

    # Offline Conversions
    OfflineConversion = "Offline Conversion"
    ConversionCurrencyCode = "Conversion Currency Code"
    ConversionName = "Conversion Name"
    ConversionTime = "Conversion Time"
    ConversionValue = "Conversion Value"
    MicrosoftClickId = "Microsoft Click Id"
    AdjustmentValue = "Adjustment Value"
    AdjustmentTime = "Adjustment Time"
    AdjustmentCurrencyCode = "Adjustment Currency Code"
    AdjustmentType = "Adjustment Type"
    ExternalAttributionModel = "External Attribution Model"
    ExternalAttributionCredit = "External Attribution Credit"
    HashedEmailAddress = "Hashed Email Address"
    HashedPhoneNumber = "Hashed Phone Number"

    # Campaign Criterion
    CampaignAgeCriterion = 'Campaign Age Criterion'
    CampaignDayTimeCriterion = 'Campaign DayTime Criterion'
    CampaignDeviceOSCriterion = 'Campaign DeviceOS Criterion'
    CampaignGenderCriterion = 'Campaign Gender Criterion'
    CampaignLocationCriterion = 'Campaign Location Criterion'
    CampaignLocationIntentCriterion = 'Campaign Location Intent Criterion'
    CampaignNegativeLocationCriterion = 'Campaign Negative Location Criterion'
    CampaignRadiusCriterion = 'Campaign Radius Criterion'
    CampaignCompanyNameCriterion = 'Campaign Company Name Criterion'
    CampaignJobFunctionCriterion = 'Campaign Job Function Criterion'
    CampaignIndustryCriterion = 'Campaign Industry Criterion'
    CampaignDealCriterion = 'Campaign Deal Criterion'

    # Online Conversion
    OnlineConversionAdjustment = "Online Conversion Adjustment"
    TransactionId = "Transaction Id"

    # Hotel Ad
    AdGroupAdvanceBookingWindowCriterion = "Ad Group Advance Booking Window Criterion"
    AdGroupCheckInDayCriterion = "Ad Group Check In Day Criterion"
    AdGroupLengthOfStayCriterion = "Ad Group Length of Stay Criterion"
    AdGroupHotelDateSelectionTypeCriterion = "Ad Group Hotel Date Selection Type Criterion"
    AdGroupCheckInDateCriterion = "Ad Group Check In Date Criterion"
    MinTargetValue = "Min Target Value"
    MaxTargetValue = "Max Target Value"

    FinalUrlSuffix = "Final Url Suffix"

    # AdCustomizerAttribute
    AdCustomizerAttribute = "Adcustomizer Attribute"
    AdCustomizerCampaign = "Campaign Adcustomizer Attribute"
    AdCustomizerAdGroup = "Adgroup Adcustomizer Attribute"
    AdCustomizerKeyword = "Keyword Adcustomizer Attribute"
    AdCustomizerDataType = "AdCustomizer DataType"
    AdCustomizerAttributeValue = "AdCustomizer AttributeValue"

    # Campaign Conversion Goal
    CampaignConversionGoal = "Campaign Conversion Goal"
    GoalId = "Goal Id"

    # PMax
    AssetGroup = "Asset Group"
    AudienceGroup = "Audience Group"
    CampaignNegativeWebpage = "Campaign Negative Webpage"
    AssetGroupListingGroup = "Asset Group Listing Group"
    AudienceGroupAssetGroupAssociation = "Audience Group Asset Group Association"
    Audiences = "Audiences"
    AudienceGroupName = "Audience Group Name"
    AgeRanges = "Age Ranges"
    GenderTypes = "Gender Types"
    ParentListingGroupId = "Parent Listing Group Id"
    AutoGeneratedTextOptOut = "Auto Generated Text Assets Opt Out"
    AutoGeneratedImageOptOut = "Auto Generated Image Assets Opt Out"
    CostPerSaleOptOut = "Cost Per Sale Opt Out"
    AssetGroupSearchTheme = "Asset Group Search Theme"
    SearchTheme = "Search Theme"

    # MultiChannel Campaign
    DestinationChannel = "Destination Channel"
    IsMultiChannelCampaign = "Is Multi Channel Campaign"

    # Seasonality Adjustment
    SeasonalityAdjustment = "Seasonality Adjustment"
    DataExclusion = "Data Exclusion"
    DeviceType = "Device Type"
    CampaignAssociations = "Campaign Associations"

    # DNV Serving on MSAN
    ShouldServeOnMSAN = "Should Serve On MSAN"

    # Conversion Goal
    AttributionModelType = "Attribution Model Type"
    ConversionWindowInMinutes = "Conversion Window In Minutes"
    CountType = "Count Type"
    ExcludeFromBidding = "Exclude From Bidding"
    GoalCategory = "Goal Category"
    IsEnhancedConversionsEnabled = "Is Enhanced Conversions Enabled"
    RevenueType = "Revenue Type"
    RevenueValue = "Revenue Value"
    TrackingStatus = "Tracking Status"
    ViewThroughConversionWindowInMinutes = "View Through Conversion Window In Minutes"
    MinimumDurationInSecond = "Minimum Duration In Second"
    ActionExpression = "Action Expression"
    ActionOperator = "Action Operator"
    CategoryExpression = "Category Expression"
    CategoryOperator = "Category Operator"
    LabelExpression = "Label Expression"
    LabelOperator = "Label Operator"
    EventValue = "Event Value"
    EventValueOperator = "Event Value Operator"
    IsExternallyAttributed = "Is Externally Attributed"
    MinimumPagesViewed = "Minimum Pages Viewed"
    UrlExpression = "URL Expression"
    UrlOperator = "URL Operator"
    ConversionGoal = "Conversion Goal"
    EventGoal = "Event Goal"
    AppInstallGoal = "AppInstall Goal"
    MultiStageGoal = "MultiStage Goal"
    DurationGoal = "Duration Goal"
    OfflineConversionGoal = "OfflineConversion Goal"
    UrlGoal = "URL Goal"
    InStoreTransactionGoal = "InStoreTransaction Goal"
    PagesViewedPerVisitGoal = "PagesViewedPerVisit Goal"
    SmartGoal = "Smart Goal"
    InStoreVisitGoal = "InStoreVisit Goal"
    ProductGoal = "Product Goal"

    # Brand List
    BrandList = "Brand List"
    BrandItem = "Brand Item"
    CampaignBrandList = "Campaign Brand List Association"
    BrandId = "Brand Id"
    BrandUrl = "Brand Url"
    BrandName = "Brand Name"
    StatusDateTime = "Editorial Status Date"

    # Asset Group Url Target
    AssetGroupUrlTarget = "Asset Group Url Target";
    AssetGroupTargetCondition1 = "Asset Group Target Condition 1";
    AssetGroupTargetCondition2 = "Asset Group Target Condition 2";
    AssetGroupTargetCondition3 = "Asset Group Target Condition 3";
    AssetGroupTargetConditionOperator1 = "Asset Group Target Condition Operator 1";
    AssetGroupTargetConditionOperator2 = "Asset Group Target Condition Operator 2";
    AssetGroupTargetConditionOperator3 = "Asset Group Target Condition Operator 3";
    AssetGroupTargetValue1 = "Asset Group Target Value 1";
    AssetGroupTargetValue2 = "Asset Group Target Value 2";
    AssetGroupTargetValue3 = "Asset Group Target Value 3";

    # New Customer Acquisition Goal
    NewCustomerAcquisitionGoal = "New Customer Acquisition Goal";
    AdditionalConversionValue = "Additional Conversion Value";
    NewCustomerAcquisitionGoalId = "New Customer Acquisition Goal Id";
    NewCustomerAcquisitionBidOnlyMode = "New Customer Acquisition Bid Only Mode";
