__author__ = 'Bing Ads SDK Team'
__email__ = 'bing_ads_sdk@microsoft.com'

from .bulk_ad_group_age_criterion import *
from .bulk_ad_group_biddable_criterion import *
from .bulk_ad_group_day_time_criterion import *
from .bulk_ad_group_device_criterion import *
from .bulk_ad_group_gender_criterion import *
from .bulk_ad_group_location_criterion import *
from .bulk_ad_group_location_intent_criterion import *
from .bulk_ad_group_negative_location_criterion import *
from .bulk_ad_group_radius_criterion import *
from .bulk_ad_group_profile_criterion import *
from .bulk_ad_group_negative_age_criterion import *
from .bulk_ad_group_negative_criterion import *
from .bulk_ad_group_negative_gender_criterion import *
from .bulk_ad_group_negative_profile_criterion import *
from .bulk_campaign_age_criterion import *
from .bulk_campaign_biddable_criterion import *
from .bulk_campaign_day_time_criterion import *
from .bulk_campaign_deal_criterion import *
from .bulk_campaign_device_criterion import *
from .bulk_campaign_gender_criterion import *
from .bulk_campaign_location_criterion import *
from .bulk_campaign_location_intent_criterion import *
from .bulk_campaign_negative_location_criterion import *
from .bulk_campaign_radius_criterion import *
from .bulk_campaign_profile_criterion import *
from .bulk_campaign_negative_criterion import *
from .bulk_ad_group_hotel_advance_booking_window_criterion import *
from .bulk_ad_group_hotel_check_in_date_criterion import *
from .bulk_ad_group_hotel_check_in_day_criterion import *
from .bulk_ad_group_hotel_date_selection_type_criterion import *
from .bulk_ad_group_hotel_length_of_stay_criterion import *
from .bulk_ad_group_genre_criterion import *
