# coding: utf-8

# flake8: noqa
"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.campaign.account_migration_statuses_info import AccountMigrationStatusesInfo
from openapi_client.models.campaign.account_negative_keyword_list import AccountNegativeKeywordList
from openapi_client.models.campaign.account_placement_exclusion_list import AccountPlacementExclusionList
from openapi_client.models.campaign.account_placement_inclusion_list import AccountPlacementInclusionList
from openapi_client.models.campaign.account_property import AccountProperty
from openapi_client.models.campaign.account_property_name import AccountPropertyName
from openapi_client.models.campaign.action_ad_extension import ActionAdExtension
from openapi_client.models.campaign.action_ad_extension_action_type import ActionAdExtensionActionType
from openapi_client.models.campaign.ad import Ad
from openapi_client.models.campaign.ad_additional_field import AdAdditionalField
from openapi_client.models.campaign.ad_api_error import AdApiError
from openapi_client.models.campaign.ad_api_fault_detail import AdApiFaultDetail
from openapi_client.models.campaign.ad_center_authentication_fault import AdCenterAuthenticationFault
from openapi_client.models.campaign.ad_center_authorization_fault import AdCenterAuthorizationFault
from openapi_client.models.campaign.ad_center_fault import AdCenterFault
from openapi_client.models.campaign.ad_center_internal_error_fault import AdCenterInternalErrorFault
from openapi_client.models.campaign.ad_center_system_fault import AdCenterSystemFault
from openapi_client.models.campaign.ad_editorial_status import AdEditorialStatus
from openapi_client.models.campaign.ad_extension import AdExtension
from openapi_client.models.campaign.ad_extension_additional_field import AdExtensionAdditionalField
from openapi_client.models.campaign.ad_extension_association import AdExtensionAssociation
from openapi_client.models.campaign.ad_extension_association_collection import AdExtensionAssociationCollection
from openapi_client.models.campaign.ad_extension_editorial_reason import AdExtensionEditorialReason
from openapi_client.models.campaign.ad_extension_editorial_reason_collection import AdExtensionEditorialReasonCollection
from openapi_client.models.campaign.ad_extension_editorial_status import AdExtensionEditorialStatus
from openapi_client.models.campaign.ad_extension_header_type import AdExtensionHeaderType
from openapi_client.models.campaign.ad_extension_id_to_entity_id_association import AdExtensionIdToEntityIdAssociation
from openapi_client.models.campaign.ad_extension_identity import AdExtensionIdentity
from openapi_client.models.campaign.ad_extension_status import AdExtensionStatus
from openapi_client.models.campaign.ad_extensions_type_filter import AdExtensionsTypeFilter
from openapi_client.models.campaign.ad_group import AdGroup
from openapi_client.models.campaign.ad_group_additional_field import AdGroupAdditionalField
from openapi_client.models.campaign.ad_group_criterion import AdGroupCriterion
from openapi_client.models.campaign.ad_group_criterion_action import AdGroupCriterionAction
from openapi_client.models.campaign.ad_group_criterion_editorial_status import AdGroupCriterionEditorialStatus
from openapi_client.models.campaign.ad_group_criterion_status import AdGroupCriterionStatus
from openapi_client.models.campaign.ad_group_criterion_type import AdGroupCriterionType
from openapi_client.models.campaign.ad_group_negative_sites import AdGroupNegativeSites
from openapi_client.models.campaign.ad_group_privacy_status import AdGroupPrivacyStatus
from openapi_client.models.campaign.ad_group_status import AdGroupStatus
from openapi_client.models.campaign.ad_recommendation_additional_field import AdRecommendationAdditionalField
from openapi_client.models.campaign.ad_recommendation_customized_property import AdRecommendationCustomizedProperty
from openapi_client.models.campaign.ad_recommendation_image_asset_property import AdRecommendationImageAssetProperty
from openapi_client.models.campaign.ad_recommendation_image_field import AdRecommendationImageField
from openapi_client.models.campaign.ad_recommendation_image_refine_operation import AdRecommendationImageRefineOperation
from openapi_client.models.campaign.ad_recommendation_image_refine_type import AdRecommendationImageRefineType
from openapi_client.models.campaign.ad_recommendation_image_suggestion import AdRecommendationImageSuggestion
from openapi_client.models.campaign.ad_recommendation_image_suggestion_metadata import AdRecommendationImageSuggestionMetadata
from openapi_client.models.campaign.ad_recommendation_media_refine_result import AdRecommendationMediaRefineResult
from openapi_client.models.campaign.ad_recommendation_refined_media import AdRecommendationRefinedMedia
from openapi_client.models.campaign.ad_recommendation_text_asset_property import AdRecommendationTextAssetProperty
from openapi_client.models.campaign.ad_recommendation_text_field import AdRecommendationTextField
from openapi_client.models.campaign.ad_recommendation_text_refine_operation import AdRecommendationTextRefineOperation
from openapi_client.models.campaign.ad_recommendation_text_refine_result import AdRecommendationTextRefineResult
from openapi_client.models.campaign.ad_recommendation_text_tone import AdRecommendationTextTone
from openapi_client.models.campaign.ad_recommendation_video_suggestion import AdRecommendationVideoSuggestion
from openapi_client.models.campaign.ad_recommendation_video_type import AdRecommendationVideoType
from openapi_client.models.campaign.ad_rotation import AdRotation
from openapi_client.models.campaign.ad_rotation_type import AdRotationType
from openapi_client.models.campaign.ad_status import AdStatus
from openapi_client.models.campaign.ad_sub_type import AdSubType
from openapi_client.models.campaign.ad_type import AdType
from openapi_client.models.campaign.add_ad_extensions_request import AddAdExtensionsRequest
from openapi_client.models.campaign.add_ad_extensions_response import AddAdExtensionsResponse
from openapi_client.models.campaign.add_ad_group_criterions_request import AddAdGroupCriterionsRequest
from openapi_client.models.campaign.add_ad_group_criterions_response import AddAdGroupCriterionsResponse
from openapi_client.models.campaign.add_ad_groups_request import AddAdGroupsRequest
from openapi_client.models.campaign.add_ad_groups_response import AddAdGroupsResponse
from openapi_client.models.campaign.add_ads_request import AddAdsRequest
from openapi_client.models.campaign.add_ads_response import AddAdsResponse
from openapi_client.models.campaign.add_asset_groups_request import AddAssetGroupsRequest
from openapi_client.models.campaign.add_asset_groups_response import AddAssetGroupsResponse
from openapi_client.models.campaign.add_audience_groups_request import AddAudienceGroupsRequest
from openapi_client.models.campaign.add_audience_groups_response import AddAudienceGroupsResponse
from openapi_client.models.campaign.add_audiences_request import AddAudiencesRequest
from openapi_client.models.campaign.add_audiences_response import AddAudiencesResponse
from openapi_client.models.campaign.add_bid_strategies_request import AddBidStrategiesRequest
from openapi_client.models.campaign.add_bid_strategies_response import AddBidStrategiesResponse
from openapi_client.models.campaign.add_brand_kits_request import AddBrandKitsRequest
from openapi_client.models.campaign.add_brand_kits_response import AddBrandKitsResponse
from openapi_client.models.campaign.add_budgets_request import AddBudgetsRequest
from openapi_client.models.campaign.add_budgets_response import AddBudgetsResponse
from openapi_client.models.campaign.add_campaign_conversion_goals_request import AddCampaignConversionGoalsRequest
from openapi_client.models.campaign.add_campaign_conversion_goals_response import AddCampaignConversionGoalsResponse
from openapi_client.models.campaign.add_campaign_criterions_request import AddCampaignCriterionsRequest
from openapi_client.models.campaign.add_campaign_criterions_response import AddCampaignCriterionsResponse
from openapi_client.models.campaign.add_campaigns_request import AddCampaignsRequest
from openapi_client.models.campaign.add_campaigns_response import AddCampaignsResponse
from openapi_client.models.campaign.add_conversion_goals_request import AddConversionGoalsRequest
from openapi_client.models.campaign.add_conversion_goals_response import AddConversionGoalsResponse
from openapi_client.models.campaign.add_conversion_value_rules_request import AddConversionValueRulesRequest
from openapi_client.models.campaign.add_conversion_value_rules_response import AddConversionValueRulesResponse
from openapi_client.models.campaign.add_data_exclusions_request import AddDataExclusionsRequest
from openapi_client.models.campaign.add_data_exclusions_response import AddDataExclusionsResponse
from openapi_client.models.campaign.add_experiments_request import AddExperimentsRequest
from openapi_client.models.campaign.add_experiments_response import AddExperimentsResponse
from openapi_client.models.campaign.add_import_jobs_request import AddImportJobsRequest
from openapi_client.models.campaign.add_import_jobs_response import AddImportJobsResponse
from openapi_client.models.campaign.add_keywords_request import AddKeywordsRequest
from openapi_client.models.campaign.add_keywords_response import AddKeywordsResponse
from openapi_client.models.campaign.add_labels_request import AddLabelsRequest
from openapi_client.models.campaign.add_labels_response import AddLabelsResponse
from openapi_client.models.campaign.add_list_items_to_shared_list_request import AddListItemsToSharedListRequest
from openapi_client.models.campaign.add_list_items_to_shared_list_response import AddListItemsToSharedListResponse
from openapi_client.models.campaign.add_media_request import AddMediaRequest
from openapi_client.models.campaign.add_media_response import AddMediaResponse
from openapi_client.models.campaign.add_negative_keywords_to_entities_request import AddNegativeKeywordsToEntitiesRequest
from openapi_client.models.campaign.add_negative_keywords_to_entities_response import AddNegativeKeywordsToEntitiesResponse
from openapi_client.models.campaign.add_new_customer_acquisition_goals_request import AddNewCustomerAcquisitionGoalsRequest
from openapi_client.models.campaign.add_new_customer_acquisition_goals_response import AddNewCustomerAcquisitionGoalsResponse
from openapi_client.models.campaign.add_seasonality_adjustments_request import AddSeasonalityAdjustmentsRequest
from openapi_client.models.campaign.add_seasonality_adjustments_response import AddSeasonalityAdjustmentsResponse
from openapi_client.models.campaign.add_shared_entity_request import AddSharedEntityRequest
from openapi_client.models.campaign.add_shared_entity_response import AddSharedEntityResponse
from openapi_client.models.campaign.add_uet_tags_request import AddUetTagsRequest
from openapi_client.models.campaign.add_uet_tags_response import AddUetTagsResponse
from openapi_client.models.campaign.add_videos_request import AddVideosRequest
from openapi_client.models.campaign.add_videos_response import AddVideosResponse
from openapi_client.models.campaign.address import Address
from openapi_client.models.campaign.age_criterion import AgeCriterion
from openapi_client.models.campaign.age_dimension import AgeDimension
from openapi_client.models.campaign.age_range import AgeRange
from openapi_client.models.campaign.annotation_opt_out import AnnotationOptOut
from openapi_client.models.campaign.api_fault_detail import ApiFaultDetail
from openapi_client.models.campaign.app_ad_extension import AppAdExtension
from openapi_client.models.campaign.app_install_ad import AppInstallAd
from openapi_client.models.campaign.app_install_goal import AppInstallGoal
from openapi_client.models.campaign.app_setting import AppSetting
from openapi_client.models.campaign.app_store import AppStore
from openapi_client.models.campaign.app_url import AppUrl
from openapi_client.models.campaign.appeal_editorial_rejections_request import AppealEditorialRejectionsRequest
from openapi_client.models.campaign.appeal_editorial_rejections_response import AppealEditorialRejectionsResponse
from openapi_client.models.campaign.appeal_status import AppealStatus
from openapi_client.models.campaign.application_fault import ApplicationFault
from openapi_client.models.campaign.apply_asset_group_listing_group_actions_request import ApplyAssetGroupListingGroupActionsRequest
from openapi_client.models.campaign.apply_asset_group_listing_group_actions_response import ApplyAssetGroupListingGroupActionsResponse
from openapi_client.models.campaign.apply_customer_list_items_request import ApplyCustomerListItemsRequest
from openapi_client.models.campaign.apply_customer_list_items_response import ApplyCustomerListItemsResponse
from openapi_client.models.campaign.apply_customer_list_user_data_request import ApplyCustomerListUserDataRequest
from openapi_client.models.campaign.apply_customer_list_user_data_response import ApplyCustomerListUserDataResponse
from openapi_client.models.campaign.apply_hotel_group_actions_request import ApplyHotelGroupActionsRequest
from openapi_client.models.campaign.apply_hotel_group_actions_response import ApplyHotelGroupActionsResponse
from openapi_client.models.campaign.apply_offline_conversion_adjustments_request import ApplyOfflineConversionAdjustmentsRequest
from openapi_client.models.campaign.apply_offline_conversion_adjustments_response import ApplyOfflineConversionAdjustmentsResponse
from openapi_client.models.campaign.apply_offline_conversions_request import ApplyOfflineConversionsRequest
from openapi_client.models.campaign.apply_offline_conversions_response import ApplyOfflineConversionsResponse
from openapi_client.models.campaign.apply_online_conversion_adjustments_request import ApplyOnlineConversionAdjustmentsRequest
from openapi_client.models.campaign.apply_online_conversion_adjustments_response import ApplyOnlineConversionAdjustmentsResponse
from openapi_client.models.campaign.apply_product_partition_actions_request import ApplyProductPartitionActionsRequest
from openapi_client.models.campaign.apply_product_partition_actions_response import ApplyProductPartitionActionsResponse
from openapi_client.models.campaign.asset import Asset
from openapi_client.models.campaign.asset_group import AssetGroup
from openapi_client.models.campaign.asset_group_additional_field import AssetGroupAdditionalField
from openapi_client.models.campaign.asset_group_editorial_reason import AssetGroupEditorialReason
from openapi_client.models.campaign.asset_group_editorial_reason_collection import AssetGroupEditorialReasonCollection
from openapi_client.models.campaign.asset_group_editorial_status import AssetGroupEditorialStatus
from openapi_client.models.campaign.asset_group_listing_group import AssetGroupListingGroup
from openapi_client.models.campaign.asset_group_listing_group_action import AssetGroupListingGroupAction
from openapi_client.models.campaign.asset_group_listing_group_additional_field import AssetGroupListingGroupAdditionalField
from openapi_client.models.campaign.asset_group_listing_type import AssetGroupListingType
from openapi_client.models.campaign.asset_group_search_theme import AssetGroupSearchTheme
from openapi_client.models.campaign.asset_group_status import AssetGroupStatus
from openapi_client.models.campaign.asset_group_url_target import AssetGroupUrlTarget
from openapi_client.models.campaign.asset_link import AssetLink
from openapi_client.models.campaign.asset_link_editorial_status import AssetLinkEditorialStatus
from openapi_client.models.campaign.association_type import AssociationType
from openapi_client.models.campaign.attribution_model_type import AttributionModelType
from openapi_client.models.campaign.audience import Audience
from openapi_client.models.campaign.audience_additional_field import AudienceAdditionalField
from openapi_client.models.campaign.audience_condition import AudienceCondition
from openapi_client.models.campaign.audience_condition_item import AudienceConditionItem
from openapi_client.models.campaign.audience_criterion import AudienceCriterion
from openapi_client.models.campaign.audience_dimension import AudienceDimension
from openapi_client.models.campaign.audience_group import AudienceGroup
from openapi_client.models.campaign.audience_group_additional_field import AudienceGroupAdditionalField
from openapi_client.models.campaign.audience_group_asset_group_association import AudienceGroupAssetGroupAssociation
from openapi_client.models.campaign.audience_group_dimension import AudienceGroupDimension
from openapi_client.models.campaign.audience_group_dimension_type import AudienceGroupDimensionType
from openapi_client.models.campaign.audience_id_name import AudienceIdName
from openapi_client.models.campaign.audience_info import AudienceInfo
from openapi_client.models.campaign.audience_type import AudienceType
from openapi_client.models.campaign.audio_filter import AudioFilter
from openapi_client.models.campaign.authentication_fault_code import AuthenticationFaultCode
from openapi_client.models.campaign.bmc_store import BMCStore
from openapi_client.models.campaign.bmc_store_additional_field import BMCStoreAdditionalField
from openapi_client.models.campaign.bmc_store_sub_type import BMCStoreSubType
from openapi_client.models.campaign.batch_error import BatchError
from openapi_client.models.campaign.batch_error_base import BatchErrorBase
from openapi_client.models.campaign.batch_error_collection import BatchErrorCollection
from openapi_client.models.campaign.batch_error_collection_base import BatchErrorCollectionBase
from openapi_client.models.campaign.bid import Bid
from openapi_client.models.campaign.bid_multiplier import BidMultiplier
from openapi_client.models.campaign.bid_option import BidOption
from openapi_client.models.campaign.bid_strategy import BidStrategy
from openapi_client.models.campaign.biddable_ad_group_criterion import BiddableAdGroupCriterion
from openapi_client.models.campaign.biddable_campaign_criterion import BiddableCampaignCriterion
from openapi_client.models.campaign.bidding_scheme import BiddingScheme
from openapi_client.models.campaign.brand_item import BrandItem
from openapi_client.models.campaign.brand_kit import BrandKit
from openapi_client.models.campaign.brand_kit_additional_field import BrandKitAdditionalField
from openapi_client.models.campaign.brand_kit_color import BrandKitColor
from openapi_client.models.campaign.brand_kit_font import BrandKitFont
from openapi_client.models.campaign.brand_kit_image import BrandKitImage
from openapi_client.models.campaign.brand_kit_palette import BrandKitPalette
from openapi_client.models.campaign.brand_list import BrandList
from openapi_client.models.campaign.budget import Budget
from openapi_client.models.campaign.budget_limit_type import BudgetLimitType
from openapi_client.models.campaign.business_geo_code_status import BusinessGeoCodeStatus
from openapi_client.models.campaign.call_ad_extension import CallAdExtension
from openapi_client.models.campaign.call_to_action import CallToAction
from openapi_client.models.campaign.call_to_action_setting import CallToActionSetting
from openapi_client.models.campaign.callout_ad_extension import CalloutAdExtension
from openapi_client.models.campaign.campaign import Campaign
from openapi_client.models.campaign.campaign_ad_group_ids import CampaignAdGroupIds
from openapi_client.models.campaign.campaign_additional_field import CampaignAdditionalField
from openapi_client.models.campaign.campaign_association import CampaignAssociation
from openapi_client.models.campaign.campaign_conversion_goal import CampaignConversionGoal
from openapi_client.models.campaign.campaign_criterion import CampaignCriterion
from openapi_client.models.campaign.campaign_criterion_status import CampaignCriterionStatus
from openapi_client.models.campaign.campaign_criterion_type import CampaignCriterionType
from openapi_client.models.campaign.campaign_negative_sites import CampaignNegativeSites
from openapi_client.models.campaign.campaign_size import CampaignSize
from openapi_client.models.campaign.campaign_status import CampaignStatus
from openapi_client.models.campaign.campaign_type import CampaignType
from openapi_client.models.campaign.cashback_adjustment import CashbackAdjustment
from openapi_client.models.campaign.clipchamp_template_info import ClipchampTemplateInfo
from openapi_client.models.campaign.co_op_setting import CoOpSetting
from openapi_client.models.campaign.combination_rule import CombinationRule
from openapi_client.models.campaign.combined_list import CombinedList
from openapi_client.models.campaign.commission_bidding_scheme import CommissionBiddingScheme
from openapi_client.models.campaign.company import Company
from openapi_client.models.campaign.compression_type import CompressionType
from openapi_client.models.campaign.confirmation_action import ConfirmationAction
from openapi_client.models.campaign.conversion_goal import ConversionGoal
from openapi_client.models.campaign.conversion_goal_additional_field import ConversionGoalAdditionalField
from openapi_client.models.campaign.conversion_goal_category import ConversionGoalCategory
from openapi_client.models.campaign.conversion_goal_count_type import ConversionGoalCountType
from openapi_client.models.campaign.conversion_goal_revenue import ConversionGoalRevenue
from openapi_client.models.campaign.conversion_goal_revenue_type import ConversionGoalRevenueType
from openapi_client.models.campaign.conversion_goal_status import ConversionGoalStatus
from openapi_client.models.campaign.conversion_goal_tracking_status import ConversionGoalTrackingStatus
from openapi_client.models.campaign.conversion_goal_type import ConversionGoalType
from openapi_client.models.campaign.conversion_value_rule import ConversionValueRule
from openapi_client.models.campaign.conversion_value_rule_operator import ConversionValueRuleOperator
from openapi_client.models.campaign.conversion_value_rule_status import ConversionValueRuleStatus
from openapi_client.models.campaign.cost_per_sale_bidding_scheme import CostPerSaleBiddingScheme
from openapi_client.models.campaign.create_asset_group_recommendation_request import CreateAssetGroupRecommendationRequest
from openapi_client.models.campaign.create_asset_group_recommendation_response import CreateAssetGroupRecommendationResponse
from openapi_client.models.campaign.create_responsive_ad_recommendation_request import CreateResponsiveAdRecommendationRequest
from openapi_client.models.campaign.create_responsive_ad_recommendation_response import CreateResponsiveAdRecommendationResponse
from openapi_client.models.campaign.create_responsive_search_ad_recommendation_request import CreateResponsiveSearchAdRecommendationRequest
from openapi_client.models.campaign.create_responsive_search_ad_recommendation_response import CreateResponsiveSearchAdRecommendationResponse
from openapi_client.models.campaign.criterion import Criterion
from openapi_client.models.campaign.criterion_additional_field import CriterionAdditionalField
from openapi_client.models.campaign.criterion_bid import CriterionBid
from openapi_client.models.campaign.criterion_cashback import CriterionCashback
from openapi_client.models.campaign.criterion_type_group import CriterionTypeGroup
from openapi_client.models.campaign.custom_audience import CustomAudience
from openapi_client.models.campaign.custom_events_rule import CustomEventsRule
from openapi_client.models.campaign.custom_parameter import CustomParameter
from openapi_client.models.campaign.custom_parameters import CustomParameters
from openapi_client.models.campaign.customer_account_share import CustomerAccountShare
from openapi_client.models.campaign.customer_account_share_association import CustomerAccountShareAssociation
from openapi_client.models.campaign.customer_list import CustomerList
from openapi_client.models.campaign.customer_list_action_type import CustomerListActionType
from openapi_client.models.campaign.customer_list_item_sub_type import CustomerListItemSubType
from openapi_client.models.campaign.customer_list_user_data import CustomerListUserData
from openapi_client.models.campaign.customer_share import CustomerShare
from openapi_client.models.campaign.daily_summary import DailySummary
from openapi_client.models.campaign.data_exclusion import DataExclusion
from openapi_client.models.campaign.data_table_ad_extension import DataTableAdExtension
from openapi_client.models.campaign.day import Day
from openapi_client.models.campaign.day_time import DayTime
from openapi_client.models.campaign.day_time_criterion import DayTimeCriterion
from openapi_client.models.campaign.deal_criterion import DealCriterion
from openapi_client.models.campaign.delete_ad_extensions_associations_request import DeleteAdExtensionsAssociationsRequest
from openapi_client.models.campaign.delete_ad_extensions_associations_response import DeleteAdExtensionsAssociationsResponse
from openapi_client.models.campaign.delete_ad_extensions_request import DeleteAdExtensionsRequest
from openapi_client.models.campaign.delete_ad_extensions_response import DeleteAdExtensionsResponse
from openapi_client.models.campaign.delete_ad_group_criterions_request import DeleteAdGroupCriterionsRequest
from openapi_client.models.campaign.delete_ad_group_criterions_response import DeleteAdGroupCriterionsResponse
from openapi_client.models.campaign.delete_ad_groups_request import DeleteAdGroupsRequest
from openapi_client.models.campaign.delete_ad_groups_response import DeleteAdGroupsResponse
from openapi_client.models.campaign.delete_ads_request import DeleteAdsRequest
from openapi_client.models.campaign.delete_ads_response import DeleteAdsResponse
from openapi_client.models.campaign.delete_asset_groups_request import DeleteAssetGroupsRequest
from openapi_client.models.campaign.delete_asset_groups_response import DeleteAssetGroupsResponse
from openapi_client.models.campaign.delete_audience_group_asset_group_associations_request import DeleteAudienceGroupAssetGroupAssociationsRequest
from openapi_client.models.campaign.delete_audience_group_asset_group_associations_response import DeleteAudienceGroupAssetGroupAssociationsResponse
from openapi_client.models.campaign.delete_audience_groups_request import DeleteAudienceGroupsRequest
from openapi_client.models.campaign.delete_audience_groups_response import DeleteAudienceGroupsResponse
from openapi_client.models.campaign.delete_audiences_request import DeleteAudiencesRequest
from openapi_client.models.campaign.delete_audiences_response import DeleteAudiencesResponse
from openapi_client.models.campaign.delete_bid_strategies_request import DeleteBidStrategiesRequest
from openapi_client.models.campaign.delete_bid_strategies_response import DeleteBidStrategiesResponse
from openapi_client.models.campaign.delete_brand_kits_request import DeleteBrandKitsRequest
from openapi_client.models.campaign.delete_brand_kits_response import DeleteBrandKitsResponse
from openapi_client.models.campaign.delete_budgets_request import DeleteBudgetsRequest
from openapi_client.models.campaign.delete_budgets_response import DeleteBudgetsResponse
from openapi_client.models.campaign.delete_campaign_conversion_goals_request import DeleteCampaignConversionGoalsRequest
from openapi_client.models.campaign.delete_campaign_conversion_goals_response import DeleteCampaignConversionGoalsResponse
from openapi_client.models.campaign.delete_campaign_criterions_request import DeleteCampaignCriterionsRequest
from openapi_client.models.campaign.delete_campaign_criterions_response import DeleteCampaignCriterionsResponse
from openapi_client.models.campaign.delete_campaigns_request import DeleteCampaignsRequest
from openapi_client.models.campaign.delete_campaigns_response import DeleteCampaignsResponse
from openapi_client.models.campaign.delete_data_exclusions_request import DeleteDataExclusionsRequest
from openapi_client.models.campaign.delete_data_exclusions_response import DeleteDataExclusionsResponse
from openapi_client.models.campaign.delete_experiments_request import DeleteExperimentsRequest
from openapi_client.models.campaign.delete_experiments_response import DeleteExperimentsResponse
from openapi_client.models.campaign.delete_import_jobs_request import DeleteImportJobsRequest
from openapi_client.models.campaign.delete_import_jobs_response import DeleteImportJobsResponse
from openapi_client.models.campaign.delete_keywords_request import DeleteKeywordsRequest
from openapi_client.models.campaign.delete_keywords_response import DeleteKeywordsResponse
from openapi_client.models.campaign.delete_label_associations_request import DeleteLabelAssociationsRequest
from openapi_client.models.campaign.delete_label_associations_response import DeleteLabelAssociationsResponse
from openapi_client.models.campaign.delete_labels_request import DeleteLabelsRequest
from openapi_client.models.campaign.delete_labels_response import DeleteLabelsResponse
from openapi_client.models.campaign.delete_list_items_from_shared_list_request import DeleteListItemsFromSharedListRequest
from openapi_client.models.campaign.delete_list_items_from_shared_list_response import DeleteListItemsFromSharedListResponse
from openapi_client.models.campaign.delete_media_request import DeleteMediaRequest
from openapi_client.models.campaign.delete_media_response import DeleteMediaResponse
from openapi_client.models.campaign.delete_negative_keywords_from_entities_request import DeleteNegativeKeywordsFromEntitiesRequest
from openapi_client.models.campaign.delete_negative_keywords_from_entities_response import DeleteNegativeKeywordsFromEntitiesResponse
from openapi_client.models.campaign.delete_seasonality_adjustments_request import DeleteSeasonalityAdjustmentsRequest
from openapi_client.models.campaign.delete_seasonality_adjustments_response import DeleteSeasonalityAdjustmentsResponse
from openapi_client.models.campaign.delete_shared_entities_request import DeleteSharedEntitiesRequest
from openapi_client.models.campaign.delete_shared_entities_response import DeleteSharedEntitiesResponse
from openapi_client.models.campaign.delete_shared_entity_associations_request import DeleteSharedEntityAssociationsRequest
from openapi_client.models.campaign.delete_shared_entity_associations_response import DeleteSharedEntityAssociationsResponse
from openapi_client.models.campaign.delete_videos_request import DeleteVideosRequest
from openapi_client.models.campaign.delete_videos_response import DeleteVideosResponse
from openapi_client.models.campaign.device_condition import DeviceCondition
from openapi_client.models.campaign.device_criterion import DeviceCriterion
from openapi_client.models.campaign.device_type import DeviceType
from openapi_client.models.campaign.disclaimer_ad_extension import DisclaimerAdExtension
from openapi_client.models.campaign.disclaimer_setting import DisclaimerSetting
from openapi_client.models.campaign.distance_unit import DistanceUnit
from openapi_client.models.campaign.duration_goal import DurationGoal
from openapi_client.models.campaign.dynamic_feed_setting import DynamicFeedSetting
from openapi_client.models.campaign.dynamic_search_ad import DynamicSearchAd
from openapi_client.models.campaign.dynamic_search_ads_setting import DynamicSearchAdsSetting
from openapi_client.models.campaign.dynamic_search_ads_source import DynamicSearchAdsSource
from openapi_client.models.campaign.editorial_api_fault_detail import EditorialApiFaultDetail
from openapi_client.models.campaign.editorial_error import EditorialError
from openapi_client.models.campaign.editorial_error_collection import EditorialErrorCollection
from openapi_client.models.campaign.editorial_reason import EditorialReason
from openapi_client.models.campaign.editorial_reason_collection import EditorialReasonCollection
from openapi_client.models.campaign.enhanced_cpc_bidding_scheme import EnhancedCpcBiddingScheme
from openapi_client.models.campaign.entity_id_to_parent_id_association import EntityIdToParentIdAssociation
from openapi_client.models.campaign.entity_negative_keyword import EntityNegativeKeyword
from openapi_client.models.campaign.entity_scope import EntityScope
from openapi_client.models.campaign.entity_type import EntityType
from openapi_client.models.campaign.event_goal import EventGoal
from openapi_client.models.campaign.expanded_text_ad import ExpandedTextAd
from openapi_client.models.campaign.experiment import Experiment
from openapi_client.models.campaign.expression_operator import ExpressionOperator
from openapi_client.models.campaign.fault_source import FaultSource
from openapi_client.models.campaign.file_import_job import FileImportJob
from openapi_client.models.campaign.file_import_option import FileImportOption
from openapi_client.models.campaign.filter_link_ad_extension import FilterLinkAdExtension
from openapi_client.models.campaign.fixed_bid import FixedBid
from openapi_client.models.campaign.flyer_ad_extension import FlyerAdExtension
from openapi_client.models.campaign.font_text_asset_type import FontTextAssetType
from openapi_client.models.campaign.frequency import Frequency
from openapi_client.models.campaign.frequency_cap_settings import FrequencyCapSettings
from openapi_client.models.campaign.frequency_cap_time_granularity import FrequencyCapTimeGranularity
from openapi_client.models.campaign.gender_criterion import GenderCriterion
from openapi_client.models.campaign.gender_dimension import GenderDimension
from openapi_client.models.campaign.gender_type import GenderType
from openapi_client.models.campaign.generic_entity_status import GenericEntityStatus
from openapi_client.models.campaign.genre_criterion import GenreCriterion
from openapi_client.models.campaign.geo_point import GeoPoint
from openapi_client.models.campaign.get_account_migration_statuses_request import GetAccountMigrationStatusesRequest
from openapi_client.models.campaign.get_account_migration_statuses_response import GetAccountMigrationStatusesResponse
from openapi_client.models.campaign.get_account_properties_request import GetAccountPropertiesRequest
from openapi_client.models.campaign.get_account_properties_response import GetAccountPropertiesResponse
from openapi_client.models.campaign.get_ad_extension_ids_by_account_id_request import GetAdExtensionIdsByAccountIdRequest
from openapi_client.models.campaign.get_ad_extension_ids_by_account_id_response import GetAdExtensionIdsByAccountIdResponse
from openapi_client.models.campaign.get_ad_extensions_associations_request import GetAdExtensionsAssociationsRequest
from openapi_client.models.campaign.get_ad_extensions_associations_response import GetAdExtensionsAssociationsResponse
from openapi_client.models.campaign.get_ad_extensions_by_ids_request import GetAdExtensionsByIdsRequest
from openapi_client.models.campaign.get_ad_extensions_by_ids_response import GetAdExtensionsByIdsResponse
from openapi_client.models.campaign.get_ad_extensions_editorial_reasons_request import GetAdExtensionsEditorialReasonsRequest
from openapi_client.models.campaign.get_ad_extensions_editorial_reasons_response import GetAdExtensionsEditorialReasonsResponse
from openapi_client.models.campaign.get_ad_group_criterions_by_ids_request import GetAdGroupCriterionsByIdsRequest
from openapi_client.models.campaign.get_ad_group_criterions_by_ids_response import GetAdGroupCriterionsByIdsResponse
from openapi_client.models.campaign.get_ad_groups_by_campaign_id_request import GetAdGroupsByCampaignIdRequest
from openapi_client.models.campaign.get_ad_groups_by_campaign_id_response import GetAdGroupsByCampaignIdResponse
from openapi_client.models.campaign.get_ad_groups_by_ids_request import GetAdGroupsByIdsRequest
from openapi_client.models.campaign.get_ad_groups_by_ids_response import GetAdGroupsByIdsResponse
from openapi_client.models.campaign.get_ads_by_ad_group_id_request import GetAdsByAdGroupIdRequest
from openapi_client.models.campaign.get_ads_by_ad_group_id_response import GetAdsByAdGroupIdResponse
from openapi_client.models.campaign.get_ads_by_editorial_status_request import GetAdsByEditorialStatusRequest
from openapi_client.models.campaign.get_ads_by_editorial_status_response import GetAdsByEditorialStatusResponse
from openapi_client.models.campaign.get_ads_by_ids_request import GetAdsByIdsRequest
from openapi_client.models.campaign.get_ads_by_ids_response import GetAdsByIdsResponse
from openapi_client.models.campaign.get_annotation_opt_out_response import GetAnnotationOptOutResponse
from openapi_client.models.campaign.get_asset_group_listing_groups_by_ids_request import GetAssetGroupListingGroupsByIdsRequest
from openapi_client.models.campaign.get_asset_group_listing_groups_by_ids_response import GetAssetGroupListingGroupsByIdsResponse
from openapi_client.models.campaign.get_asset_groups_by_campaign_id_request import GetAssetGroupsByCampaignIdRequest
from openapi_client.models.campaign.get_asset_groups_by_campaign_id_response import GetAssetGroupsByCampaignIdResponse
from openapi_client.models.campaign.get_asset_groups_by_ids_request import GetAssetGroupsByIdsRequest
from openapi_client.models.campaign.get_asset_groups_by_ids_response import GetAssetGroupsByIdsResponse
from openapi_client.models.campaign.get_asset_groups_editorial_reasons_request import GetAssetGroupsEditorialReasonsRequest
from openapi_client.models.campaign.get_asset_groups_editorial_reasons_response import GetAssetGroupsEditorialReasonsResponse
from openapi_client.models.campaign.get_audience_group_asset_group_associations_by_asset_group_ids_request import GetAudienceGroupAssetGroupAssociationsByAssetGroupIdsRequest
from openapi_client.models.campaign.get_audience_group_asset_group_associations_by_asset_group_ids_response import GetAudienceGroupAssetGroupAssociationsByAssetGroupIdsResponse
from openapi_client.models.campaign.get_audience_group_asset_group_associations_by_audience_group_ids_request import GetAudienceGroupAssetGroupAssociationsByAudienceGroupIdsRequest
from openapi_client.models.campaign.get_audience_group_asset_group_associations_by_audience_group_ids_response import GetAudienceGroupAssetGroupAssociationsByAudienceGroupIdsResponse
from openapi_client.models.campaign.get_audience_groups_by_ids_request import GetAudienceGroupsByIdsRequest
from openapi_client.models.campaign.get_audience_groups_by_ids_response import GetAudienceGroupsByIdsResponse
from openapi_client.models.campaign.get_audiences_by_ids_request import GetAudiencesByIdsRequest
from openapi_client.models.campaign.get_audiences_by_ids_response import GetAudiencesByIdsResponse
from openapi_client.models.campaign.get_bmc_stores_by_customer_id_request import GetBMCStoresByCustomerIdRequest
from openapi_client.models.campaign.get_bmc_stores_by_customer_id_response import GetBMCStoresByCustomerIdResponse
from openapi_client.models.campaign.get_bsc_countries_response import GetBSCCountriesResponse
from openapi_client.models.campaign.get_bid_strategies_by_ids_request import GetBidStrategiesByIdsRequest
from openapi_client.models.campaign.get_bid_strategies_by_ids_response import GetBidStrategiesByIdsResponse
from openapi_client.models.campaign.get_brand_kits_by_account_id_request import GetBrandKitsByAccountIdRequest
from openapi_client.models.campaign.get_brand_kits_by_account_id_response import GetBrandKitsByAccountIdResponse
from openapi_client.models.campaign.get_brand_kits_by_ids_request import GetBrandKitsByIdsRequest
from openapi_client.models.campaign.get_brand_kits_by_ids_response import GetBrandKitsByIdsResponse
from openapi_client.models.campaign.get_budgets_by_ids_request import GetBudgetsByIdsRequest
from openapi_client.models.campaign.get_budgets_by_ids_response import GetBudgetsByIdsResponse
from openapi_client.models.campaign.get_campaign_criterions_by_ids_request import GetCampaignCriterionsByIdsRequest
from openapi_client.models.campaign.get_campaign_criterions_by_ids_response import GetCampaignCriterionsByIdsResponse
from openapi_client.models.campaign.get_campaign_ids_by_bid_strategy_ids_request import GetCampaignIdsByBidStrategyIdsRequest
from openapi_client.models.campaign.get_campaign_ids_by_bid_strategy_ids_response import GetCampaignIdsByBidStrategyIdsResponse
from openapi_client.models.campaign.get_campaign_ids_by_budget_ids_request import GetCampaignIdsByBudgetIdsRequest
from openapi_client.models.campaign.get_campaign_ids_by_budget_ids_response import GetCampaignIdsByBudgetIdsResponse
from openapi_client.models.campaign.get_campaign_sizes_by_account_id_request import GetCampaignSizesByAccountIdRequest
from openapi_client.models.campaign.get_campaign_sizes_by_account_id_response import GetCampaignSizesByAccountIdResponse
from openapi_client.models.campaign.get_campaigns_by_account_id_request import GetCampaignsByAccountIdRequest
from openapi_client.models.campaign.get_campaigns_by_account_id_response import GetCampaignsByAccountIdResponse
from openapi_client.models.campaign.get_campaigns_by_ids_request import GetCampaignsByIdsRequest
from openapi_client.models.campaign.get_campaigns_by_ids_response import GetCampaignsByIdsResponse
from openapi_client.models.campaign.get_clipchamp_templates_request import GetClipchampTemplatesRequest
from openapi_client.models.campaign.get_clipchamp_templates_response import GetClipchampTemplatesResponse
from openapi_client.models.campaign.get_config_value_request import GetConfigValueRequest
from openapi_client.models.campaign.get_config_value_response import GetConfigValueResponse
from openapi_client.models.campaign.get_conversion_goals_by_ids_request import GetConversionGoalsByIdsRequest
from openapi_client.models.campaign.get_conversion_goals_by_ids_response import GetConversionGoalsByIdsResponse
from openapi_client.models.campaign.get_conversion_goals_by_tag_ids_request import GetConversionGoalsByTagIdsRequest
from openapi_client.models.campaign.get_conversion_goals_by_tag_ids_response import GetConversionGoalsByTagIdsResponse
from openapi_client.models.campaign.get_conversion_value_rules_by_account_id_request import GetConversionValueRulesByAccountIdRequest
from openapi_client.models.campaign.get_conversion_value_rules_by_account_id_response import GetConversionValueRulesByAccountIdResponse
from openapi_client.models.campaign.get_conversion_value_rules_by_ids_request import GetConversionValueRulesByIdsRequest
from openapi_client.models.campaign.get_conversion_value_rules_by_ids_response import GetConversionValueRulesByIdsResponse
from openapi_client.models.campaign.get_data_exclusions_by_account_id_request import GetDataExclusionsByAccountIdRequest
from openapi_client.models.campaign.get_data_exclusions_by_account_id_response import GetDataExclusionsByAccountIdResponse
from openapi_client.models.campaign.get_data_exclusions_by_ids_request import GetDataExclusionsByIdsRequest
from openapi_client.models.campaign.get_data_exclusions_by_ids_response import GetDataExclusionsByIdsResponse
from openapi_client.models.campaign.get_editorial_reasons_by_ids_request import GetEditorialReasonsByIdsRequest
from openapi_client.models.campaign.get_editorial_reasons_by_ids_response import GetEditorialReasonsByIdsResponse
from openapi_client.models.campaign.get_experiments_by_ids_request import GetExperimentsByIdsRequest
from openapi_client.models.campaign.get_experiments_by_ids_response import GetExperimentsByIdsResponse
from openapi_client.models.campaign.get_file_import_upload_url_response import GetFileImportUploadUrlResponse
from openapi_client.models.campaign.get_geo_locations_file_url_request import GetGeoLocationsFileUrlRequest
from openapi_client.models.campaign.get_geo_locations_file_url_response import GetGeoLocationsFileUrlResponse
from openapi_client.models.campaign.get_health_check_request import GetHealthCheckRequest
from openapi_client.models.campaign.get_health_check_response import GetHealthCheckResponse
from openapi_client.models.campaign.get_import_entity_ids_mapping_request import GetImportEntityIdsMappingRequest
from openapi_client.models.campaign.get_import_entity_ids_mapping_response import GetImportEntityIdsMappingResponse
from openapi_client.models.campaign.get_import_jobs_by_ids_request import GetImportJobsByIdsRequest
from openapi_client.models.campaign.get_import_jobs_by_ids_response import GetImportJobsByIdsResponse
from openapi_client.models.campaign.get_import_results_request import GetImportResultsRequest
from openapi_client.models.campaign.get_import_results_response import GetImportResultsResponse
from openapi_client.models.campaign.get_keywords_by_ad_group_id_request import GetKeywordsByAdGroupIdRequest
from openapi_client.models.campaign.get_keywords_by_ad_group_id_response import GetKeywordsByAdGroupIdResponse
from openapi_client.models.campaign.get_keywords_by_editorial_status_request import GetKeywordsByEditorialStatusRequest
from openapi_client.models.campaign.get_keywords_by_editorial_status_response import GetKeywordsByEditorialStatusResponse
from openapi_client.models.campaign.get_keywords_by_ids_request import GetKeywordsByIdsRequest
from openapi_client.models.campaign.get_keywords_by_ids_response import GetKeywordsByIdsResponse
from openapi_client.models.campaign.get_label_associations_by_entity_ids_request import GetLabelAssociationsByEntityIdsRequest
from openapi_client.models.campaign.get_label_associations_by_entity_ids_response import GetLabelAssociationsByEntityIdsResponse
from openapi_client.models.campaign.get_label_associations_by_label_ids_request import GetLabelAssociationsByLabelIdsRequest
from openapi_client.models.campaign.get_label_associations_by_label_ids_response import GetLabelAssociationsByLabelIdsResponse
from openapi_client.models.campaign.get_labels_by_ids_request import GetLabelsByIdsRequest
from openapi_client.models.campaign.get_labels_by_ids_response import GetLabelsByIdsResponse
from openapi_client.models.campaign.get_list_items_by_shared_list_request import GetListItemsBySharedListRequest
from openapi_client.models.campaign.get_list_items_by_shared_list_response import GetListItemsBySharedListResponse
from openapi_client.models.campaign.get_media_associations_request import GetMediaAssociationsRequest
from openapi_client.models.campaign.get_media_associations_response import GetMediaAssociationsResponse
from openapi_client.models.campaign.get_media_meta_data_by_account_id_request import GetMediaMetaDataByAccountIdRequest
from openapi_client.models.campaign.get_media_meta_data_by_account_id_response import GetMediaMetaDataByAccountIdResponse
from openapi_client.models.campaign.get_media_meta_data_by_ids_request import GetMediaMetaDataByIdsRequest
from openapi_client.models.campaign.get_media_meta_data_by_ids_response import GetMediaMetaDataByIdsResponse
from openapi_client.models.campaign.get_negative_keywords_by_entity_ids_request import GetNegativeKeywordsByEntityIdsRequest
from openapi_client.models.campaign.get_negative_keywords_by_entity_ids_response import GetNegativeKeywordsByEntityIdsResponse
from openapi_client.models.campaign.get_negative_sites_by_ad_group_ids_request import GetNegativeSitesByAdGroupIdsRequest
from openapi_client.models.campaign.get_negative_sites_by_ad_group_ids_response import GetNegativeSitesByAdGroupIdsResponse
from openapi_client.models.campaign.get_negative_sites_by_campaign_ids_request import GetNegativeSitesByCampaignIdsRequest
from openapi_client.models.campaign.get_negative_sites_by_campaign_ids_response import GetNegativeSitesByCampaignIdsResponse
from openapi_client.models.campaign.get_new_customer_acquisition_goals_by_account_id_request import GetNewCustomerAcquisitionGoalsByAccountIdRequest
from openapi_client.models.campaign.get_new_customer_acquisition_goals_by_account_id_response import GetNewCustomerAcquisitionGoalsByAccountIdResponse
from openapi_client.models.campaign.get_offline_conversion_reports_request import GetOfflineConversionReportsRequest
from openapi_client.models.campaign.get_offline_conversion_reports_response import GetOfflineConversionReportsResponse
from openapi_client.models.campaign.get_profile_data_file_url_request import GetProfileDataFileUrlRequest
from openapi_client.models.campaign.get_profile_data_file_url_response import GetProfileDataFileUrlResponse
from openapi_client.models.campaign.get_seasonality_adjustments_by_account_id_request import GetSeasonalityAdjustmentsByAccountIdRequest
from openapi_client.models.campaign.get_seasonality_adjustments_by_account_id_response import GetSeasonalityAdjustmentsByAccountIdResponse
from openapi_client.models.campaign.get_seasonality_adjustments_by_ids_request import GetSeasonalityAdjustmentsByIdsRequest
from openapi_client.models.campaign.get_seasonality_adjustments_by_ids_response import GetSeasonalityAdjustmentsByIdsResponse
from openapi_client.models.campaign.get_shared_entities_by_account_id_request import GetSharedEntitiesByAccountIdRequest
from openapi_client.models.campaign.get_shared_entities_by_account_id_response import GetSharedEntitiesByAccountIdResponse
from openapi_client.models.campaign.get_shared_entities_request import GetSharedEntitiesRequest
from openapi_client.models.campaign.get_shared_entities_response import GetSharedEntitiesResponse
from openapi_client.models.campaign.get_shared_entity_associations_by_entity_ids_request import GetSharedEntityAssociationsByEntityIdsRequest
from openapi_client.models.campaign.get_shared_entity_associations_by_entity_ids_response import GetSharedEntityAssociationsByEntityIdsResponse
from openapi_client.models.campaign.get_shared_entity_associations_by_shared_entity_ids_request import GetSharedEntityAssociationsBySharedEntityIdsRequest
from openapi_client.models.campaign.get_shared_entity_associations_by_shared_entity_ids_response import GetSharedEntityAssociationsBySharedEntityIdsResponse
from openapi_client.models.campaign.get_supported_clipchamp_audio_request import GetSupportedClipchampAudioRequest
from openapi_client.models.campaign.get_supported_clipchamp_audio_response import GetSupportedClipchampAudioResponse
from openapi_client.models.campaign.get_supported_fonts_request import GetSupportedFontsRequest
from openapi_client.models.campaign.get_supported_fonts_response import GetSupportedFontsResponse
from openapi_client.models.campaign.get_uet_tags_by_ids_request import GetUetTagsByIdsRequest
from openapi_client.models.campaign.get_uet_tags_by_ids_response import GetUetTagsByIdsResponse
from openapi_client.models.campaign.get_videos_by_ids_request import GetVideosByIdsRequest
from openapi_client.models.campaign.get_videos_by_ids_response import GetVideosByIdsResponse
from openapi_client.models.campaign.google_import_job import GoogleImportJob
from openapi_client.models.campaign.google_import_option import GoogleImportOption
from openapi_client.models.campaign.health_check_action_link_metadata import HealthCheckActionLinkMetadata
from openapi_client.models.campaign.health_check_column_metadata import HealthCheckColumnMetadata
from openapi_client.models.campaign.health_check_data import HealthCheckData
from openapi_client.models.campaign.health_check_entity import HealthCheckEntity
from openapi_client.models.campaign.health_check_error import HealthCheckError
from openapi_client.models.campaign.health_check_metadata import HealthCheckMetadata
from openapi_client.models.campaign.health_check_sub_entity_data import HealthCheckSubEntityData
from openapi_client.models.campaign.hotel_ad import HotelAd
from openapi_client.models.campaign.hotel_ad_group_type import HotelAdGroupType
from openapi_client.models.campaign.hotel_advance_booking_window_criterion import HotelAdvanceBookingWindowCriterion
from openapi_client.models.campaign.hotel_check_in_date_criterion import HotelCheckInDateCriterion
from openapi_client.models.campaign.hotel_check_in_day_criterion import HotelCheckInDayCriterion
from openapi_client.models.campaign.hotel_date_selection_type import HotelDateSelectionType
from openapi_client.models.campaign.hotel_date_selection_type_criterion import HotelDateSelectionTypeCriterion
from openapi_client.models.campaign.hotel_group import HotelGroup
from openapi_client.models.campaign.hotel_length_of_stay_criterion import HotelLengthOfStayCriterion
from openapi_client.models.campaign.hotel_listing import HotelListing
from openapi_client.models.campaign.hotel_listing_type import HotelListingType
from openapi_client.models.campaign.hotel_setting import HotelSetting
from openapi_client.models.campaign.id_collection import IdCollection
from openapi_client.models.campaign.image import Image
from openapi_client.models.campaign.image_ad_extension import ImageAdExtension
from openapi_client.models.campaign.image_asset import ImageAsset
from openapi_client.models.campaign.image_media_representation import ImageMediaRepresentation
from openapi_client.models.campaign.import_additional_field import ImportAdditionalField
from openapi_client.models.campaign.import_entity_statistics import ImportEntityStatistics
from openapi_client.models.campaign.import_entity_type import ImportEntityType
from openapi_client.models.campaign.import_job import ImportJob
from openapi_client.models.campaign.import_option import ImportOption
from openapi_client.models.campaign.import_result import ImportResult
from openapi_client.models.campaign.import_search_and_replace_for_string_property import ImportSearchAndReplaceForStringProperty
from openapi_client.models.campaign.impression_based_entity_type import ImpressionBasedEntityType
from openapi_client.models.campaign.impression_based_remarketing_list import ImpressionBasedRemarketingList
from openapi_client.models.campaign.in_market_audience import InMarketAudience
from openapi_client.models.campaign.in_store_transaction_goal import InStoreTransactionGoal
from openapi_client.models.campaign.inherit_from_parent_bidding_scheme import InheritFromParentBiddingScheme
from openapi_client.models.campaign.intent_option import IntentOption
from openapi_client.models.campaign.item_action import ItemAction
from openapi_client.models.campaign.key_frame import KeyFrame
from openapi_client.models.campaign.key_value_pair_oflong_andlong import KeyValuePairOflongAndlong
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.campaign.keyword import Keyword
from openapi_client.models.campaign.keyword_editorial_status import KeywordEditorialStatus
from openapi_client.models.campaign.keyword_status import KeywordStatus
from openapi_client.models.campaign.label import Label
from openapi_client.models.campaign.label_association import LabelAssociation
from openapi_client.models.campaign.language_name import LanguageName
from openapi_client.models.campaign.lead_delivery import LeadDelivery
from openapi_client.models.campaign.lead_form_ad_extension import LeadFormAdExtension
from openapi_client.models.campaign.lead_form_call_to_action import LeadFormCallToAction
from openapi_client.models.campaign.lead_form_question_definition import LeadFormQuestionDefinition
from openapi_client.models.campaign.location_ad_extension import LocationAdExtension
from openapi_client.models.campaign.location_condition import LocationCondition
from openapi_client.models.campaign.location_condition_item import LocationConditionItem
from openapi_client.models.campaign.location_criterion import LocationCriterion
from openapi_client.models.campaign.location_intent_criterion import LocationIntentCriterion
from openapi_client.models.campaign.location_type import LocationType
from openapi_client.models.campaign.logical_operator import LogicalOperator
from openapi_client.models.campaign.logo_ad_extension import LogoAdExtension
from openapi_client.models.campaign.manual_cpa_bidding_scheme import ManualCpaBiddingScheme
from openapi_client.models.campaign.manual_cpc_bidding_scheme import ManualCpcBiddingScheme
from openapi_client.models.campaign.manual_cpm_bidding_scheme import ManualCpmBiddingScheme
from openapi_client.models.campaign.manual_cpv_bidding_scheme import ManualCpvBiddingScheme
from openapi_client.models.campaign.match_type import MatchType
from openapi_client.models.campaign.max_clicks_bidding_scheme import MaxClicksBiddingScheme
from openapi_client.models.campaign.max_conversion_value_bidding_scheme import MaxConversionValueBiddingScheme
from openapi_client.models.campaign.max_conversions_bidding_scheme import MaxConversionsBiddingScheme
from openapi_client.models.campaign.max_roas_bidding_scheme import MaxRoasBiddingScheme
from openapi_client.models.campaign.media import Media
from openapi_client.models.campaign.media_additional_field import MediaAdditionalField
from openapi_client.models.campaign.media_association import MediaAssociation
from openapi_client.models.campaign.media_enabled_entity_filter import MediaEnabledEntityFilter
from openapi_client.models.campaign.media_meta_data import MediaMetaData
from openapi_client.models.campaign.media_representation import MediaRepresentation
from openapi_client.models.campaign.media_representation_base import MediaRepresentationBase
from openapi_client.models.campaign.migration_status import MigrationStatus
from openapi_client.models.campaign.migration_status_info import MigrationStatusInfo
from openapi_client.models.campaign.minute import Minute
from openapi_client.models.campaign.model_date import ModelDate
from openapi_client.models.campaign.negative_ad_group_criterion import NegativeAdGroupCriterion
from openapi_client.models.campaign.negative_campaign_criterion import NegativeCampaignCriterion
from openapi_client.models.campaign.negative_keyword import NegativeKeyword
from openapi_client.models.campaign.negative_keyword_list import NegativeKeywordList
from openapi_client.models.campaign.negative_site import NegativeSite
from openapi_client.models.campaign.network import Network
from openapi_client.models.campaign.new_customer_acquisition_goal import NewCustomerAcquisitionGoal
from openapi_client.models.campaign.new_customer_acquisition_goal_setting import NewCustomerAcquisitionGoalSetting
from openapi_client.models.campaign.news_ad_extension import NewsAdExtension
from openapi_client.models.campaign.normal_form import NormalForm
from openapi_client.models.campaign.number_operator import NumberOperator
from openapi_client.models.campaign.number_rule_item import NumberRuleItem
from openapi_client.models.campaign.offline_conversion import OfflineConversion
from openapi_client.models.campaign.offline_conversion_adjustment import OfflineConversionAdjustment
from openapi_client.models.campaign.offline_conversion_goal import OfflineConversionGoal
from openapi_client.models.campaign.online_conversion_adjustment import OnlineConversionAdjustment
from openapi_client.models.campaign.operation_error import OperationError
from openapi_client.models.campaign.page_visitors_rule import PageVisitorsRule
from openapi_client.models.campaign.page_visitors_who_did_not_visit_another_page_rule import PageVisitorsWhoDidNotVisitAnotherPageRule
from openapi_client.models.campaign.page_visitors_who_visited_another_page_rule import PageVisitorsWhoVisitedAnotherPageRule
from openapi_client.models.campaign.pages_viewed_per_visit_goal import PagesViewedPerVisitGoal
from openapi_client.models.campaign.paging import Paging
from openapi_client.models.campaign.percent_cpc_bidding_scheme import PercentCpcBiddingScheme
from openapi_client.models.campaign.performance_max_setting import PerformanceMaxSetting
from openapi_client.models.campaign.placement_criterion import PlacementCriterion
from openapi_client.models.campaign.placement_exclusion_list import PlacementExclusionList
from openapi_client.models.campaign.price_ad_extension import PriceAdExtension
from openapi_client.models.campaign.price_extension_type import PriceExtensionType
from openapi_client.models.campaign.price_qualifier import PriceQualifier
from openapi_client.models.campaign.price_table_row import PriceTableRow
from openapi_client.models.campaign.price_unit import PriceUnit
from openapi_client.models.campaign.product_ad import ProductAd
from openapi_client.models.campaign.product_audience import ProductAudience
from openapi_client.models.campaign.product_audience_type import ProductAudienceType
from openapi_client.models.campaign.product_condition import ProductCondition
from openapi_client.models.campaign.product_partition import ProductPartition
from openapi_client.models.campaign.product_partition_type import ProductPartitionType
from openapi_client.models.campaign.product_scope import ProductScope
from openapi_client.models.campaign.profile_criterion import ProfileCriterion
from openapi_client.models.campaign.profile_dimension import ProfileDimension
from openapi_client.models.campaign.profile_info import ProfileInfo
from openapi_client.models.campaign.profile_type import ProfileType
from openapi_client.models.campaign.promotion_ad_extension import PromotionAdExtension
from openapi_client.models.campaign.promotion_discount_modifier import PromotionDiscountModifier
from openapi_client.models.campaign.promotion_occasion import PromotionOccasion
from openapi_client.models.campaign.radius_criterion import RadiusCriterion
from openapi_client.models.campaign.rate_amount import RateAmount
from openapi_client.models.campaign.rate_bid import RateBid
from openapi_client.models.campaign.recommendation_asset_property import RecommendationAssetProperty
from openapi_client.models.campaign.recommendation_audio import RecommendationAudio
from openapi_client.models.campaign.recommendation_background import RecommendationBackground
from openapi_client.models.campaign.recommendation_business_name import RecommendationBusinessName
from openapi_client.models.campaign.recommendation_color import RecommendationColor
from openapi_client.models.campaign.recommendation_crop_setting import RecommendationCropSetting
from openapi_client.models.campaign.recommendation_image import RecommendationImage
from openapi_client.models.campaign.recommendation_image_size import RecommendationImageSize
from openapi_client.models.campaign.recommendation_logo import RecommendationLogo
from openapi_client.models.campaign.recommendation_scene import RecommendationScene
from openapi_client.models.campaign.recommendation_template import RecommendationTemplate
from openapi_client.models.campaign.recommendation_template_video import RecommendationTemplateVideo
from openapi_client.models.campaign.recommendation_text import RecommendationText
from openapi_client.models.campaign.refine_asset_group_recommendation_request import RefineAssetGroupRecommendationRequest
from openapi_client.models.campaign.refine_asset_group_recommendation_response import RefineAssetGroupRecommendationResponse
from openapi_client.models.campaign.refine_responsive_ad_recommendation_request import RefineResponsiveAdRecommendationRequest
from openapi_client.models.campaign.refine_responsive_ad_recommendation_response import RefineResponsiveAdRecommendationResponse
from openapi_client.models.campaign.refine_responsive_search_ad_recommendation_request import RefineResponsiveSearchAdRecommendationRequest
from openapi_client.models.campaign.refine_responsive_search_ad_recommendation_response import RefineResponsiveSearchAdRecommendationResponse
from openapi_client.models.campaign.remarketing_list import RemarketingList
from openapi_client.models.campaign.remarketing_rule import RemarketingRule
from openapi_client.models.campaign.responsive_ad import ResponsiveAd
from openapi_client.models.campaign.responsive_search_ad import ResponsiveSearchAd
from openapi_client.models.campaign.responsive_search_ads_setting import ResponsiveSearchAdsSetting
from openapi_client.models.campaign.review_ad_extension import ReviewAdExtension
from openapi_client.models.campaign.rule_item import RuleItem
from openapi_client.models.campaign.rule_item_group import RuleItemGroup
from openapi_client.models.campaign.schedule import Schedule
from openapi_client.models.campaign.search_companies_request import SearchCompaniesRequest
from openapi_client.models.campaign.search_companies_response import SearchCompaniesResponse
from openapi_client.models.campaign.seasonality_adjustment import SeasonalityAdjustment
from openapi_client.models.campaign.set_account_properties_request import SetAccountPropertiesRequest
from openapi_client.models.campaign.set_ad_extensions_associations_request import SetAdExtensionsAssociationsRequest
from openapi_client.models.campaign.set_ad_extensions_associations_response import SetAdExtensionsAssociationsResponse
from openapi_client.models.campaign.set_audience_group_asset_group_associations_request import SetAudienceGroupAssetGroupAssociationsRequest
from openapi_client.models.campaign.set_audience_group_asset_group_associations_response import SetAudienceGroupAssetGroupAssociationsResponse
from openapi_client.models.campaign.set_label_associations_request import SetLabelAssociationsRequest
from openapi_client.models.campaign.set_label_associations_response import SetLabelAssociationsResponse
from openapi_client.models.campaign.set_negative_sites_to_ad_groups_request import SetNegativeSitesToAdGroupsRequest
from openapi_client.models.campaign.set_negative_sites_to_ad_groups_response import SetNegativeSitesToAdGroupsResponse
from openapi_client.models.campaign.set_negative_sites_to_campaigns_request import SetNegativeSitesToCampaignsRequest
from openapi_client.models.campaign.set_negative_sites_to_campaigns_response import SetNegativeSitesToCampaignsResponse
from openapi_client.models.campaign.set_shared_entity_associations_request import SetSharedEntityAssociationsRequest
from openapi_client.models.campaign.set_shared_entity_associations_response import SetSharedEntityAssociationsResponse
from openapi_client.models.campaign.setting import Setting
from openapi_client.models.campaign.shared_entity import SharedEntity
from openapi_client.models.campaign.shared_entity_association import SharedEntityAssociation
from openapi_client.models.campaign.shared_list import SharedList
from openapi_client.models.campaign.shared_list_item import SharedListItem
from openapi_client.models.campaign.shopping_setting import ShoppingSetting
from openapi_client.models.campaign.similar_remarketing_list import SimilarRemarketingList
from openapi_client.models.campaign.site import Site
from openapi_client.models.campaign.sitelink_ad_extension import SitelinkAdExtension
from openapi_client.models.campaign.solid_background_asset import SolidBackgroundAsset
from openapi_client.models.campaign.store_criterion import StoreCriterion
from openapi_client.models.campaign.string_operator import StringOperator
from openapi_client.models.campaign.string_rule_item import StringRuleItem
from openapi_client.models.campaign.structured_snippet_ad_extension import StructuredSnippetAdExtension
from openapi_client.models.campaign.supported_clipchamp_audio import SupportedClipchampAudio
from openapi_client.models.campaign.supported_clipchamp_audio_data import SupportedClipchampAudioData
from openapi_client.models.campaign.supported_font import SupportedFont
from openapi_client.models.campaign.supported_fonts_data import SupportedFontsData
from openapi_client.models.campaign.target_cpa_bidding_scheme import TargetCpaBiddingScheme
from openapi_client.models.campaign.target_impression_share_bidding_scheme import TargetImpressionShareBiddingScheme
from openapi_client.models.campaign.target_roas_bidding_scheme import TargetRoasBiddingScheme
from openapi_client.models.campaign.target_setting import TargetSetting
from openapi_client.models.campaign.target_setting_detail import TargetSettingDetail
from openapi_client.models.campaign.text_ad import TextAd
from openapi_client.models.campaign.text_asset import TextAsset
from openapi_client.models.campaign.third_party_measurement_setting import ThirdPartyMeasurementSetting
from openapi_client.models.campaign.uet_tag import UetTag
from openapi_client.models.campaign.uet_tag_tracking_status import UetTagTrackingStatus
from openapi_client.models.campaign.update_ad_extensions_request import UpdateAdExtensionsRequest
from openapi_client.models.campaign.update_ad_extensions_response import UpdateAdExtensionsResponse
from openapi_client.models.campaign.update_ad_group_criterions_request import UpdateAdGroupCriterionsRequest
from openapi_client.models.campaign.update_ad_group_criterions_response import UpdateAdGroupCriterionsResponse
from openapi_client.models.campaign.update_ad_groups_request import UpdateAdGroupsRequest
from openapi_client.models.campaign.update_ad_groups_response import UpdateAdGroupsResponse
from openapi_client.models.campaign.update_ads_request import UpdateAdsRequest
from openapi_client.models.campaign.update_ads_response import UpdateAdsResponse
from openapi_client.models.campaign.update_annotation_opt_out_request import UpdateAnnotationOptOutRequest
from openapi_client.models.campaign.update_asset_groups_request import UpdateAssetGroupsRequest
from openapi_client.models.campaign.update_asset_groups_response import UpdateAssetGroupsResponse
from openapi_client.models.campaign.update_audience_groups_request import UpdateAudienceGroupsRequest
from openapi_client.models.campaign.update_audience_groups_response import UpdateAudienceGroupsResponse
from openapi_client.models.campaign.update_audiences_request import UpdateAudiencesRequest
from openapi_client.models.campaign.update_audiences_response import UpdateAudiencesResponse
from openapi_client.models.campaign.update_bid_strategies_request import UpdateBidStrategiesRequest
from openapi_client.models.campaign.update_bid_strategies_response import UpdateBidStrategiesResponse
from openapi_client.models.campaign.update_brand_kits_request import UpdateBrandKitsRequest
from openapi_client.models.campaign.update_brand_kits_response import UpdateBrandKitsResponse
from openapi_client.models.campaign.update_budgets_request import UpdateBudgetsRequest
from openapi_client.models.campaign.update_budgets_response import UpdateBudgetsResponse
from openapi_client.models.campaign.update_campaign_criterions_request import UpdateCampaignCriterionsRequest
from openapi_client.models.campaign.update_campaign_criterions_response import UpdateCampaignCriterionsResponse
from openapi_client.models.campaign.update_campaigns_request import UpdateCampaignsRequest
from openapi_client.models.campaign.update_campaigns_response import UpdateCampaignsResponse
from openapi_client.models.campaign.update_conversion_goals_request import UpdateConversionGoalsRequest
from openapi_client.models.campaign.update_conversion_goals_response import UpdateConversionGoalsResponse
from openapi_client.models.campaign.update_conversion_value_rules_request import UpdateConversionValueRulesRequest
from openapi_client.models.campaign.update_conversion_value_rules_response import UpdateConversionValueRulesResponse
from openapi_client.models.campaign.update_conversion_value_rules_status_request import UpdateConversionValueRulesStatusRequest
from openapi_client.models.campaign.update_conversion_value_rules_status_response import UpdateConversionValueRulesStatusResponse
from openapi_client.models.campaign.update_data_exclusions_request import UpdateDataExclusionsRequest
from openapi_client.models.campaign.update_data_exclusions_response import UpdateDataExclusionsResponse
from openapi_client.models.campaign.update_experiments_request import UpdateExperimentsRequest
from openapi_client.models.campaign.update_experiments_response import UpdateExperimentsResponse
from openapi_client.models.campaign.update_import_jobs_request import UpdateImportJobsRequest
from openapi_client.models.campaign.update_import_jobs_response import UpdateImportJobsResponse
from openapi_client.models.campaign.update_keywords_request import UpdateKeywordsRequest
from openapi_client.models.campaign.update_keywords_response import UpdateKeywordsResponse
from openapi_client.models.campaign.update_labels_request import UpdateLabelsRequest
from openapi_client.models.campaign.update_labels_response import UpdateLabelsResponse
from openapi_client.models.campaign.update_new_customer_acquisition_goals_request import UpdateNewCustomerAcquisitionGoalsRequest
from openapi_client.models.campaign.update_new_customer_acquisition_goals_response import UpdateNewCustomerAcquisitionGoalsResponse
from openapi_client.models.campaign.update_seasonality_adjustments_request import UpdateSeasonalityAdjustmentsRequest
from openapi_client.models.campaign.update_seasonality_adjustments_response import UpdateSeasonalityAdjustmentsResponse
from openapi_client.models.campaign.update_shared_entities_request import UpdateSharedEntitiesRequest
from openapi_client.models.campaign.update_shared_entities_response import UpdateSharedEntitiesResponse
from openapi_client.models.campaign.update_uet_tags_request import UpdateUetTagsRequest
from openapi_client.models.campaign.update_uet_tags_response import UpdateUetTagsResponse
from openapi_client.models.campaign.update_videos_request import UpdateVideosRequest
from openapi_client.models.campaign.update_videos_response import UpdateVideosResponse
from openapi_client.models.campaign.url_goal import UrlGoal
from openapi_client.models.campaign.value_operator import ValueOperator
from openapi_client.models.campaign.vanity_pharma_display_url_mode import VanityPharmaDisplayUrlMode
from openapi_client.models.campaign.vanity_pharma_setting import VanityPharmaSetting
from openapi_client.models.campaign.vanity_pharma_website_description import VanityPharmaWebsiteDescription
from openapi_client.models.campaign.verified_tracking_setting import VerifiedTrackingSetting
from openapi_client.models.campaign.video import Video
from openapi_client.models.campaign.video_ad_extension import VideoAdExtension
from openapi_client.models.campaign.video_asset import VideoAsset
from openapi_client.models.campaign.video_recommendation_image_data import VideoRecommendationImageData
from openapi_client.models.campaign.video_recommendation_text_data import VideoRecommendationTextData
from openapi_client.models.campaign.video_template_filter import VideoTemplateFilter
from openapi_client.models.campaign.webpage import Webpage
from openapi_client.models.campaign.webpage_condition import WebpageCondition
from openapi_client.models.campaign.webpage_condition_operand import WebpageConditionOperand
from openapi_client.models.campaign.webpage_condition_operator import WebpageConditionOperator
from openapi_client.models.campaign.webpage_parameter import WebpageParameter
