# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ActionAdExtensionActionType(str, Enum):
    UNKNOWN = 'Unknown'

    ACTNOW = 'ActNow'

    APPLYNOW = 'ApplyNow'

    BETNOW = 'BetNow'

    BIDNOW = 'BidNow'

    BOOKACAR = 'BookACar'

    BOOKHOTEL = 'BookHotel'

    BOOKNOW = 'BookNow'

    BROWSE = 'Browse'

    BUYNOW = 'BuyNow'

    CHATNOW = 'ChatNow'

    COMPARE = 'Compare'

    CONTACTUS = 'ContactUs'

    COUPON = 'Coupon'

    DONATE = 'Donate'

    DOWNLOAD = 'Download'

    EMAILNOW = 'EmailNow'

    ENROLLNOW = 'EnrollNow'

    EXPLORE = 'Explore'

    FILENOW = 'FileNow'

    FINDJOB = 'FindJob'

    FREEPLAY = 'FreePlay'

    FREEQUOTE = 'FreeQuote'

    FREETRIAL = 'FreeTrial'

    GETDEALS = 'GetDeals'

    GETOFFER = 'GetOffer'

    GETQUOTE = 'GetQuote'

    JOINNOW = 'JoinNow'

    LEARNMORE = 'LearnMore'

    LISTENNOW = 'ListenNow'

    LOGIN = 'LogIn'

    MESSAGE = 'Message'

    NEWCARS = 'NewCars'

    ORDERNOW = 'OrderNow'

    PLAYGAME = 'PlayGame'

    PLAYNOW = 'PlayNow'

    POSTJOB = 'PostJob'

    REGISTER = 'Register'

    RENTACAR = 'RentACar'

    RENTNOW = 'RentNow'

    RESERVE = 'Reserve'

    SALE = 'Sale'

    SAVENOW = 'SaveNow'

    SCHEDULE = 'Schedule'

    SEEMENU = 'SeeMenu'

    SEEMORE = 'SeeMore'

    SEEOFFER = 'SeeOffer'

    SELLNOW = 'SellNow'

    SHOPNOW = 'ShopNow'

    SHOWTIMES = 'Showtimes'

    SIGNIN = 'SignIn'

    SIGNUP = 'SignUp'

    STARTFREE = 'StartFree'

    STARTNOW = 'StartNow'

    SUBSCRIBE = 'Subscribe'

    TESTDRIVE = 'TestDrive'

    TRYNOW = 'TryNow'

    USEDCARS = 'UsedCars'

    VIEWCARS = 'ViewCars'

    VIEWNOW = 'ViewNow'

    VIEWPLANS = 'ViewPlans'

    VISITSITE = 'VisitSite'

    VOTENOW = 'VoteNow'

    WATCH = 'Watch'

    WATCHMORE = 'WatchMore'

    WATCHNOW = 'WatchNow'

    DIRECTIONS = 'Directions'

    FINDSTORE = 'FindStore'

    SWITCHNOW = 'SwitchNow'

    VISITSTORE = 'VisitStore'

    RENEWNOW = 'RenewNow'

    REORDER = 'Reorder'

    BUILDNOW = 'BuildNow'

    DEALERS = 'Dealers'

    GETDEMO = 'GetDemo'

    GETNOW = 'GetNow'

    GOTODEMO = 'GoToDemo'

    SEEDEMO = 'SeeDemo'

    SEEMODELS = 'SeeModels'

    SEEOFFERS = 'SeeOffers'

    VIEWDEMO = 'ViewDemo'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ActionAdExtensionActionType from a JSON string"""
        return cls(json.loads(json_str))


