# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class PerformanceInsightsMessageTemplateId(str, Enum):
    FLUXIMPRINCREASEACCT = 'FluxImprIncreaseAcct'

    FLUXIMPRDECREASEACCT = 'FluxImprDecreaseAcct'

    FLUXIMPRINCREASEINFINITYACCT = 'FluxImprIncreaseInfinityAcct'

    FLUXIMPRINCREASEACCTPREVDAY = 'FluxImprIncreaseAcctPrevDay'

    FLUXIMPRDECREASEACCTPREVDAY = 'FluxImprDecreaseAcctPrevDay'

    FLUXIMPRINCREASEINFINITYACCTPREVDAY = 'FluxImprIncreaseInfinityAcctPrevDay'

    FLUXCLICKINCREASEACCT = 'FluxClickIncreaseAcct'

    FLUXCLICKDECREASEACCT = 'FluxClickDecreaseAcct'

    FLUXCLICKINCREASEINFINITYACCT = 'FluxClickIncreaseInfinityAcct'

    FLUXCLICKINCREASEACCTPREVDAY = 'FluxClickIncreaseAcctPrevDay'

    FLUXCLICKDECREASEACCTPREVDAY = 'FluxClickDecreaseAcctPrevDay'

    FLUXCLICKINCREASEINFINITYACCTPREVDAY = 'FluxClickIncreaseInfinityAcctPrevDay'

    FLUXCOSTINCREASEACCT = 'FluxCostIncreaseAcct'

    FLUXCOSTDECREASEACCT = 'FluxCostDecreaseAcct'

    FLUXCOSTINCREASEINFINITYACCT = 'FluxCostIncreaseInfinityAcct'

    FLUXCOSTINCREASEACCTPREVDAY = 'FluxCostIncreaseAcctPrevDay'

    FLUXCOSTDECREASEACCTPREVDAY = 'FluxCostDecreaseAcctPrevDay'

    FLUXCOSTINCREASEINFINITYACCTPREVDAY = 'FluxCostIncreaseInfinityAcctPrevDay'

    FLUXIMPRINCREASECAMPAIGN = 'FluxImprIncreaseCampaign'

    FLUXIMPRDECREASECAMPAIGN = 'FluxImprDecreaseCampaign'

    FLUXIMPRINCREASEINFINITYCAMPAIGN = 'FluxImprIncreaseInfinityCampaign'

    FLUXIMPRINCREASECAMPAIGNPREVDAY = 'FluxImprIncreaseCampaignPrevDay'

    FLUXIMPRDECREASECAMPAIGNPREVDAY = 'FluxImprDecreaseCampaignPrevDay'

    FLUXIMPRINCREASEINFINITYCAMPAIGNPREVDAY = 'FluxImprIncreaseInfinityCampaignPrevDay'

    FLUXCLICKINCREASECAMPAIGN = 'FluxClickIncreaseCampaign'

    FLUXCLICKDECREASECAMPAIGN = 'FluxClickDecreaseCampaign'

    FLUXCLICKINCREASEINFINITYCAMPAIGN = 'FluxClickIncreaseInfinityCampaign'

    FLUXCLICKINCREASECAMPAIGNPREVDAY = 'FluxClickIncreaseCampaignPrevDay'

    FLUXCLICKDECREASECAMPAIGNPREVDAY = 'FluxClickDecreaseCampaignPrevDay'

    FLUXCLICKINCREASEINFINITYCAMPAIGNPREVDAY = 'FluxClickIncreaseInfinityCampaignPrevDay'

    FLUXCOSTINCREASECAMPAIGN = 'FluxCostIncreaseCampaign'

    FLUXCOSTDECREASECAMPAIGN = 'FluxCostDecreaseCampaign'

    FLUXCOSTINCREASEINFINITYCAMPAIGN = 'FluxCostIncreaseInfinityCampaign'

    FLUXCOSTINCREASECAMPAIGNPREVDAY = 'FluxCostIncreaseCampaignPrevDay'

    FLUXCOSTDECREASECAMPAIGNPREVDAY = 'FluxCostDecreaseCampaignPrevDay'

    FLUXCOSTINCREASEINFINITYCAMPAIGNPREVDAY = 'FluxCostIncreaseInfinityCampaignPrevDay'

    FLUXCONVERSIONDECREASEACCT = 'FluxConversionDecreaseAcct'

    FLUXCONVERSIONZEROACCT = 'FluxConversionZeroAcct'

    RCAUDIENCEINCREASE = 'RCAudienceIncrease'

    RCAUDIENCEDECREASE = 'RCAudienceDecrease'

    RCBIDINCREASE = 'RCBidIncrease'

    RCBIDDECREASE = 'RCBidDecrease'

    RCBUDGETINCREASE = 'RCBudgetIncrease'

    RCBUDGETDECREASE = 'RCBudgetDecrease'

    RCCLICKINCREASE = 'RCClickIncrease'

    RCCLICKDECREASE = 'RCClickDecrease'

    RCCONVERSIONGOALISSUE = 'RCConversionGoalIssue'

    RCDEVICETARGETINGBIDCHANGE = 'RCDeviceTargetingBidChange'

    RCEDITORIALAPPROVAL = 'RCEditorialApproval'

    RCEDITORIALDISAPPROVAL = 'RCEditorialDisapproval'

    RCENTITYADDITIONWITHCOUNT = 'RCEntityAdditionWithCount'

    RCENTITYDELETIONWITHCOUNT = 'RCEntityDeletionWithCount'

    RCENTITYPAUSEWITHCOUNT = 'RCEntityPauseWithCount'

    RCENTITYUNPAUSEWITHCOUNT = 'RCEntityUnpauseWithCount'

    RCMORECOMPETITIVE = 'RCMoreCompetitive'

    RCNETWORKLESSRESTRICT = 'RCNetworkLessRestrict'

    RCNETWORKMORERESTRICT = 'RCNetworkMoreRestrict'

    RCPAYMENTISSUEINVOICEACCOUNTPAUSED = 'RCPaymentIssueInvoiceAccountPaused'

    RCPAYMENTISSUEINVOICEACCOUNTONHOLD = 'RCPaymentIssueInvoiceAccountOnHold'

    RCPAYMENTISSUEPREPAYACCOUNTPAUSED = 'RCPaymentIssuePrepayAccountPaused'

    RCPAYMENTISSUETHRESHOLDACCOUNTONHOLD = 'RCPaymentIssueThresholdAccountOnHold'

    RCPAYMENTISSUEYESTERDAY = 'RCPaymentIssueYesterday'

    RCSEARCHTERMINCREASE = 'RCSearchTermIncrease'

    RCSEARCHTERMINCREASECLICK = 'RCSearchTermIncreaseClick'

    RCSEARCHTERMDECREASE = 'RCSearchTermDecrease'

    RCSEARCHTERMDECREASECLICK = 'RCSearchTermDecreaseClick'

    RCSEARCHTERMCOMPETITIONINCREASE = 'RCSearchTermCompetitionIncrease'

    RCSEARCHTERMCOMPETITIONINCREASECLICK = 'RCSearchTermCompetitionIncreaseClick'

    RCSEARCHTERMCOMPETITIONDECREASE = 'RCSearchTermCompetitionDecrease'

    RCSEARCHTERMCOMPETITIONDECREASECLICK = 'RCSearchTermCompetitionDecreaseClick'

    RCSINGLECAMPAIGNADDITION = 'RCSingleCampaignAddition'

    RCSINGLECAMPAIGNPAUSE = 'RCSingleCampaignPause'

    RCSINGLECAMPAIGNUNPAUSE = 'RCSingleCampaignUnpause'

    RCSINGLECAMPAIGNBUDGETINCREASE = 'RCSingleCampaignBudgetIncrease'

    RCSINGLECAMPAIGNBUDGETDECREASE = 'RCSingleCampaignBudgetDecrease'

    RCSINGLECAMPAIGNDEVICETARGETINGBIDCHANGE = 'RCSingleCampaignDeviceTargetingBidChange'

    RCSTRONGERCOMPETITORADQUALITY = 'RCStrongerCompetitorAdQuality'

    RCSTRONGERCOMPETITORBID = 'RCStrongerCompetitorBid'

    RCSYNDTRAFFICINCREASE = 'RCSyndTrafficIncrease'

    RCSYNDTRAFFICDECREASE = 'RCSyndTrafficDecrease'

    ACTADSOPPR = 'ActAdsOppr'

    ACTALLAVAILABLEOPPR = 'ActAllAvailableOppr'

    ACTBMADOPTIONOPPR = 'ActBMAdoptionOppr'

    ACTBMMKEYWORDOPPR = 'ActBMMKeywordOppr'

    ACTCAMPAIGNCONTEXTKEYWORDOPPR = 'ActCampaignContextKeywordOppr'

    ACTCHECKPUBLISHERWEBSITE = 'ActCheckPublisherWebsite'

    ACTCOMPETITIONAUCTIONINSIGHT = 'ActCompetitionAuctionInsight'

    ACTCOMPETITIVEBUDGETOPPR = 'ActCompetitiveBudgetOppr'

    ACTCOMPETITIVEKEYWORDOPPR = 'ActCompetitiveKeywordOppr'

    ACTCOMPETITIVELOCATIONTARGETOPPR = 'ActCompetitiveLocationTargetOppr'

    ACTCTRQUALITYBUNDLE = 'ActCTRQualityBundle'

    ACTEDITORIALREVIEWPROCESS = 'ActEditorialReviewProcess'

    ACTFIXCONVERSIONGOALSETTINGOPPR = 'ActFixConversionGoalSettingOppr'

    ACTFIXCONVERSIONTRACKINGOPPR = 'ActFixConversionTrackingOppr'

    ACTGENERALBUDGETOPPR = 'ActGeneralBudgetOppr'

    ACTNEWCONVERSIONGOALOPPR = 'ActNewConversionGoalOppr'

    ACTINVOICEACCOUNTPAUSED = 'ActInvoiceAccountPaused'

    ACTINVOICEACCOUNTONHOLD = 'ActInvoiceAccountOnHold'

    ACTPREPAYACCOUNTPAUSED = 'ActPrepayAccountPaused'

    ACTREALLOCATEBUDGETOPPR = 'ActReallocateBudgetOppr'

    ACTREPAIRKEYWORDSOPPR = 'ActRepairKeywordsOppr'

    ACTRSAOPPR = 'ActRSAOppr'

    ACTSEARCHTERM = 'ActSearchTerm'

    ACTSETUPCONVERSIONTRACKINGOPPR = 'ActSetupConversionTrackingOppr'

    ACTSITELINKOPPR = 'ActSiteLinkOppr'

    ACTTHRESHOLDACCOUNTONHOLD = 'ActThresholdAccountOnHold'

    ACTWEBSITEEXCLUSION = 'ActWebsiteExclusion'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PerformanceInsightsMessageTemplateId from a JSON string"""
        return cls(json.loads(json_str))


