# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AdPosition(str, Enum):
    ALL = 'All'

    MAINLINE1 = 'MainLine1'

    MAINLINE2 = 'MainLine2'

    MAINLINE3 = 'MainLine3'

    MAINLINE4 = 'MainLine4'

    FIRSTPAGE1 = 'FirstPage1'

    FIRSTPAGE2 = 'FirstPage2'

    FIRSTPAGE3 = 'FirstPage3'

    FIRSTPAGE4 = 'FirstPage4'

    FIRSTPAGE5 = 'FirstPage5'

    FIRSTPAGE6 = 'FirstPage6'

    FIRSTPAGE7 = 'FirstPage7'

    FIRSTPAGE8 = 'FirstPage8'

    FIRSTPAGE9 = 'FirstPage9'

    FIRSTPAGE10 = 'FirstPage10'

    AGGREGATE = 'Aggregate'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AdPosition from a JSON string"""
        return cls(json.loads(json_str))


