# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AuctionSegment(str, Enum):
    DAY = 'Day'

    WEEK = 'Week'

    MONTH = 'Month'

    QUARTER = 'Quarter'

    DAYOFWEEK = 'DayOfWeek'

    DEVICE = 'Device'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AuctionSegment from a JSON string"""
        return cls(json.loads(json_str))


