# coding: utf-8

# flake8: noqa
"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.reporting.account_performance_report_column import AccountPerformanceReportColumn
from openapi_client.models.reporting.account_performance_report_filter import AccountPerformanceReportFilter
from openapi_client.models.reporting.account_performance_report_request import AccountPerformanceReportRequest
from openapi_client.models.reporting.account_report_scope import AccountReportScope
from openapi_client.models.reporting.account_status_report_filter import AccountStatusReportFilter
from openapi_client.models.reporting.account_through_ad_group_report_scope import AccountThroughAdGroupReportScope
from openapi_client.models.reporting.account_through_asset_group_report_scope import AccountThroughAssetGroupReportScope
from openapi_client.models.reporting.account_through_campaign_report_scope import AccountThroughCampaignReportScope
from openapi_client.models.reporting.ad_api_error import AdApiError
from openapi_client.models.reporting.ad_api_fault_detail import AdApiFaultDetail
from openapi_client.models.reporting.ad_distribution_report_filter import AdDistributionReportFilter
from openapi_client.models.reporting.ad_dynamic_text_performance_report_column import AdDynamicTextPerformanceReportColumn
from openapi_client.models.reporting.ad_dynamic_text_performance_report_filter import AdDynamicTextPerformanceReportFilter
from openapi_client.models.reporting.ad_dynamic_text_performance_report_request import AdDynamicTextPerformanceReportRequest
from openapi_client.models.reporting.ad_extension_by_ad_report_column import AdExtensionByAdReportColumn
from openapi_client.models.reporting.ad_extension_by_ad_report_filter import AdExtensionByAdReportFilter
from openapi_client.models.reporting.ad_extension_by_ad_report_request import AdExtensionByAdReportRequest
from openapi_client.models.reporting.ad_extension_by_keyword_report_column import AdExtensionByKeywordReportColumn
from openapi_client.models.reporting.ad_extension_by_keyword_report_filter import AdExtensionByKeywordReportFilter
from openapi_client.models.reporting.ad_extension_by_keyword_report_request import AdExtensionByKeywordReportRequest
from openapi_client.models.reporting.ad_extension_detail_report_column import AdExtensionDetailReportColumn
from openapi_client.models.reporting.ad_extension_detail_report_filter import AdExtensionDetailReportFilter
from openapi_client.models.reporting.ad_extension_detail_report_request import AdExtensionDetailReportRequest
from openapi_client.models.reporting.ad_group_performance_report_column import AdGroupPerformanceReportColumn
from openapi_client.models.reporting.ad_group_performance_report_filter import AdGroupPerformanceReportFilter
from openapi_client.models.reporting.ad_group_performance_report_request import AdGroupPerformanceReportRequest
from openapi_client.models.reporting.ad_group_report_scope import AdGroupReportScope
from openapi_client.models.reporting.ad_group_status_report_filter import AdGroupStatusReportFilter
from openapi_client.models.reporting.ad_performance_report_column import AdPerformanceReportColumn
from openapi_client.models.reporting.ad_performance_report_filter import AdPerformanceReportFilter
from openapi_client.models.reporting.ad_performance_report_request import AdPerformanceReportRequest
from openapi_client.models.reporting.ad_status_report_filter import AdStatusReportFilter
from openapi_client.models.reporting.ad_type_report_filter import AdTypeReportFilter
from openapi_client.models.reporting.age_gender_audience_report_column import AgeGenderAudienceReportColumn
from openapi_client.models.reporting.age_gender_audience_report_filter import AgeGenderAudienceReportFilter
from openapi_client.models.reporting.age_gender_audience_report_request import AgeGenderAudienceReportRequest
from openapi_client.models.reporting.api_fault_detail import ApiFaultDetail
from openapi_client.models.reporting.application_fault import ApplicationFault
from openapi_client.models.reporting.apps_performance_report_column import AppsPerformanceReportColumn
from openapi_client.models.reporting.apps_performance_report_filter import AppsPerformanceReportFilter
from openapi_client.models.reporting.apps_performance_report_request import AppsPerformanceReportRequest
from openapi_client.models.reporting.asset_group_performance_report_column import AssetGroupPerformanceReportColumn
from openapi_client.models.reporting.asset_group_performance_report_filter import AssetGroupPerformanceReportFilter
from openapi_client.models.reporting.asset_group_performance_report_request import AssetGroupPerformanceReportRequest
from openapi_client.models.reporting.asset_group_report_scope import AssetGroupReportScope
from openapi_client.models.reporting.asset_group_status_report_filter import AssetGroupStatusReportFilter
from openapi_client.models.reporting.asset_performance_report_column import AssetPerformanceReportColumn
from openapi_client.models.reporting.asset_performance_report_request import AssetPerformanceReportRequest
from openapi_client.models.reporting.audience_performance_report_column import AudiencePerformanceReportColumn
from openapi_client.models.reporting.audience_performance_report_filter import AudiencePerformanceReportFilter
from openapi_client.models.reporting.audience_performance_report_request import AudiencePerformanceReportRequest
from openapi_client.models.reporting.batch_error import BatchError
from openapi_client.models.reporting.bid_match_type_report_filter import BidMatchTypeReportFilter
from openapi_client.models.reporting.bid_strategy_type_report_filter import BidStrategyTypeReportFilter
from openapi_client.models.reporting.budget_summary_report_column import BudgetSummaryReportColumn
from openapi_client.models.reporting.budget_summary_report_request import BudgetSummaryReportRequest
from openapi_client.models.reporting.call_detail_report_column import CallDetailReportColumn
from openapi_client.models.reporting.call_detail_report_filter import CallDetailReportFilter
from openapi_client.models.reporting.call_detail_report_request import CallDetailReportRequest
from openapi_client.models.reporting.campaign_performance_report_column import CampaignPerformanceReportColumn
from openapi_client.models.reporting.campaign_performance_report_filter import CampaignPerformanceReportFilter
from openapi_client.models.reporting.campaign_performance_report_request import CampaignPerformanceReportRequest
from openapi_client.models.reporting.campaign_report_scope import CampaignReportScope
from openapi_client.models.reporting.campaign_status_report_filter import CampaignStatusReportFilter
from openapi_client.models.reporting.campaign_type_report_filter import CampaignTypeReportFilter
from openapi_client.models.reporting.category_click_coverage_report_column import CategoryClickCoverageReportColumn
from openapi_client.models.reporting.category_click_coverage_report_filter import CategoryClickCoverageReportFilter
from openapi_client.models.reporting.category_click_coverage_report_request import CategoryClickCoverageReportRequest
from openapi_client.models.reporting.category_insights_report_column import CategoryInsightsReportColumn
from openapi_client.models.reporting.category_insights_report_filter import CategoryInsightsReportFilter
from openapi_client.models.reporting.category_insights_report_request import CategoryInsightsReportRequest
from openapi_client.models.reporting.change_entity_report_filter import ChangeEntityReportFilter
from openapi_client.models.reporting.change_type_report_filter import ChangeTypeReportFilter
from openapi_client.models.reporting.combination_performance_report_column import CombinationPerformanceReportColumn
from openapi_client.models.reporting.combination_performance_report_request import CombinationPerformanceReportRequest
from openapi_client.models.reporting.conversion_performance_report_column import ConversionPerformanceReportColumn
from openapi_client.models.reporting.conversion_performance_report_filter import ConversionPerformanceReportFilter
from openapi_client.models.reporting.conversion_performance_report_request import ConversionPerformanceReportRequest
from openapi_client.models.reporting.dsa_auto_target_performance_report_column import DSAAutoTargetPerformanceReportColumn
from openapi_client.models.reporting.dsa_auto_target_performance_report_filter import DSAAutoTargetPerformanceReportFilter
from openapi_client.models.reporting.dsa_auto_target_performance_report_request import DSAAutoTargetPerformanceReportRequest
from openapi_client.models.reporting.dsa_category_performance_report_column import DSACategoryPerformanceReportColumn
from openapi_client.models.reporting.dsa_category_performance_report_filter import DSACategoryPerformanceReportFilter
from openapi_client.models.reporting.dsa_category_performance_report_request import DSACategoryPerformanceReportRequest
from openapi_client.models.reporting.dsa_search_query_performance_report_column import DSASearchQueryPerformanceReportColumn
from openapi_client.models.reporting.dsa_search_query_performance_report_filter import DSASearchQueryPerformanceReportFilter
from openapi_client.models.reporting.dsa_search_query_performance_report_request import DSASearchQueryPerformanceReportRequest
from openapi_client.models.reporting.delivered_match_type_report_filter import DeliveredMatchTypeReportFilter
from openapi_client.models.reporting.destination_url_performance_report_column import DestinationUrlPerformanceReportColumn
from openapi_client.models.reporting.destination_url_performance_report_filter import DestinationUrlPerformanceReportFilter
from openapi_client.models.reporting.destination_url_performance_report_request import DestinationUrlPerformanceReportRequest
from openapi_client.models.reporting.device_os_report_filter import DeviceOSReportFilter
from openapi_client.models.reporting.device_type_report_filter import DeviceTypeReportFilter
from openapi_client.models.reporting.dynamic_ad_target_status_report_filter import DynamicAdTargetStatusReportFilter
from openapi_client.models.reporting.feed_item_performance_report_column import FeedItemPerformanceReportColumn
from openapi_client.models.reporting.feed_item_performance_report_request import FeedItemPerformanceReportRequest
from openapi_client.models.reporting.geographic_performance_report_column import GeographicPerformanceReportColumn
from openapi_client.models.reporting.geographic_performance_report_filter import GeographicPerformanceReportFilter
from openapi_client.models.reporting.geographic_performance_report_request import GeographicPerformanceReportRequest
from openapi_client.models.reporting.goals_and_funnels_report_column import GoalsAndFunnelsReportColumn
from openapi_client.models.reporting.goals_and_funnels_report_filter import GoalsAndFunnelsReportFilter
from openapi_client.models.reporting.goals_and_funnels_report_request import GoalsAndFunnelsReportRequest
from openapi_client.models.reporting.hotel_dimension_performance_report_column import HotelDimensionPerformanceReportColumn
from openapi_client.models.reporting.hotel_dimension_performance_report_filter import HotelDimensionPerformanceReportFilter
from openapi_client.models.reporting.hotel_dimension_performance_report_request import HotelDimensionPerformanceReportRequest
from openapi_client.models.reporting.hotel_group_performance_report_column import HotelGroupPerformanceReportColumn
from openapi_client.models.reporting.hotel_group_performance_report_filter import HotelGroupPerformanceReportFilter
from openapi_client.models.reporting.hotel_group_performance_report_request import HotelGroupPerformanceReportRequest
from openapi_client.models.reporting.keyword_performance_report_column import KeywordPerformanceReportColumn
from openapi_client.models.reporting.keyword_performance_report_filter import KeywordPerformanceReportFilter
from openapi_client.models.reporting.keyword_performance_report_request import KeywordPerformanceReportRequest
from openapi_client.models.reporting.keyword_performance_report_sort import KeywordPerformanceReportSort
from openapi_client.models.reporting.keyword_status_report_filter import KeywordStatusReportFilter
from openapi_client.models.reporting.language_report_filter import LanguageReportFilter
from openapi_client.models.reporting.model_date import ModelDate
from openapi_client.models.reporting.negative_keyword_conflict_report_column import NegativeKeywordConflictReportColumn
from openapi_client.models.reporting.negative_keyword_conflict_report_filter import NegativeKeywordConflictReportFilter
from openapi_client.models.reporting.negative_keyword_conflict_report_request import NegativeKeywordConflictReportRequest
from openapi_client.models.reporting.operation_error import OperationError
from openapi_client.models.reporting.poll_generate_report_request import PollGenerateReportRequest
from openapi_client.models.reporting.poll_generate_report_response import PollGenerateReportResponse
from openapi_client.models.reporting.product_dimension_performance_report_column import ProductDimensionPerformanceReportColumn
from openapi_client.models.reporting.product_dimension_performance_report_filter import ProductDimensionPerformanceReportFilter
from openapi_client.models.reporting.product_dimension_performance_report_request import ProductDimensionPerformanceReportRequest
from openapi_client.models.reporting.product_match_count_report_column import ProductMatchCountReportColumn
from openapi_client.models.reporting.product_match_count_report_request import ProductMatchCountReportRequest
from openapi_client.models.reporting.product_negative_keyword_conflict_report_column import ProductNegativeKeywordConflictReportColumn
from openapi_client.models.reporting.product_negative_keyword_conflict_report_filter import ProductNegativeKeywordConflictReportFilter
from openapi_client.models.reporting.product_negative_keyword_conflict_report_request import ProductNegativeKeywordConflictReportRequest
from openapi_client.models.reporting.product_partition_performance_report_column import ProductPartitionPerformanceReportColumn
from openapi_client.models.reporting.product_partition_performance_report_filter import ProductPartitionPerformanceReportFilter
from openapi_client.models.reporting.product_partition_performance_report_request import ProductPartitionPerformanceReportRequest
from openapi_client.models.reporting.product_partition_unit_performance_report_column import ProductPartitionUnitPerformanceReportColumn
from openapi_client.models.reporting.product_partition_unit_performance_report_filter import ProductPartitionUnitPerformanceReportFilter
from openapi_client.models.reporting.product_partition_unit_performance_report_request import ProductPartitionUnitPerformanceReportRequest
from openapi_client.models.reporting.product_search_query_performance_report_column import ProductSearchQueryPerformanceReportColumn
from openapi_client.models.reporting.product_search_query_performance_report_filter import ProductSearchQueryPerformanceReportFilter
from openapi_client.models.reporting.product_search_query_performance_report_request import ProductSearchQueryPerformanceReportRequest
from openapi_client.models.reporting.professional_demographics_audience_report_column import ProfessionalDemographicsAudienceReportColumn
from openapi_client.models.reporting.professional_demographics_audience_report_filter import ProfessionalDemographicsAudienceReportFilter
from openapi_client.models.reporting.professional_demographics_audience_report_request import ProfessionalDemographicsAudienceReportRequest
from openapi_client.models.reporting.publisher_usage_performance_report_column import PublisherUsagePerformanceReportColumn
from openapi_client.models.reporting.publisher_usage_performance_report_filter import PublisherUsagePerformanceReportFilter
from openapi_client.models.reporting.publisher_usage_performance_report_request import PublisherUsagePerformanceReportRequest
from openapi_client.models.reporting.report_aggregation import ReportAggregation
from openapi_client.models.reporting.report_format import ReportFormat
from openapi_client.models.reporting.report_request import ReportRequest
from openapi_client.models.reporting.report_request_status import ReportRequestStatus
from openapi_client.models.reporting.report_request_status_type import ReportRequestStatusType
from openapi_client.models.reporting.report_time import ReportTime
from openapi_client.models.reporting.report_time_period import ReportTimePeriod
from openapi_client.models.reporting.report_time_zone import ReportTimeZone
from openapi_client.models.reporting.search_campaign_change_history_report_column import SearchCampaignChangeHistoryReportColumn
from openapi_client.models.reporting.search_campaign_change_history_report_filter import SearchCampaignChangeHistoryReportFilter
from openapi_client.models.reporting.search_campaign_change_history_report_request import SearchCampaignChangeHistoryReportRequest
from openapi_client.models.reporting.search_insight_performance_report_column import SearchInsightPerformanceReportColumn
from openapi_client.models.reporting.search_insight_performance_report_filter import SearchInsightPerformanceReportFilter
from openapi_client.models.reporting.search_insight_performance_report_request import SearchInsightPerformanceReportRequest
from openapi_client.models.reporting.search_query_performance_report_column import SearchQueryPerformanceReportColumn
from openapi_client.models.reporting.search_query_performance_report_filter import SearchQueryPerformanceReportFilter
from openapi_client.models.reporting.search_query_performance_report_request import SearchQueryPerformanceReportRequest
from openapi_client.models.reporting.share_of_voice_report_column import ShareOfVoiceReportColumn
from openapi_client.models.reporting.share_of_voice_report_filter import ShareOfVoiceReportFilter
from openapi_client.models.reporting.share_of_voice_report_request import ShareOfVoiceReportRequest
from openapi_client.models.reporting.sort_order import SortOrder
from openapi_client.models.reporting.submit_generate_report_request import SubmitGenerateReportRequest
from openapi_client.models.reporting.submit_generate_report_response import SubmitGenerateReportResponse
from openapi_client.models.reporting.travel_query_insight_report_column import TravelQueryInsightReportColumn
from openapi_client.models.reporting.travel_query_insight_report_request import TravelQueryInsightReportRequest
from openapi_client.models.reporting.user_location_performance_report_column import UserLocationPerformanceReportColumn
from openapi_client.models.reporting.user_location_performance_report_filter import UserLocationPerformanceReportFilter
from openapi_client.models.reporting.user_location_performance_report_request import UserLocationPerformanceReportRequest
