# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AgeGenderAudienceReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    TIMEPERIOD = 'TimePeriod'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    ADDISTRIBUTION = 'AdDistribution'

    AGEGROUP = 'AgeGroup'

    GENDER = 'Gender'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CONVERSIONS = 'Conversions'

    SPEND = 'Spend'

    REVENUE = 'Revenue'

    EXTENDEDCOST = 'ExtendedCost'

    ASSISTS = 'Assists'

    LANGUAGE = 'Language'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

    BASECAMPAIGNID = 'BaseCampaignId'

    ALLCONVERSIONS = 'AllConversions'

    ALLREVENUE = 'AllRevenue'

    VIEWTHROUGHCONVERSIONS = 'ViewThroughConversions'

    GOAL = 'Goal'

    GOALTYPE = 'GoalType'

    ABSOLUTETOPIMPRESSIONRATEPERCENT = 'AbsoluteTopImpressionRatePercent'

    TOPIMPRESSIONRATEPERCENT = 'TopImpressionRatePercent'

    CONVERSIONSQUALIFIED = 'ConversionsQualified'

    ALLCONVERSIONSQUALIFIED = 'AllConversionsQualified'

    VIEWTHROUGHCONVERSIONSQUALIFIED = 'ViewThroughConversionsQualified'

    VIEWTHROUGHREVENUE = 'ViewThroughRevenue'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AgeGenderAudienceReportColumn from a JSON string"""
        return cls(json.loads(json_str))


