# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AccountPropertyName(str, Enum):
    NONE = 'None'

    TRACKINGURLTEMPLATE = 'TrackingUrlTemplate'

    MSCLKIDAUTOTAGGINGENABLED = 'MSCLKIDAutoTaggingEnabled'

    ADCLICKPARALLELTRACKING = 'AdClickParallelTracking'

    FINALURLSUFFIX = 'FinalUrlSuffix'

    INCLUDEVIEWTHROUGHCONVERSIONS = 'IncludeViewThroughConversions'

    PROFILEEXPANSIONENABLED = 'ProfileExpansionEnabled'

    ALLOWIMAGEAUTORETRIEVE = 'AllowImageAutoRetrieve'

    AUTOAPPLYRECOMMENDATIONS = 'AutoApplyRecommendations'

    INCLUDEAUTOBIDDINGVIEWTHROUGHCONVERSIONS = 'IncludeAutoBiddingViewThroughConversions'

    AUTOBIDDINGVIEWTHROUGHCONVERSIONSVALUEATTRIBUTIONWEIGHT = 'AutoBiddingViewThroughConversionsValueAttributionWeight'

    LOOPBACKWINDOWFORVIEWTHROUGHCONVERSIONS = 'LoopBackWindowForViewThroughConversions'

    BUSINESSATTRIBUTES = 'BusinessAttributes'

    ENABLEMMAUNDERDSAADGROUPS = 'EnableMMAUnderDSAAdgroups'

    OPTOUTFROMMCM = 'OptOutFromMCM'

    NETFLIXTCACCEPTED = 'NetflixTCAccepted'

    BLOCKEDCONTENTSEGMENTS = 'BlockedContentSegments'

    ASSETAIENHANCEMENTOPTOUT = 'AssetAIEnhancementOptout'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AccountPropertyName from a JSON string"""
        return cls(json.loads(json_str))


