# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ClientLinkStatus(str, Enum):
    LINKPENDING = 'LinkPending'

    LINKCANCELED = 'LinkCanceled'

    LINKEXPIRED = 'LinkExpired'

    LINKACCEPTED = 'LinkAccepted'

    LINKDECLINED = 'LinkDeclined'

    LINKINPROGRESS = 'LinkInProgress'

    ACTIVE = 'Active'

    LINKFAILED = 'LinkFailed'

    UNLINKREQUESTED = 'UnlinkRequested'

    UNLINKPENDING = 'UnlinkPending'

    UNLINKCANCELED = 'UnlinkCanceled'

    UNLINKINPROGRESS = 'UnlinkInProgress'

    INACTIVE = 'Inactive'

    UNLINKFAILED = 'UnlinkFailed'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ClientLinkStatus from a JSON string"""
        return cls(json.loads(json_str))


