# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ReportTimeZone(str, Enum):
    NUKUALOFA = 'Nukualofa'

    FIJIKAMCHATKAMARSHALLISLAND = 'FijiKamchatkaMarshallIsland'

    AUCKLANDWELLINGTON = 'AucklandWellington'

    MAGADANSOLOMONISLANDNEWCALEDONIA = 'MagadanSolomonIslandNewCaledonia'

    VLADIVOSTOK = 'Vladivostok'

    HOBART = 'Hobart'

    GUAMPORTMORESBY = 'GuamPortMoresby'

    CANBERRAMELBOURNESYDNEY = 'CanberraMelbourneSydney'

    BRISBANE = 'Brisbane'

    DARWIN = 'Darwin'

    ADELAIDE = 'Adelaide'

    YAKUTSK = 'Yakutsk'

    SEOUL = 'Seoul'

    OSAKASAPPOROTOKYO = 'OsakaSapporoTokyo'

    TAIPEI = 'Taipei'

    PERTH = 'Perth'

    KUALALUMPURSINGAPORE = 'KualaLumpurSingapore'

    IRKUTSKULAANBATAAR = 'IrkutskUlaanBataar'

    BEIJINGCHONGQINGHONGKONGURUMQI = 'BeijingChongqingHongKongUrumqi'

    KRASNOYARSK = 'Krasnoyarsk'

    BANGKOKHANOIJAKARTA = 'BangkokHanoiJakarta'

    RANGOON = 'Rangoon'

    SRIJAYAWARDENEPURA = 'SriJayawardenepura'

    ASTANADHAKA = 'AstanaDhaka'

    ALMATYNOVOSIBIRSK = 'AlmatyNovosibirsk'

    KATHMANDU = 'Kathmandu'

    CHENNAIKOLKATAMUMBAINEWDELHI = 'ChennaiKolkataMumbaiNewDelhi'

    ISLANDAMABADKARACHITASHKENT = 'IslandamabadKarachiTashkent'

    EKATERINBURG = 'Ekaterinburg'

    KABUL = 'Kabul'

    BAKUTBILISIYEREVAN = 'BakuTbilisiYerevan'

    ABUDHABIMUSCAT = 'AbuDhabiMuscat'

    TEHRAN = 'Tehran'

    NAIROBI = 'Nairobi'

    MOSCOWSTPETERSBURGVOLGOGRAD = 'MoscowStPetersburgVolgograd'

    KUWAITRIYADH = 'KuwaitRiyadh'

    BAGHDAD = 'Baghdad'

    JERUSALEM = 'Jerusalem'

    HELSINKIKYIVRIGASOFIATALLINNVILNIUS = 'HelsinkiKyivRigaSofiaTallinnVilnius'

    HARAREPRETORIA = 'HararePretoria'

    CAIRO = 'Cairo'

    BUCHAREST = 'Bucharest'

    ATHENSISLANDANBULMINSK = 'AthensIslandanbulMinsk'

    WESTCENTRALAFRICA = 'WestCentralAfrica'

    SARAJEVOSKOPJEWARSAWZAGREB = 'SarajevoSkopjeWarsawZagreb'

    BRUSSELSCOPENHAGENMADRIDPARIS = 'BrusselsCopenhagenMadridParis'

    BELGRADEBRATISLAVABUDAPESTLJUBLJANAPRAGUE = 'BelgradeBratislavaBudapestLjubljanaPrague'

    AMSTERDAMBERLINBERNROMESTOCKHOLMVIENNA = 'AmsterdamBerlinBernRomeStockholmVienna'

    CASABLANCAMONROVIA = 'CasablancaMonrovia'

    GREENWICHMEANTIMEDUBLINEDINBURGHLISBONLONDON = 'GreenwichMeanTimeDublinEdinburghLisbonLondon'

    AZORES = 'Azores'

    CAPEVERDEISLAND = 'CapeVerdeIsland'

    MIDATLANTIC = 'MidAtlantic'

    BRASILIA = 'Brasilia'

    BUENOSAIRESGEORGETOWN = 'BuenosAiresGeorgetown'

    GREENLAND = 'Greenland'

    NEWFOUNDLAND = 'Newfoundland'

    ATLANTICTIMECANADA = 'AtlanticTimeCanada'

    CARACASLAPAZ = 'CaracasLaPaz'

    SANTIAGO = 'Santiago'

    BOGOTALIMAQUITO = 'BogotaLimaQuito'

    EASTERNTIMEUSCANADA = 'EasternTimeUSCanada'

    INDIANAEAST = 'IndianaEast'

    CENTRALAMERICA = 'CentralAmerica'

    CENTRALTIMEUSCANADA = 'CentralTimeUSCanada'

    GUADALAJARAMEXICOCITYMONTERREY = 'GuadalajaraMexicoCityMonterrey'

    SASKATCHEWAN = 'Saskatchewan'

    ARIZONA = 'Arizona'

    CHIHUAHUALAPAZMAZATLAN = 'ChihuahuaLaPazMazatlan'

    MOUNTAINTIMEUSCANADA = 'MountainTimeUSCanada'

    PACIFICTIMEUSCANADATIJUANA = 'PacificTimeUSCanadaTijuana'

    ALASKA = 'Alaska'

    HAWAII = 'Hawaii'

    MIDWAYISLANDANDSAMOA = 'MidwayIslandandSamoa'

    INTERNATIONALDATELINEWEST = 'InternationalDateLineWest'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ReportTimeZone from a JSON string"""
        return cls(json.loads(json_str))


