from bingads.v13.bulk.entities.audiences.bulk_ad_group_negative_audience_association import *

class BulkAdGroupNegativeProductAudienceAssociation(BulkAdGroupNegativeAudienceAssociation):
    """ Represents an Ad Group Negative Product Audience Association that can be read or written in a bulk file.

    For more information, see Ad Group Negative Product Audience Association at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """
