# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AdExtensionHeaderType(str, Enum):
    UNKNOWN = 'Unknown'

    AMENITIES = 'Amenities'

    BRANDS = 'Brands'

    CLASSES = 'Classes'

    COURSES = 'Courses'

    DAILYRATES = 'DailyRates'

    DEGREEPROGRAMS = 'DegreePrograms'

    DEPARTMENTS = 'Departments'

    DESTINATIONS = 'Destinations'

    FEATUREDHOTELS = 'FeaturedHotels'

    GOODS = 'Goods'

    GRADES = 'Grades'

    HIGHLIGHTS = 'Highlights'

    INSURANCECOVERAGE = 'InsuranceCoverage'

    ITEMS = 'Items'

    LANGUAGES = 'Languages'

    LOCATIONS = 'Locations'

    MODELS = 'Models'

    NEIGHBORHOODS = 'Neighborhoods'

    PRICES = 'Prices'

    RATES = 'Rates'

    RATINGS = 'Ratings'

    SCHOOLDISTRICTS = 'SchoolDistricts'

    SERVICES = 'Services'

    SERVICECATALOG = 'ServiceCatalog'

    SHOWS = 'Shows'

    SIZES = 'Sizes'

    STYLES = 'Styles'

    TOOLS = 'Tools'

    TOPICS = 'Topics'

    TYPES = 'Types'

    VACATIONS = 'Vacations'

    VEHICLES = 'Vehicles'

    WHAT = 'What'

    WHO = 'Who'

    WHY = 'Why'

    DEALS = 'Deals'

    BESTSELLERS = 'BestSellers'

    AGEGROUPS = 'AgeGroups'

    OCCASIONS = 'Occasions'

    FLOWERS = 'Flowers'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AdExtensionHeaderType from a JSON string"""
        return cls(json.loads(json_str))


