__author__ = 'Bing Ads SDK Team'
__email__ = 'bing_ads_sdk@microsoft.com'

from .common import *

from .bulk_error import *
from .bulk_entity import *
from .bid_suggestion_data import *
from .unknown_bulk_entity import *
from .bulk_account import *
from .bulk_budget import *
from .bulk_campaign import *
from .bulk_ad_group import *
from .bulk_keyword import *
from .bulk_campaign_product_scope import *
from .bulk_ad_group_product_partition import *
from .bulk_campaign_negative_store_criterion import *
from .bulk_campaign_negative_dynamic_search_ad_target import *
from .bulk_ad_group_dynamic_search_ad_target import *
from .bulk_ad_group_negative_dynamic_search_ad_target import *
from .bulk_bid_strategy import *
from .bulk_ad_group_criterion import *
from .bulk_ad_group_hotel_listing_group import *

from .ad_extensions import *
from .bulk_ads import *
from .bulk_negative_keywords import *
from .bulk_negative_sites import *
from .audiences import *
from .target_criterions import *
from .labels import *
from .bulk_offline_conversion import *
from .bulk_online_conversion_adjustment import *
from .bulk_experiment import *
from .bulk_image import *
from .bulk_video import *
from .feeds import *
from .bulk_campaign_conversion_goal import *
from .bulk_ad_customizer_attribute import *
from .bulk_ad_customizer_attribute_entity_base import *
from .bulk_ad_customizer_attribute_campaign import *
from .bulk_ad_customizer_attribute_ad_group import *
from .bulk_ad_customizer_attribute_keyword import *

from .bulk_asset_group import *
from .bulk_audience_group import *
from .bulk_asset_group_listing_group import *
from .bulk_asset_group_url_target import *
from .bulk_audience_group_asset_group_association import *
from .bulk_campaign_negative_webpage import *
from .bulk_seasonality_adjustment import *
from .bulk_data_exclusion import *
from .bulk_account_negative_keyword_list import *
from .bulk_asset_group_search_theme import *
from .bulk_campaign_brand_list_association import *
from .bulk_brand_item import *
from .bulk_brand_list import *
from .bulk_new_customer_acquisition_goal import *
from .goals import *
