# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class CategoryInsightsReportColumn(str, Enum):
    L1VERTICAL = 'L1Vertical'

    L2VERTICAL = 'L2Vertical'

    L3VERTICAL = 'L3Vertical'

    WEEKSTARTINGMONDAY = 'WeekStartingMonday'

    WOWSEARCHES = 'WoWSearches'

    MOMSEARCHES = 'MoMSearches'

    YOYLAST1WEEKSEARCHES = 'YoYLast1WeekSearches'

    YOYLAST4WEEKSSEARCHES = 'YoYLast4WeeksSearches'

    SEARCHESINDEXED = 'SearchesIndexed'

    SEARCHESPEAKWEEK = 'SearchesPeakWeek'

    WOWCLICKS = 'WoWClicks'

    MOMCLICKS = 'MoMClicks'

    YOYLAST1WEEKCLICKS = 'YoYLast1WeekClicks'

    CLICKSINDEXED = 'ClicksIndexed'

    CLICKSPEAKWEEK = 'ClicksPeakWeek'

    YOYLAST4WEEKSCLICKS = 'YoYLast4WeeksClicks'

    COUNTRY = 'Country'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CategoryInsightsReportColumn from a JSON string"""
        return cls(json.loads(json_str))


