# coding: utf-8

# flake8: noqa
"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.adinsight.ad_group_bid_landscape import AdGroupBidLandscape
from openapi_client.models.adinsight.ad_group_bid_landscape_input import AdGroupBidLandscapeInput
from openapi_client.models.adinsight.ad_group_bid_landscape_type import AdGroupBidLandscapeType
from openapi_client.models.adinsight.ad_group_estimate import AdGroupEstimate
from openapi_client.models.adinsight.ad_group_estimator import AdGroupEstimator
from openapi_client.models.adinsight.ad_position import AdPosition
from openapi_client.models.adinsight.age_enum import AgeEnum
from openapi_client.models.adinsight.api_fault_detail import ApiFaultDetail
from openapi_client.models.adinsight.apply_recommendation_entity import ApplyRecommendationEntity
from openapi_client.models.adinsight.apply_recommendations_request import ApplyRecommendationsRequest
from openapi_client.models.adinsight.apply_recommendations_response import ApplyRecommendationsResponse
from openapi_client.models.adinsight.auction_insight_entry import AuctionInsightEntry
from openapi_client.models.adinsight.auction_insight_kpi import AuctionInsightKpi
from openapi_client.models.adinsight.auction_insight_kpi_additional_field import AuctionInsightKpiAdditionalField
from openapi_client.models.adinsight.auction_insight_result import AuctionInsightResult
from openapi_client.models.adinsight.auction_segment import AuctionSegment
from openapi_client.models.adinsight.auction_segment_search_parameter import AuctionSegmentSearchParameter
from openapi_client.models.adinsight.auto_apply_recommendations_info import AutoApplyRecommendationsInfo
from openapi_client.models.adinsight.batch_error import BatchError
from openapi_client.models.adinsight.bid_landscape_point import BidLandscapePoint
from openapi_client.models.adinsight.bid_opportunity import BidOpportunity
from openapi_client.models.adinsight.bid_opportunity_type import BidOpportunityType
from openapi_client.models.adinsight.broad_match_keyword_opportunity import BroadMatchKeywordOpportunity
from openapi_client.models.adinsight.broad_match_search_query_kpi import BroadMatchSearchQueryKPI
from openapi_client.models.adinsight.budget_limit_type import BudgetLimitType
from openapi_client.models.adinsight.budget_opportunity import BudgetOpportunity
from openapi_client.models.adinsight.budget_point import BudgetPoint
from openapi_client.models.adinsight.budget_point_type import BudgetPointType
from openapi_client.models.adinsight.campaign_budget_recommendation import CampaignBudgetRecommendation
from openapi_client.models.adinsight.campaign_estimate import CampaignEstimate
from openapi_client.models.adinsight.campaign_estimator import CampaignEstimator
from openapi_client.models.adinsight.category_search_parameter import CategorySearchParameter
from openapi_client.models.adinsight.competition_level import CompetitionLevel
from openapi_client.models.adinsight.competition_search_parameter import CompetitionSearchParameter
from openapi_client.models.adinsight.criterion import Criterion
from openapi_client.models.adinsight.currency import Currency
from openapi_client.models.adinsight.currency_code import CurrencyCode
from openapi_client.models.adinsight.date_range_search_parameter import DateRangeSearchParameter
from openapi_client.models.adinsight.day_month_and_year import DayMonthAndYear
from openapi_client.models.adinsight.decimal_rounded_result import DecimalRoundedResult
from openapi_client.models.adinsight.device_criterion import DeviceCriterion
from openapi_client.models.adinsight.device_search_parameter import DeviceSearchParameter
from openapi_client.models.adinsight.dismiss_recommendation_entity import DismissRecommendationEntity
from openapi_client.models.adinsight.dismiss_recommendations_request import DismissRecommendationsRequest
from openapi_client.models.adinsight.dismiss_recommendations_response import DismissRecommendationsResponse
from openapi_client.models.adinsight.domain_category import DomainCategory
from openapi_client.models.adinsight.entity_detail import EntityDetail
from openapi_client.models.adinsight.entity_parameter import EntityParameter
from openapi_client.models.adinsight.entity_type import EntityType
from openapi_client.models.adinsight.estimated_bid_and_traffic import EstimatedBidAndTraffic
from openapi_client.models.adinsight.estimated_position_and_traffic import EstimatedPositionAndTraffic
from openapi_client.models.adinsight.exclude_account_keywords_search_parameter import ExcludeAccountKeywordsSearchParameter
from openapi_client.models.adinsight.feed import Feed
from openapi_client.models.adinsight.gender_enum import GenderEnum
from openapi_client.models.adinsight.get_auction_insight_data_request import GetAuctionInsightDataRequest
from openapi_client.models.adinsight.get_auction_insight_data_response import GetAuctionInsightDataResponse
from openapi_client.models.adinsight.get_audience_full_estimation_request import GetAudienceFullEstimationRequest
from openapi_client.models.adinsight.get_audience_full_estimation_response import GetAudienceFullEstimationResponse
from openapi_client.models.adinsight.get_auto_apply_opt_in_status_request import GetAutoApplyOptInStatusRequest
from openapi_client.models.adinsight.get_auto_apply_opt_in_status_response import GetAutoApplyOptInStatusResponse
from openapi_client.models.adinsight.get_bid_landscape_by_ad_group_ids_request import GetBidLandscapeByAdGroupIdsRequest
from openapi_client.models.adinsight.get_bid_landscape_by_ad_group_ids_response import GetBidLandscapeByAdGroupIdsResponse
from openapi_client.models.adinsight.get_bid_landscape_by_keyword_ids_request import GetBidLandscapeByKeywordIdsRequest
from openapi_client.models.adinsight.get_bid_landscape_by_keyword_ids_response import GetBidLandscapeByKeywordIdsResponse
from openapi_client.models.adinsight.get_bid_opportunities_request import GetBidOpportunitiesRequest
from openapi_client.models.adinsight.get_bid_opportunities_response import GetBidOpportunitiesResponse
from openapi_client.models.adinsight.get_budget_opportunities_request import GetBudgetOpportunitiesRequest
from openapi_client.models.adinsight.get_budget_opportunities_response import GetBudgetOpportunitiesResponse
from openapi_client.models.adinsight.get_domain_categories_request import GetDomainCategoriesRequest
from openapi_client.models.adinsight.get_domain_categories_response import GetDomainCategoriesResponse
from openapi_client.models.adinsight.get_estimated_bid_by_keyword_ids_request import GetEstimatedBidByKeywordIdsRequest
from openapi_client.models.adinsight.get_estimated_bid_by_keyword_ids_response import GetEstimatedBidByKeywordIdsResponse
from openapi_client.models.adinsight.get_estimated_bid_by_keywords_request import GetEstimatedBidByKeywordsRequest
from openapi_client.models.adinsight.get_estimated_bid_by_keywords_response import GetEstimatedBidByKeywordsResponse
from openapi_client.models.adinsight.get_estimated_position_by_keyword_ids_request import GetEstimatedPositionByKeywordIdsRequest
from openapi_client.models.adinsight.get_estimated_position_by_keyword_ids_response import GetEstimatedPositionByKeywordIdsResponse
from openapi_client.models.adinsight.get_estimated_position_by_keywords_request import GetEstimatedPositionByKeywordsRequest
from openapi_client.models.adinsight.get_estimated_position_by_keywords_response import GetEstimatedPositionByKeywordsResponse
from openapi_client.models.adinsight.get_historical_keyword_performance_request import GetHistoricalKeywordPerformanceRequest
from openapi_client.models.adinsight.get_historical_keyword_performance_response import GetHistoricalKeywordPerformanceResponse
from openapi_client.models.adinsight.get_historical_search_count_request import GetHistoricalSearchCountRequest
from openapi_client.models.adinsight.get_historical_search_count_response import GetHistoricalSearchCountResponse
from openapi_client.models.adinsight.get_keyword_categories_request import GetKeywordCategoriesRequest
from openapi_client.models.adinsight.get_keyword_categories_response import GetKeywordCategoriesResponse
from openapi_client.models.adinsight.get_keyword_demographics_request import GetKeywordDemographicsRequest
from openapi_client.models.adinsight.get_keyword_demographics_response import GetKeywordDemographicsResponse
from openapi_client.models.adinsight.get_keyword_idea_categories_response import GetKeywordIdeaCategoriesResponse
from openapi_client.models.adinsight.get_keyword_ideas_request import GetKeywordIdeasRequest
from openapi_client.models.adinsight.get_keyword_ideas_response import GetKeywordIdeasResponse
from openapi_client.models.adinsight.get_keyword_locations_request import GetKeywordLocationsRequest
from openapi_client.models.adinsight.get_keyword_locations_response import GetKeywordLocationsResponse
from openapi_client.models.adinsight.get_keyword_opportunities_request import GetKeywordOpportunitiesRequest
from openapi_client.models.adinsight.get_keyword_opportunities_response import GetKeywordOpportunitiesResponse
from openapi_client.models.adinsight.get_keyword_traffic_estimates_request import GetKeywordTrafficEstimatesRequest
from openapi_client.models.adinsight.get_keyword_traffic_estimates_response import GetKeywordTrafficEstimatesResponse
from openapi_client.models.adinsight.get_performance_insights_detail_data_by_account_id_request import GetPerformanceInsightsDetailDataByAccountIdRequest
from openapi_client.models.adinsight.get_performance_insights_detail_data_by_account_id_response import GetPerformanceInsightsDetailDataByAccountIdResponse
from openapi_client.models.adinsight.get_recommendations_request import GetRecommendationsRequest
from openapi_client.models.adinsight.get_recommendations_response import GetRecommendationsResponse
from openapi_client.models.adinsight.get_text_asset_suggestions_by_final_urls_request import GetTextAssetSuggestionsByFinalUrlsRequest
from openapi_client.models.adinsight.get_text_asset_suggestions_by_final_urls_response import GetTextAssetSuggestionsByFinalUrlsResponse
from openapi_client.models.adinsight.historical_search_count_periodic import HistoricalSearchCountPeriodic
from openapi_client.models.adinsight.idea_text_search_parameter import IdeaTextSearchParameter
from openapi_client.models.adinsight.impression_share_search_parameter import ImpressionShareSearchParameter
from openapi_client.models.adinsight.kpi_type import KPIType
from openapi_client.models.adinsight.keyword import Keyword
from openapi_client.models.adinsight.keyword_and_confidence import KeywordAndConfidence
from openapi_client.models.adinsight.keyword_and_match_type import KeywordAndMatchType
from openapi_client.models.adinsight.keyword_bid_landscape import KeywordBidLandscape
from openapi_client.models.adinsight.keyword_category import KeywordCategory
from openapi_client.models.adinsight.keyword_category_result import KeywordCategoryResult
from openapi_client.models.adinsight.keyword_demographic import KeywordDemographic
from openapi_client.models.adinsight.keyword_demographic_result import KeywordDemographicResult
from openapi_client.models.adinsight.keyword_estimate import KeywordEstimate
from openapi_client.models.adinsight.keyword_estimated_bid import KeywordEstimatedBid
from openapi_client.models.adinsight.keyword_estimated_position import KeywordEstimatedPosition
from openapi_client.models.adinsight.keyword_estimator import KeywordEstimator
from openapi_client.models.adinsight.keyword_historical_performance import KeywordHistoricalPerformance
from openapi_client.models.adinsight.keyword_id_estimated_bid import KeywordIdEstimatedBid
from openapi_client.models.adinsight.keyword_id_estimated_position import KeywordIdEstimatedPosition
from openapi_client.models.adinsight.keyword_idea import KeywordIdea
from openapi_client.models.adinsight.keyword_idea_attribute import KeywordIdeaAttribute
from openapi_client.models.adinsight.keyword_idea_category import KeywordIdeaCategory
from openapi_client.models.adinsight.keyword_kpi import KeywordKPI
from openapi_client.models.adinsight.keyword_location import KeywordLocation
from openapi_client.models.adinsight.keyword_location_result import KeywordLocationResult
from openapi_client.models.adinsight.keyword_opportunity import KeywordOpportunity
from openapi_client.models.adinsight.keyword_opportunity_base import KeywordOpportunityBase
from openapi_client.models.adinsight.keyword_opportunity_type import KeywordOpportunityType
from openapi_client.models.adinsight.keyword_recommendation import KeywordRecommendation
from openapi_client.models.adinsight.keyword_search_count import KeywordSearchCount
from openapi_client.models.adinsight.keyword_suggestion import KeywordSuggestion
from openapi_client.models.adinsight.language_criterion import LanguageCriterion
from openapi_client.models.adinsight.language_search_parameter import LanguageSearchParameter
from openapi_client.models.adinsight.location_criterion import LocationCriterion
from openapi_client.models.adinsight.location_search_parameter import LocationSearchParameter
from openapi_client.models.adinsight.match_type import MatchType
from openapi_client.models.adinsight.negative_keyword import NegativeKeyword
from openapi_client.models.adinsight.network_criterion import NetworkCriterion
from openapi_client.models.adinsight.network_search_parameter import NetworkSearchParameter
from openapi_client.models.adinsight.network_type import NetworkType
from openapi_client.models.adinsight.operation_error import OperationError
from openapi_client.models.adinsight.parameter_type import ParameterType
from openapi_client.models.adinsight.performance_insights_detail import PerformanceInsightsDetail
from openapi_client.models.adinsight.performance_insights_entity_type import PerformanceInsightsEntityType
from openapi_client.models.adinsight.performance_insights_message import PerformanceInsightsMessage
from openapi_client.models.adinsight.performance_insights_message_parameter import PerformanceInsightsMessageParameter
from openapi_client.models.adinsight.performance_insights_message_template_id import PerformanceInsightsMessageTemplateId
from openapi_client.models.adinsight.performance_insights_url_category import PerformanceInsightsUrlCategory
from openapi_client.models.adinsight.performance_insights_url_id import PerformanceInsightsUrlId
from openapi_client.models.adinsight.query_search_parameter import QuerySearchParameter
from openapi_client.models.adinsight.rsa_recommendation_info import RSARecommendationInfo
from openapi_client.models.adinsight.radius_target import RadiusTarget
from openapi_client.models.adinsight.radius_unit import RadiusUnit
from openapi_client.models.adinsight.range_result_of_decimal_rounded_result import RangeResultOfDecimalRoundedResult
from openapi_client.models.adinsight.range_result_ofdouble import RangeResultOfdouble
from openapi_client.models.adinsight.recommendation import Recommendation
from openapi_client.models.adinsight.recommendation_base import RecommendationBase
from openapi_client.models.adinsight.recommendation_base_base import RecommendationBaseBase
from openapi_client.models.adinsight.recommendation_info import RecommendationInfo
from openapi_client.models.adinsight.recommendation_info_base import RecommendationInfoBase
from openapi_client.models.adinsight.recommendation_type import RecommendationType
from openapi_client.models.adinsight.remove_conflicting_negative_keyword_recommendation import RemoveConflictingNegativeKeywordRecommendation
from openapi_client.models.adinsight.responsive_search_ad_asset_recommendation import ResponsiveSearchAdAssetRecommendation
from openapi_client.models.adinsight.responsive_search_ad_recommendation import ResponsiveSearchAdRecommendation
from openapi_client.models.adinsight.responsive_search_ads_recommendation import ResponsiveSearchAdsRecommendation
from openapi_client.models.adinsight.retrieve_recommendations_request import RetrieveRecommendationsRequest
from openapi_client.models.adinsight.retrieve_recommendations_response import RetrieveRecommendationsResponse
from openapi_client.models.adinsight.scale_unit import ScaleUnit
from openapi_client.models.adinsight.search_counts_by_attributes import SearchCountsByAttributes
from openapi_client.models.adinsight.search_parameter import SearchParameter
from openapi_client.models.adinsight.search_volume_search_parameter import SearchVolumeSearchParameter
from openapi_client.models.adinsight.selection_of_age_enum import SelectionOfAgeEnum
from openapi_client.models.adinsight.selection_of_gender_enum import SelectionOfGenderEnum
from openapi_client.models.adinsight.selection_oflong import SelectionOflong
from openapi_client.models.adinsight.set_auto_apply_opt_in_status_request import SetAutoApplyOptInStatusRequest
from openapi_client.models.adinsight.set_auto_apply_opt_in_status_response import SetAutoApplyOptInStatusResponse
from openapi_client.models.adinsight.source_type import SourceType
from openapi_client.models.adinsight.suggest_keywords_for_url_request import SuggestKeywordsForUrlRequest
from openapi_client.models.adinsight.suggest_keywords_for_url_response import SuggestKeywordsForUrlResponse
from openapi_client.models.adinsight.suggest_keywords_from_existing_keywords_request import SuggestKeywordsFromExistingKeywordsRequest
from openapi_client.models.adinsight.suggest_keywords_from_existing_keywords_response import SuggestKeywordsFromExistingKeywordsResponse
from openapi_client.models.adinsight.suggested_bid_search_parameter import SuggestedBidSearchParameter
from openapi_client.models.adinsight.suggested_responsive_search_ad import SuggestedResponsiveSearchAd
from openapi_client.models.adinsight.tag_recommendations_request import TagRecommendationsRequest
from openapi_client.models.adinsight.tag_recommendations_response import TagRecommendationsResponse
from openapi_client.models.adinsight.target_ad_position import TargetAdPosition
from openapi_client.models.adinsight.text_asset_suggestions import TextAssetSuggestions
from openapi_client.models.adinsight.text_parameter import TextParameter
from openapi_client.models.adinsight.time_interval import TimeInterval
from openapi_client.models.adinsight.traffic_estimate import TrafficEstimate
from openapi_client.models.adinsight.url_parameter import UrlParameter
from openapi_client.models.adinsight.url_search_parameter import UrlSearchParameter
from openapi_client.models.adinsight.use_broad_match_keyword_recommendation import UseBroadMatchKeywordRecommendation
