# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class HotelDimensionPerformanceReportColumn(str, Enum):
    TIMEPERIOD = 'TimePeriod'

    CUSTOMERID = 'CustomerId'

    CUSTOMERNAME = 'CustomerName'

    ACCOUNTNAME = 'AccountName'

    ACCOUNTID = 'AccountId'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    ADGROUPID = 'AdGroupId'

    ADGROUPNAME = 'AdGroupName'

    ADID = 'AdId'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

    SOURCEHOTELID = 'SourceHotelId'

    CAMPAIGNHOTELID = 'CampaignHotelId'

    ADVERTISERHOTELID = 'AdvertiserHotelId'

    HOTELNAME = 'HotelName'

    COUNTRYNAME = 'CountryName'

    STATE = 'State'

    CITY = 'City'

    BRAND = 'Brand'

    CATEGORY = 'Category'

    STARRATING = 'StarRating'

    CUSTOMLABEL0 = 'CustomLabel0'

    CUSTOMLABEL1 = 'CustomLabel1'

    CUSTOMLABEL2 = 'CustomLabel2'

    CUSTOMLABEL3 = 'CustomLabel3'

    CUSTOMLABEL4 = 'CustomLabel4'

    BIDSTRATEGYTYPE = 'BidStrategyType'

    DEVICETYPE = 'DeviceType'

    CURRENCYCODE = 'CurrencyCode'

    CUSTOMPARAMETERS = 'CustomParameters'

    FINALURLSUFFIX = 'FinalUrlSuffix'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    SPEND = 'Spend'

    AVERAGECPC = 'AverageCpc'

    AVERAGECPM = 'AverageCpm'

    CONVERSIONS = 'Conversions'

    REVENUE = 'Revenue'

    NETREVENUE = 'NetRevenue'

    GROSSRETURNONADSPEND = 'GrossReturnOnAdSpend'

    NETRETURNONADSPEND = 'NetReturnOnAdSpend'

    COSTPERASSIST = 'CostPerAssist'

    BOOKEDLENGTHOFSTAY = 'BookedLengthOfStay'

    AVGBOOKEDABW = 'AvgBookedABW'

    AVGBOOKEDNIGHTS = 'AvgBookedNights'

    BOOKEDABW = 'BookedABW'

    CHECKINDATE = 'CheckInDate'

    DATETYPE = 'DateType'

    PARTNERIMPRESSIONS = 'PartnerImpressions'

    ELIGIBLEIMPRESSIONS = 'EligibleImpressions'

    MISSEDIMPRESSIONS = 'MissedImpressions'

    IMPRESSIONSHARE = 'ImpressionShare'

    MISSEDIMPRESSIONSNOBID = 'MissedImpressionsNoBid'

    MISSEDIMPRESSIONSSPENDINGCAPREACHED = 'MissedImpressionsSpendingCapReached'

    MISSEDIMPRESSIONSNOTAX = 'MissedImpressionsNoTax'

    MISSEDIMPRESSIONSOTHER = 'MissedImpressionsOther'

    CHECKINDATEDAYOFWEEK = 'CheckInDateDayOfWeek'

    LENGTHOFSTAY = 'LengthOfStay'

    ADVANCEDBOOKINGWINDOW = 'AdvancedBookingWindow'

    SITETYPE = 'SiteType'

    USERCOUNTRY = 'UserCountry'

    CLICKSHARE = 'ClickShare'

    MISSEDIMPRESSIONSINSUFFICIENTBID = 'MissedImpressionsInsufficientBid'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of HotelDimensionPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))


