# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class LanguageName(str, Enum):
    NONE = 'None'

    ALBANIAN = 'Albanian'

    ARABIC = 'Arabic'

    BULGARIAN = 'Bulgarian'

    TRADITIONALCHINESE = 'TraditionalChinese'

    CROATIAN = 'Croatian'

    CZECH = 'Czech'

    DANISH = 'Danish'

    DUTCH = 'Dutch'

    ENGLISH = 'English'

    ESTONIAN = 'Estonian'

    FINNISH = 'Finnish'

    FRENCH = 'French'

    GERMAN = 'German'

    GREEK = 'Greek'

    HEBREW = 'Hebrew'

    HINDI = 'Hindi'

    HUNGARIAN = 'Hungarian'

    ICELANDIC = 'Icelandic'

    INDONESIAN = 'Indonesian'

    ITALIAN = 'Italian'

    JAPANESE = 'Japanese'

    KOREAN = 'Korean'

    LATVIAN = 'Latvian'

    LITHUANIAN = 'Lithuanian'

    MACEDONIAN = 'Macedonian'

    MALAY = 'Malay'

    MALTESE = 'Maltese'

    NORWEGIAN = 'Norwegian'

    POLISH = 'Polish'

    PORTUGUESE = 'Portuguese'

    ROMANIAN = 'Romanian'

    RUSSIAN = 'Russian'

    SERBIAN = 'Serbian'

    SLOVAK = 'Slovak'

    SLOVENIAN = 'Slovenian'

    SPANISH = 'Spanish'

    SWEDISH = 'Swedish'

    TAGALOG = 'Tagalog'

    THAI = 'Thai'

    TURKISH = 'Turkish'

    VIETNAMESE = 'Vietnamese'

    SIMPLIFIEDCHINESE = 'SimplifiedChinese'

    BOSNIAN = 'Bosnian'

    ALL = 'All'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of LanguageName from a JSON string"""
        return cls(json.loads(json_str))


