# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class SearchCampaignChangeHistoryReportColumn(str, Enum):
    DATETIME = 'DateTime'

    ACCOUNTID = 'AccountId'

    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    CHANGEDBY = 'ChangedBy'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    ADTITLE = 'AdTitle'

    ADDESCRIPTION = 'AdDescription'

    DISPLAYURL = 'DisplayUrl'

    KEYWORD = 'Keyword'

    ITEMCHANGED = 'ItemChanged'

    ATTRIBUTECHANGED = 'AttributeChanged'

    HOWCHANGED = 'HowChanged'

    OLDVALUE = 'OldValue'

    NEWVALUE = 'NewValue'

    ENTITYNAME = 'EntityName'

    ENTITYID = 'EntityId'

    TOOL = 'Tool'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SearchCampaignChangeHistoryReportColumn from a JSON string"""
        return cls(json.loads(json_str))


