# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class FaultSource(str, Enum):
    MIDDLETIER = 'MiddleTier'

    DATABASE = 'Database'

    EDITORIAL = 'Editorial'

    AUTHENTICATION = 'Authentication'

    AUTHORIZATION = 'Authorization'

    PUBLISHER = 'Publisher'

    AUDIENCEINTELIGENCE = 'AudienceInteligence'

    CAM = 'Cam'

    DAM = 'Dam'

    FLEXTAG = 'FlexTag'

    BINGMERCHANTCENTER = 'BingMerchantCenter'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of FaultSource from a JSON string"""
        return cls(json.loads(json_str))


