# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ReportTimePeriod(str, Enum):
    TODAY = 'Today'

    YESTERDAY = 'Yesterday'

    LASTSEVENDAYS = 'LastSevenDays'

    THISWEEK = 'ThisWeek'

    LASTWEEK = 'LastWeek'

    LAST14DAYS = 'Last14Days'

    LAST30DAYS = 'Last30Days'

    LASTFOURWEEKS = 'LastFourWeeks'

    THISMONTH = 'ThisMonth'

    LASTMONTH = 'LastMonth'

    LASTTHREEMONTHS = 'LastThreeMonths'

    LASTSIXMONTHS = 'LastSixMonths'

    THISYEAR = 'ThisYear'

    LASTYEAR = 'LastYear'

    THISWEEKSTARTINGMONDAY = 'ThisWeekStartingMonday'

    LASTWEEKSTARTINGMONDAY = 'LastWeekStartingMonday'

    LASTFOURWEEKSSTARTINGMONDAY = 'LastFourWeeksStartingMonday'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ReportTimePeriod from a JSON string"""
        return cls(json.loads(json_str))


