# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class PromotionOccasion(str, Enum):
    UNKNOWN = 'Unknown'

    NEWYEARS = 'NewYears'

    VALENTINESDAY = 'ValentinesDay'

    EASTER = 'Easter'

    MOTHERSDAY = 'MothersDay'

    FATHERSDAY = 'FathersDay'

    LABORDAY = 'LaborDay'

    BACKTOSCHOOL = 'BackToSchool'

    HALLOWEEN = 'Halloween'

    BLACKFRIDAY = 'BlackFriday'

    CYBERMONDAY = 'CyberMonday'

    CHRISTMAS = 'Christmas'

    BOXINGDAY = 'BoxingDay'

    NONE = 'None'

    INDEPENDENCEDAY = 'IndependenceDay'

    NATIONALDAY = 'NationalDay'

    ENDOFSEASON = 'EndOfSeason'

    WINTERSALE = 'WinterSale'

    SUMMERSALE = 'SummerSale'

    FALLSALE = 'FallSale'

    SPRINGSALE = 'SpringSale'

    RAMADAN = 'Ramadan'

    EIDALFITR = 'EidAlFitr'

    EIDALADHA = 'EidAlAdha'

    SINGLESDAY = 'SinglesDay'

    WOMENSDAY = 'WomensDay'

    HOLI = 'Holi'

    PARENTSDAY = 'ParentsDay'

    STNICHOLASDAY = 'StNicholasDay'

    CHINESENEWYEAR = 'ChineseNewYear'

    CARNIVAL = 'Carnival'

    EPIPHANY = 'Epiphany'

    ROSHHASHANAH = 'RoshHashanah'

    PASSOVER = 'Passover'

    HANUKKAH = 'Hanukkah'

    DIWALI = 'Diwali'

    NAVRATRI = 'Navratri'

    SONGKRAN = 'Songkran'

    YEARENDGIFT = 'YearEndGift'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PromotionOccasion from a JSON string"""
        return cls(json.loads(json_str))


