# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class CombinationPerformanceReportColumn(str, Enum):
    TIMEPERIOD = 'TimePeriod'

    ACCOUNTID = 'AccountId'

    ACCOUNTNAME = 'AccountName'

    CAMPAIGNID = 'CampaignId'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNTYPE = 'CampaignType'

    ADGROUPID = 'AdGroupId'

    ADGROUPNAME = 'AdGroupName'

    ASSETGROUPID = 'AssetGroupId'

    ASSETGROUPNAME = 'AssetGroupName'

    ADID = 'AdId'

    ADTYPE = 'AdType'

    HEADLINE1 = 'Headline1'

    HEADLINE2 = 'Headline2'

    HEADLINE3 = 'Headline3'

    COMBINATIONLONGHEADLINE = 'CombinationLongHeadline'

    DESCRIPTION1 = 'Description1'

    DESCRIPTION2 = 'Description2'

    IMAGE = 'Image'

    LOGO = 'Logo'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    SPEND = 'Spend'

    CTR = 'Ctr'

    CONVERSIONS = 'Conversions'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CombinationPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))


