# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class VanityPharmaWebsiteDescription(str, Enum):
    UNSPECIFIED = 'Unspecified'

    PRESCRIPTION_TREATMENT_WEBSITE_EN = 'PrescriptionTreatmentWebsite_En'

    PRESCRIPTION_TREATMENT_WEBSITE_ES = 'PrescriptionTreatmentWebsite_Es'

    PRESCRIPTION_DEVICE_WEBSITE_EN = 'PrescriptionDeviceWebsite_En'

    PRESCRIPTION_DEVICE_WEBSITE_ES = 'PrescriptionDeviceWebsite_Es'

    MEDICAL_DEVICE_WEBSITE_EN = 'MedicalDeviceWebsite_En'

    MEDICAL_DEVICE_WEBSITE_ES = 'MedicalDeviceWebsite_Es'

    PREVENTATIVE_TREATMENT_WEBSITE_EN = 'PreventativeTreatmentWebsite_En'

    PREVENTATIVE_TREATMENT_WEBSITE_ES = 'PreventativeTreatmentWebsite_Es'

    PRESCRIPTION_CONTRACEPTION_WEBSITE_EN = 'PrescriptionContraceptionWebsite_En'

    PRESCRIPTION_CONTRACEPTION_WEBSITE_ES = 'PrescriptionContraceptionWebsite_Es'

    PRESCRIPTION_VACCINE_WEBSITE_EN = 'PrescriptionVaccineWebsite_En'

    PRESCRIPTION_VACCINE_WEBSITE_ES = 'PrescriptionVaccineWebsite_Es'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of VanityPharmaWebsiteDescription from a JSON string"""
        return cls(json.loads(json_str))


