# coding: utf-8

"""
    BulkService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class DownloadEntity(str, Enum):
    NONE = 'None'

    CAMPAIGNS = 'Campaigns'

    ADGROUPS = 'AdGroups'

    ADS = 'Ads'

    KEYWORDS = 'Keywords'

    CAMPAIGNNEGATIVEKEYWORDS = 'CampaignNegativeKeywords'

    ADGROUPNEGATIVEKEYWORDS = 'AdGroupNegativeKeywords'

    CAMPAIGNTARGETCRITERIONS = 'CampaignTargetCriterions'

    ADGROUPTARGETCRITERIONS = 'AdGroupTargetCriterions'

    CAMPAIGNNEGATIVESITES = 'CampaignNegativeSites'

    ADGROUPNEGATIVESITES = 'AdGroupNegativeSites'

    CAMPAIGNLOCATIONADEXTENSIONS = 'CampaignLocationAdExtensions'

    CAMPAIGNCALLADEXTENSIONS = 'CampaignCallAdExtensions'

    LOCATIONADEXTENSIONS = 'LocationAdExtensions'

    CALLADEXTENSIONS = 'CallAdExtensions'

    NEGATIVEKEYWORDLISTS = 'NegativeKeywordLists'

    SHAREDNEGATIVEKEYWORDS = 'SharedNegativeKeywords'

    CAMPAIGNNEGATIVEKEYWORDLISTASSOCIATIONS = 'CampaignNegativeKeywordListAssociations'

    IMAGEADEXTENSIONS = 'ImageAdExtensions'

    CAMPAIGNIMAGEADEXTENSIONS = 'CampaignImageAdExtensions'

    ADGROUPIMAGEADEXTENSIONS = 'AdGroupImageAdExtensions'

    APPADEXTENSIONS = 'AppAdExtensions'

    ADGROUPAPPADEXTENSIONS = 'AdGroupAppAdExtensions'

    CAMPAIGNAPPADEXTENSIONS = 'CampaignAppAdExtensions'

    PRICEADEXTENSIONS = 'PriceAdExtensions'

    REVIEWADEXTENSIONS = 'ReviewAdExtensions'

    CAMPAIGNNEGATIVEDYNAMICSEARCHADTARGETS = 'CampaignNegativeDynamicSearchAdTargets'

    ADGROUPPRODUCTPARTITIONS = 'AdGroupProductPartitions'

    CAMPAIGNPRODUCTSCOPES = 'CampaignProductScopes'

    CAMPAIGNREVIEWADEXTENSIONS = 'CampaignReviewAdExtensions'

    ADGROUPREVIEWADEXTENSIONS = 'AdGroupReviewAdExtensions'

    CALLOUTADEXTENSIONS = 'CalloutAdExtensions'

    CAMPAIGNCALLOUTADEXTENSIONS = 'CampaignCalloutAdExtensions'

    ADGROUPCALLOUTADEXTENSIONS = 'AdGroupCalloutAdExtensions'

    SITELINKADEXTENSIONS = 'SitelinkAdExtensions'

    CAMPAIGNSITELINKADEXTENSIONS = 'CampaignSitelinkAdExtensions'

    ADGROUPSITELINKADEXTENSIONS = 'AdGroupSitelinkAdExtensions'

    STRUCTUREDSNIPPETADEXTENSIONS = 'StructuredSnippetAdExtensions'

    CAMPAIGNSTRUCTUREDSNIPPETADEXTENSIONS = 'CampaignStructuredSnippetAdExtensions'

    ADGROUPSTRUCTUREDSNIPPETADEXTENSIONS = 'AdGroupStructuredSnippetAdExtensions'

    REMARKETINGLISTS = 'RemarketingLists'

    ADGROUPREMARKETINGLISTASSOCIATIONS = 'AdGroupRemarketingListAssociations'

    BUDGETS = 'Budgets'

    TEXTADS = 'TextAds'

    PRODUCTADS = 'ProductAds'

    APPINSTALLADS = 'AppInstallAds'

    EXPANDEDTEXTADS = 'ExpandedTextAds'

    DYNAMICSEARCHADS = 'DynamicSearchAds'

    ADGROUPDYNAMICSEARCHADTARGETS = 'AdGroupDynamicSearchAdTargets'

    ADGROUPNEGATIVEDYNAMICSEARCHADTARGETS = 'AdGroupNegativeDynamicSearchAdTargets'

    CAMPAIGNPRICEADEXTENSIONS = 'CampaignPriceAdExtensions'

    ADGROUPPRICEADEXTENSIONS = 'AdGroupPriceAdExtensions'

    LABELS = 'Labels'

    CAMPAIGNLABELS = 'CampaignLabels'

    ADGROUPLABELS = 'AdGroupLabels'

    TEXTADLABELS = 'TextAdLabels'

    KEYWORDLABELS = 'KeywordLabels'

    ADGROUPNEGATIVEREMARKETINGLISTASSOCIATIONS = 'AdGroupNegativeRemarketingListAssociations'

    CUSTOMAUDIENCES = 'CustomAudiences'

    ADGROUPCUSTOMAUDIENCEASSOCIATIONS = 'AdGroupCustomAudienceAssociations'

    ADGROUPNEGATIVECUSTOMAUDIENCEASSOCIATIONS = 'AdGroupNegativeCustomAudienceAssociations'

    INMARKETAUDIENCES = 'InMarketAudiences'

    ADGROUPINMARKETAUDIENCEASSOCIATIONS = 'AdGroupInMarketAudienceAssociations'

    ADGROUPNEGATIVEINMARKETAUDIENCEASSOCIATIONS = 'AdGroupNegativeInMarketAudienceAssociations'

    AUDIENCES = 'Audiences'

    ADGROUPAUDIENCEASSOCIATIONS = 'AdGroupAudienceAssociations'

    ADGROUPNEGATIVEAUDIENCEASSOCIATIONS = 'AdGroupNegativeAudienceAssociations'

    PRODUCTADLABELS = 'ProductAdLabels'

    APPINSTALLADLABELS = 'AppInstallAdLabels'

    EXPANDEDTEXTADLABELS = 'ExpandedTextAdLabels'

    DYNAMICSEARCHADLABELS = 'DynamicSearchAdLabels'

    ACCOUNTLOCATIONADEXTENSIONS = 'AccountLocationAdExtensions'

    ACCOUNTIMAGEADEXTENSIONS = 'AccountImageAdExtensions'

    ACCOUNTAPPADEXTENSIONS = 'AccountAppAdExtensions'

    ACCOUNTPRICEADEXTENSIONS = 'AccountPriceAdExtensions'

    ACCOUNTREVIEWADEXTENSIONS = 'AccountReviewAdExtensions'

    ACCOUNTCALLOUTADEXTENSIONS = 'AccountCalloutAdExtensions'

    ACCOUNTSITELINKADEXTENSIONS = 'AccountSitelinkAdExtensions'

    ACCOUNTSTRUCTUREDSNIPPETADEXTENSIONS = 'AccountStructuredSnippetAdExtensions'

    RESPONSIVEADS = 'ResponsiveAds'

    RESPONSIVEADLABELS = 'ResponsiveAdLabels'

    PRODUCTAUDIENCES = 'ProductAudiences'

    ADGROUPPRODUCTAUDIENCEASSOCIATIONS = 'AdGroupProductAudienceAssociations'

    ADGROUPNEGATIVEPRODUCTAUDIENCEASSOCIATIONS = 'AdGroupNegativeProductAudienceAssociations'

    SIMILARREMARKETINGLISTS = 'SimilarRemarketingLists'

    ADGROUPSIMILARREMARKETINGLISTASSOCIATIONS = 'AdGroupSimilarRemarketingListAssociations'

    ADGROUPNEGATIVESIMILARREMARKETINGLISTASSOCIATIONS = 'AdGroupNegativeSimilarRemarketingListAssociations'

    EXPERIMENTS = 'Experiments'

    ACTIONADEXTENSIONS = 'ActionAdExtensions'

    CAMPAIGNACTIONADEXTENSIONS = 'CampaignActionAdExtensions'

    ADGROUPACTIONADEXTENSIONS = 'AdGroupActionAdExtensions'

    ACCOUNTACTIONADEXTENSIONS = 'AccountActionAdExtensions'

    RESPONSIVESEARCHADS = 'ResponsiveSearchAds'

    RESPONSIVESEARCHADLABELS = 'ResponsiveSearchAdLabels'

    CAMPAIGNAUDIENCEASSOCIATIONS = 'CampaignAudienceAssociations'

    CAMPAIGNNEGATIVEAUDIENCEASSOCIATIONS = 'CampaignNegativeAudienceAssociations'

    CAMPAIGNCUSTOMAUDIENCEASSOCIATIONS = 'CampaignCustomAudienceAssociations'

    CAMPAIGNNEGATIVECUSTOMAUDIENCEASSOCIATIONS = 'CampaignNegativeCustomAudienceAssociations'

    CAMPAIGNINMARKETAUDIENCEASSOCIATIONS = 'CampaignInMarketAudienceAssociations'

    CAMPAIGNNEGATIVEINMARKETAUDIENCEASSOCIATIONS = 'CampaignNegativeInMarketAudienceAssociations'

    CAMPAIGNPRODUCTAUDIENCEASSOCIATIONS = 'CampaignProductAudienceAssociations'

    CAMPAIGNNEGATIVEPRODUCTAUDIENCEASSOCIATIONS = 'CampaignNegativeProductAudienceAssociations'

    CAMPAIGNREMARKETINGLISTASSOCIATIONS = 'CampaignRemarketingListAssociations'

    CAMPAIGNNEGATIVEREMARKETINGLISTASSOCIATIONS = 'CampaignNegativeRemarketingListAssociations'

    CAMPAIGNSIMILARREMARKETINGLISTASSOCIATIONS = 'CampaignSimilarRemarketingListAssociations'

    CAMPAIGNNEGATIVESIMILARREMARKETINGLISTASSOCIATIONS = 'CampaignNegativeSimilarRemarketingListAssociations'

    FEEDS = 'Feeds'

    FEEDITEMS = 'FeedItems'

    CAMPAIGNNEGATIVESTORECRITERIONS = 'CampaignNegativeStoreCriterions'

    PROMOTIONADEXTENSIONS = 'PromotionAdExtensions'

    ACCOUNTPROMOTIONADEXTENSIONS = 'AccountPromotionAdExtensions'

    CAMPAIGNPROMOTIONADEXTENSIONS = 'CampaignPromotionAdExtensions'

    ADGROUPPROMOTIONADEXTENSIONS = 'AdGroupPromotionAdExtensions'

    COMBINEDLISTS = 'CombinedLists'

    ADGROUPCOMBINEDLISTASSOCIATIONS = 'AdGroupCombinedListAssociations'

    ADGROUPNEGATIVECOMBINEDLISTASSOCIATIONS = 'AdGroupNegativeCombinedListAssociations'

    CAMPAIGNCOMBINEDLISTASSOCIATIONS = 'CampaignCombinedListAssociations'

    CAMPAIGNNEGATIVECOMBINEDLISTASSOCIATIONS = 'CampaignNegativeCombinedListAssociations'

    IMAGES = 'Images'

    CUSTOMERLISTS = 'CustomerLists'

    ADGROUPCUSTOMERLISTASSOCIATIONS = 'AdGroupCustomerListAssociations'

    ADGROUPNEGATIVECUSTOMERLISTASSOCIATIONS = 'AdGroupNegativeCustomerListAssociations'

    CAMPAIGNCUSTOMERLISTASSOCIATIONS = 'CampaignCustomerListAssociations'

    CAMPAIGNNEGATIVECUSTOMERLISTASSOCIATIONS = 'CampaignNegativeCustomerListAssociations'

    FILTERLINKADEXTENSIONS = 'FilterLinkAdExtensions'

    ACCOUNTFILTERLINKADEXTENSIONS = 'AccountFilterLinkAdExtensions'

    CAMPAIGNFILTERLINKADEXTENSIONS = 'CampaignFilterLinkAdExtensions'

    ADGROUPFILTERLINKADEXTENSIONS = 'AdGroupFilterLinkAdExtensions'

    FLYERADEXTENSIONS = 'FlyerAdExtensions'

    ACCOUNTFLYERADEXTENSIONS = 'AccountFlyerAdExtensions'

    CAMPAIGNFLYERADEXTENSIONS = 'CampaignFlyerAdExtensions'

    ADGROUPFLYERADEXTENSIONS = 'AdGroupFlyerAdExtensions'

    BIDSTRATEGIES = 'BidStrategies'

    VIDEOADEXTENSIONS = 'VideoAdExtensions'

    ACCOUNTVIDEOADEXTENSIONS = 'AccountVideoAdExtensions'

    CAMPAIGNVIDEOADEXTENSIONS = 'CampaignVideoAdExtensions'

    ADGROUPVIDEOADEXTENSIONS = 'AdGroupVideoAdExtensions'

    VIDEOS = 'Videos'

    DISCLAIMERADEXTENSIONS = 'DisclaimerAdExtensions'

    CAMPAIGNDISCLAIMERADEXTENSIONS = 'CampaignDisclaimerAdExtensions'

    ADCUSTOMIZERATTRIBUTE = 'AdcustomizerAttribute'

    CAMPAIGNADCUSTOMIZERATTRIBUTE = 'CampaignAdcustomizerAttribute'

    ADGROUPADCUSTOMIZERATTRIBUTE = 'AdGroupAdcustomizerAttribute'

    KEYWORDADCUSTOMIZERATTRIBUTE = 'KeywordAdcustomizerAttribute'

    CAMPAIGNCONVERSIONGOAL = 'CampaignConversionGoal'

    ADGROUPHOTELLISTINGGROUPS = 'AdGroupHotelListingGroups'

    ASSETGROUPS = 'AssetGroups'

    AUDIENCEGROUPS = 'AudienceGroups'

    CAMPAIGNNEGATIVEWEBPAGES = 'CampaignNegativeWebpages'

    AUDIENCEGROUPASSETGROUPASSOCIATIONS = 'AudienceGroupAssetGroupAssociations'

    ASSETGROUPLISTINGGROUPS = 'AssetGroupListingGroups'

    LOGOADEXTENSIONS = 'LogoAdExtensions'

    ACCOUNTLOGOADEXTENSIONS = 'AccountLogoAdExtensions'

    CAMPAIGNLOGOADEXTENSIONS = 'CampaignLogoAdExtensions'

    ADGROUPLOGOADEXTENSIONS = 'AdGroupLogoAdExtensions'

    LEADFORMADEXTENSIONS = 'LeadFormAdExtensions'

    ACCOUNTLEADFORMADEXTENSIONS = 'AccountLeadFormAdExtensions'

    CAMPAIGNLEADFORMADEXTENSIONS = 'CampaignLeadFormAdExtensions'

    ACCOUNTNEGATIVEKEYWORDLIST = 'AccountNegativeKeywordList'

    ACCOUNTNEGATIVEKEYWORDLISTASSOCIATION = 'AccountNegativeKeywordListAssociation'

    ACCOUNTNEGATIVEKEYWORD = 'AccountNegativeKeyword'

    CONVERSIONGOAL = 'ConversionGoal'

    EVENTGOAL = 'EventGoal'

    APPINSTALLGOAL = 'AppInstallGoal'

    MULTISTAGEGOAL = 'MultiStageGoal'

    DURATIONGOAL = 'DurationGoal'

    OFFLINECONVERSIONGOAL = 'OfflineConversionGoal'

    URLGOAL = 'UrlGoal'

    INSTORETRANSACTIONGOAL = 'InStoreTransactionGoal'

    PAGESVIEWEDPERVISITGOAL = 'PagesViewedPerVisitGoal'

    SMARTGOAL = 'SmartGoal'

    INSTOREVISITGOAL = 'InStoreVisitGoal'

    PRODUCTGOAL = 'ProductGoal'

    SEASONALITYADJUSTMENTS = 'SeasonalityAdjustments'

    DATAEXCLUSIONS = 'DataExclusions'

    BRANDLIST = 'BrandList'

    BRANDITEM = 'BrandItem'

    CAMPAIGNBRANDLISTASSOCIATION = 'CampaignBrandListAssociation'

    ASSETGROUPSEARCHTHEMES = 'AssetGroupSearchThemes'

    IMPRESSIONBASEDREMARKETINGLIST = 'ImpressionBasedRemarketingList'

    ADGROUPIMPRESSIONBASEDREMARKETINGLISTASSOCIATIONS = 'AdGroupImpressionBasedRemarketingListAssociations'

    ADGROUPNEGATIVEIMPRESSIONBASEDREMARKETINGLISTASSOCIATIONS = 'AdGroupNegativeImpressionBasedRemarketingListAssociations'

    CAMPAIGNIMPRESSIONBASEDREMARKETINGLISTASSOCIATIONS = 'CampaignImpressionBasedRemarketingListAssociations'

    CAMPAIGNNEGATIVEIMPRESSIONBASEDREMARKETINGLISTASSOCIATIONS = 'CampaignNegativeImpressionBasedRemarketingListAssociations'

    ASSETGROUPURLTARGETS = 'AssetGroupUrlTargets'

    NEWCUSTOMERACQUISITIONGOAL = 'NewCustomerAcquisitionGoal'

    CONVERSIONVALUERULE = 'ConversionValueRule'

    ACCOUNTPLACEMENTEXCLUSIONLIST = 'AccountPlacementExclusionList'

    ACCOUNTPLACEMENTEXCLUSIONLISTITEM = 'AccountPlacementExclusionListItem'

    CAMPAIGNACCOUNTPLACEMENTEXCLUSIONLISTASSOCIATION = 'CampaignAccountPlacementExclusionListAssociation'

    ACCOUNTPLACEMENTINCLUSIONLIST = 'AccountPlacementInclusionList'

    ACCOUNTPLACEMENTINCLUSIONLISTITEM = 'AccountPlacementInclusionListItem'

    CAMPAIGNACCOUNTPLACEMENTINCLUSIONLISTASSOCIATION = 'CampaignAccountPlacementInclusionListAssociation'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of DownloadEntity from a JSON string"""
        return cls(json.loads(json_str))


