# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ProductPartitionPerformanceReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    TIMEPERIOD = 'TimePeriod'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    PRODUCTGROUP = 'ProductGroup'

    ADGROUPCRITERIONID = 'AdGroupCriterionId'

    PARTITIONTYPE = 'PartitionType'

    ADID = 'AdId'

    CURRENTMAXCPC = 'CurrentMaxCpc'

    CURRENCYCODE = 'CurrencyCode'

    DELIVEREDMATCHTYPE = 'DeliveredMatchType'

    BIDMATCHTYPE = 'BidMatchType'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    AVERAGECPC = 'AverageCpc'

    SPEND = 'Spend'

    CONVERSIONS = 'Conversions'

    CONVERSIONRATE = 'ConversionRate'

    COSTPERCONVERSION = 'CostPerConversion'

    DEVICETYPE = 'DeviceType'

    LANGUAGE = 'Language'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ACCOUNTSTATUS = 'AccountStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

    DESTINATIONURL = 'DestinationUrl'

    NETWORK = 'Network'

    TOPVSOTHER = 'TopVsOther'

    ASSISTS = 'Assists'

    REVENUE = 'Revenue'

    COSTPERASSIST = 'CostPerAssist'

    REVENUEPERCONVERSION = 'RevenuePerConversion'

    REVENUEPERASSIST = 'RevenuePerAssist'

    OFFERLANGUAGE = 'OfferLanguage'

    COUNTRYOFSALE = 'CountryOfSale'

    ADSTATUS = 'AdStatus'

    TRACKINGTEMPLATE = 'TrackingTemplate'

    CUSTOMPARAMETERS = 'CustomParameters'

    IMPRESSIONSHAREPERCENT = 'ImpressionSharePercent'

    IMPRESSIONLOSTTOBUDGETPERCENT = 'ImpressionLostToBudgetPercent'

    IMPRESSIONLOSTTORANKPERCENT = 'ImpressionLostToRankPercent'

    BENCHMARKBID = 'BenchmarkBid'

    BENCHMARKCTR = 'BenchmarkCtr'

    ADDISTRIBUTION = 'AdDistribution'

    CLICKTYPEID = 'ClickTypeId'

    TOTALCLICKSONADELEMENTS = 'TotalClicksOnAdElements'

    CLICKTYPE = 'ClickType'

    RETURNONADSPEND = 'ReturnOnAdSpend'

    BIDSTRATEGYTYPE = 'BidStrategyType'

    LOCALSTORECODE = 'LocalStoreCode'

    ASSISTEDIMPRESSIONS = 'AssistedImpressions'

    ASSISTEDCLICKS = 'AssistedClicks'

    CLICKSHAREPERCENT = 'ClickSharePercent'

    ABSOLUTETOPIMPRESSIONSHAREPERCENT = 'AbsoluteTopImpressionSharePercent'

    ASSISTEDCONVERSIONS = 'AssistedConversions'

    ALLCONVERSIONS = 'AllConversions'

    ALLREVENUE = 'AllRevenue'

    ALLCONVERSIONRATE = 'AllConversionRate'

    ALLCOSTPERCONVERSION = 'AllCostPerConversion'

    ALLRETURNONADSPEND = 'AllReturnOnAdSpend'

    ALLREVENUEPERCONVERSION = 'AllRevenuePerConversion'

    VIEWTHROUGHCONVERSIONS = 'ViewThroughConversions'

    GOAL = 'Goal'

    GOALTYPE = 'GoalType'

    ABSOLUTETOPIMPRESSIONRATEPERCENT = 'AbsoluteTopImpressionRatePercent'

    AVERAGECPM = 'AverageCpm'

    CONVERSIONSQUALIFIED = 'ConversionsQualified'

    ASSISTEDCONVERSIONSQUALIFIED = 'AssistedConversionsQualified'

    ALLCONVERSIONSQUALIFIED = 'AllConversionsQualified'

    VIEWTHROUGHCONVERSIONSQUALIFIED = 'ViewThroughConversionsQualified'

    VIEWTHROUGHREVENUE = 'ViewThroughRevenue'

    CAMPAIGNTYPE = 'CampaignType'

    ASSETGROUPID = 'AssetGroupId'

    ASSETGROUPNAME = 'AssetGroupName'

    ASSETGROUPSTATUS = 'AssetGroupStatus'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ProductPartitionPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))


