# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ConversionGoalCategory(str, Enum):
    UNKNOWN = 'Unknown'

    NONE = 'None'

    PURCHASE = 'Purchase'

    ADDTOCART = 'AddToCart'

    BEGINCHECKOUT = 'BeginCheckout'

    SUBSCRIBE = 'Subscribe'

    SUBMITLEADFORM = 'SubmitLeadForm'

    BOOKAPPOINTMENT = 'BookAppointment'

    SIGNUP = 'Signup'

    REQUESTQUOTE = 'RequestQuote'

    GETDIRECTIONS = 'GetDirections'

    OUTBOUNDCLICK = 'OutboundClick'

    CONTACT = 'Contact'

    PAGEVIEW = 'PageView'

    DOWNLOAD = 'Download'

    OTHER = 'Other'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ConversionGoalCategory from a JSON string"""
        return cls(json.loads(json_str))


