# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class LCID(str, Enum):
    ARABICSAUDIARABIA = 'ArabicSaudiArabia'

    BULGARIAN = 'Bulgarian'

    CHINESETAIWAN = 'ChineseTaiwan'

    CZECHREPUBLICCZ = 'CzechRepublicCZ'

    DANISHDENMARK = 'DanishDenmark'

    GERMANGERMANY = 'GermanGermany'

    GREEKGREECE = 'GreekGreece'

    ENGLISHUS = 'EnglishUS'

    SPANISHSPAIN = 'SpanishSpain'

    FINNISHFINLAND = 'FinnishFinland'

    FRENCHFRANCE = 'FrenchFrance'

    HEBREWISRAEL = 'HebrewIsrael'

    HUNGARYHU = 'HungaryHU'

    ITALIANITALY = 'ItalianItaly'

    JAPANESEJAPAN = 'JapaneseJapan'

    KOREANKOREA = 'KoreanKorea'

    DUTCHNETHERLANDS = 'DutchNetherlands'

    NORWEGIANNORWAY = 'NorwegianNorway'

    POLANDPOLISH = 'PolandPolish'

    PORTUGUESEBRAZIL = 'PortugueseBrazil'

    ROMANIARO = 'RomaniaRO'

    RUSSIANRUSSIA = 'RussianRussia'

    CROATIAN = 'Croatian'

    SWEDISHSWEDEN = 'SwedishSweden'

    ENGLISHTHAILAND = 'EnglishThailand'

    TURKISHTURKEY = 'TurkishTurkey'

    ENGLISHINDONESIA = 'EnglishIndonesia'

    UKRAINIANUKRAINE = 'UkrainianUkraine'

    SLOVENIAN = 'Slovenian'

    LATVIAN = 'Latvian'

    LITHUANIAN = 'Lithuanian'

    ENGLISHVIETNAM = 'EnglishVietnam'

    HINDIINDIA = 'HindiIndia'

    MALAYMALAYSIA = 'MalayMalaysia'

    FILIPINOPHILIPPINES = 'FilipinoPhilippines'

    ARABICIRAQ = 'ArabicIraq'

    CHINESECHINA = 'ChineseChina'

    GERMANSWITZERLAND = 'GermanSwitzerland'

    ENGLISHUK = 'EnglishUK'

    SPANISHMEXICO = 'SpanishMexico'

    ARABICEGYPT = 'ArabicEgypt'

    CHINESEHONGKONG = 'ChineseHongKong'

    GERMANAUSTRIA = 'GermanAustria'

    ENGLISHAUSTRALIA = 'EnglishAustralia'

    SPANISHSPAINMODERNSORT = 'SpanishSpainModernSort'

    FRENCHCANADA = 'FrenchCanada'

    ARABICLIBYA = 'ArabicLibya'

    ENGLISHCANADA = 'EnglishCanada'

    ARABICALGERIA = 'ArabicAlgeria'

    ENGLISHNEWZEALAND = 'EnglishNewZealand'

    ARABICMOROCCO = 'ArabicMorocco'

    ENGLISHIRELAND = 'EnglishIreland'

    ARABICTUNISIA = 'ArabicTunisia'

    ARABICOMAN = 'ArabicOman'

    SPANISHVENEZUELA = 'SpanishVenezuela'

    ARABICYEMEN = 'ArabicYemen'

    SPANISHCOLOMBIA = 'SpanishColombia'

    SPANISHPERU = 'SpanishPeru'

    ARABICJORDAN = 'ArabicJordan'

    SPANISHARGENTINA = 'SpanishArgentina'

    ARABICLEBANON = 'ArabicLebanon'

    ARABICKUWAIT = 'ArabicKuwait'

    ENGLISHPHILIPPINES = 'EnglishPhilippines'

    SPANISHCHILE = 'SpanishChile'

    ARABICUNITEDARABEMIRATES = 'ArabicUnitedArabEmirates'

    ARABICBAHRAIN = 'ArabicBahrain'

    ARABICQATAR = 'ArabicQatar'

    ENGLISHINDIA = 'EnglishIndia'

    ENGLISHMALAYSIA = 'EnglishMalaysia'

    ENGLISHSINGAPORE = 'EnglishSingapore'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of LCID from a JSON string"""
        return cls(json.loads(json_str))


