# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AdDynamicTextPerformanceReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTID = 'AccountId'

    ACCOUNTNUMBER = 'AccountNumber'

    TIMEPERIOD = 'TimePeriod'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    KEYWORD = 'Keyword'

    ADID = 'AdId'

    ADTITLE = 'AdTitle'

    ADTYPE = 'AdType'

    DESTINATIONURL = 'DestinationUrl'

    PARAM1 = 'Param1'

    PARAM2 = 'Param2'

    PARAM3 = 'Param3'

    CURRENCYCODE = 'CurrencyCode'

    ADDISTRIBUTION = 'AdDistribution'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    AVERAGECPC = 'AverageCpc'

    SPEND = 'Spend'

    AVERAGEPOSITION = 'AveragePosition'

    CONVERSIONS = 'Conversions'

    CONVERSIONRATE = 'ConversionRate'

    COSTPERCONVERSION = 'CostPerConversion'

    DEVICETYPE = 'DeviceType'

    LANGUAGE = 'Language'

    ACCOUNTSTATUS = 'AccountStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

    ADSTATUS = 'AdStatus'

    KEYWORDSTATUS = 'KeywordStatus'

    TITLEPART1 = 'TitlePart1'

    TITLEPART2 = 'TitlePart2'

    TITLEPART3 = 'TitlePart3'

    PATH1 = 'Path1'

    PATH2 = 'Path2'

    FINALURL = 'FinalUrl'

    FINALMOBILEURL = 'FinalMobileUrl'

    FINALAPPURL = 'FinalAppUrl'

    ADDESCRIPTION = 'AdDescription'

    ADDESCRIPTION2 = 'AdDescription2'

    ADLABELS = 'AdLabels'

    ALLCONVERSIONS = 'AllConversions'

    ALLCONVERSIONRATE = 'AllConversionRate'

    ALLCOSTPERCONVERSION = 'AllCostPerConversion'

    VIEWTHROUGHCONVERSIONS = 'ViewThroughConversions'

    GOAL = 'Goal'

    GOALTYPE = 'GoalType'

    ABSOLUTETOPIMPRESSIONRATEPERCENT = 'AbsoluteTopImpressionRatePercent'

    TOPIMPRESSIONRATEPERCENT = 'TopImpressionRatePercent'

    AVERAGECPM = 'AverageCpm'

    CONVERSIONSQUALIFIED = 'ConversionsQualified'

    ALLCONVERSIONSQUALIFIED = 'AllConversionsQualified'

    VIEWTHROUGHCONVERSIONSQUALIFIED = 'ViewThroughConversionsQualified'

    VIEWTHROUGHREVENUE = 'ViewThroughRevenue'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AdDynamicTextPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))


