# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class CallDetailReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    CAMPAIGNNAME = 'CampaignName'

    ADGROUPNAME = 'AdGroupName'

    STARTTIME = 'StartTime'

    ENDTIME = 'EndTime'

    DURATION = 'Duration'

    AREACODE = 'AreaCode'

    CITY = 'City'

    STATE = 'State'

    ACCOUNTID = 'AccountId'

    CAMPAIGNID = 'CampaignId'

    ADGROUPID = 'AdGroupId'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CallDetailReportColumn from a JSON string"""
        return cls(json.loads(json_str))


