# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ProductNegativeKeywordConflictReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    ADGROUPSTATUS = 'AdGroupStatus'

    MERCHANTPRODUCTID = 'MerchantProductId'

    TITLE = 'Title'

    ADGROUPCRITERIONID = 'AdGroupCriterionId'

    PRODUCTGROUP = 'ProductGroup'

    NEGATIVEKEYWORDID = 'NegativeKeywordId'

    NEGATIVEKEYWORD = 'NegativeKeyword'

    NEGATIVEKEYWORDLISTID = 'NegativeKeywordListId'

    CONFLICTLEVEL = 'ConflictLevel'

    NEGATIVEKEYWORDMATCHTYPE = 'NegativeKeywordMatchType'

    CAMPAIGNTYPE = 'CampaignType'

    ASSETGROUPID = 'AssetGroupId'

    ASSETGROUPNAME = 'AssetGroupName'

    ASSETGROUPSTATUS = 'AssetGroupStatus'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ProductNegativeKeywordConflictReportColumn from a JSON string"""
        return cls(json.loads(json_str))


